#ifndef _KFbxImporter_h
#define _KFbxImporter_h

/**************************************************************************************

 Copyright  2001 - 2005 Systmes Alias Qubec Inc. and/or its licensors.  
 All rights reserved.

 The coded instructions, statements, computer programs, and/or related material 
 (collectively the "Data") in these files contain unpublished information proprietary
 to Systmes Alias Qubec Inc. and/or its licensors, which is protected by Canadian
 and US federal copyright law and by international treaties.

 The Data may not be disclosed or distributed to third parties, in whole or in part,
 without the prior written consent of Alias Systems Corp. (Alias).

 THE DATA IS PROVIDED AS IS AND WITHOUT WARRANTY. ALL WARRANTIES ARE EXPRESSLY
 EXCLUDED AND DISCLAIMED. ALIAS MAKES NO WARRANTY OF ANY KIND WITH RESPECT TO THE DATA,
 EXPRESS, IMPLIED OR ARISING BY CUSTOM OR TRADE USAGE, AND DISCLAIMS ANY IMPLIED
 WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 PURPOSE. IN NO EVENT SHALL ALIAS, ITS AFFILIATES, PARENT COMPANIES, LICENSORS OR 
 SUPPLIERS (ALIAS GROUP) BE LIABLE FOR ANY DAMAGES OR EXPENSES OF ANY KIND INCLUDING
 WITHOUT LIMITATION PUNITIVE OR MULTIPLE DAMAGES OR OTHER SPECIAL, DIRECT, INDIRECT, 
 EXEMPLARY, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY KIND, WHETHER OR NOT ALIAS HAS
 BEEN ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWSOEVER CAUSED AND ON ANY THEORY OF
 LIABILITY. 

**************************************************************************************/


#include <kaydaradef.h>
#ifndef KFBX_DLL 
	#define KFBX_DLL K_DLLIMPORT
#endif

#include <kfbxio/kfbxio.h>
			
#include <klib/kstring.h>
#include <klib/karrayul.h>

#ifdef KARCH_DEV_MACOSX_CFM
	#include <CFURL.h>
	#include <Files.h>
#endif

class IFbxObject;
class KStreamOptions;

#include <kfbxplugins/kfbxnamespacedef.h>
namespace FBXSDK_NAMESPACE {

class KFbxScene;
class KFbxImporterImp;
class KStreamOptionsFbx;
class KFbxTakeInfo;
class KFbxSceneInfo;


/** Class to import an FBX file into SDK objects.
  * \nosubgrouping
  *	Typical way to use the import feature:
  *		- create an importer
  *		- initialize it with a file name
  *	    - set numerous states, take information, defining how the importer will behave
  *		- call KFbxImporter::Import() with an empty scene
  */
class KFBX_DLL KFbxImporter : public KFbxIO
{

public:

	/** 
	  * \name Import Functions
	  */
	//@{

	/** Initialize object.
	  *	\param pFileName Name of file to access.
	  *	\return \c true on success, \c false otherwise.
	  * To identify the error that occurred, call KFbxIo::GetLastErrorID().	  
	  */
	virtual bool Initialize(const char *pFileName);

#ifdef KARCH_DEV_MACOSX_CFM
    virtual bool Initialize(const FSSpec &pMacFileSpec);
    virtual bool Initialize(const FSRef &pMacFileRef);
    virtual bool Initialize(const CFURLRef &pMacURL);
#endif

	/** Get the FBX version number of the FBX file.
	  * FBX version numbers start at 5.0.0.
	  * \param pMajor Version major number.
	  * \param pMinor Version minor number.
	  * \param pRevision Version revision number.
	  *	\remarks This function must be called after KFbxImporter::Initialize().	  
	  */
	void GetFileVersion(int& pMajor, int& pMinor, int& pRevision);

	/** 
	  *	Get the default rendering resolution if present in the file header.
	  * \param pCamName Returned name of the camera.
	  * \param pResolutionMode Returned resolution mode.
	  * \param pW Returned Width.
	  * \param pH Returned Height.
	  * \return \c true if the default rendering settings are defined in the file. Otherwise
	  * return \c false and don't fill the parameters.
	  */
	bool GetDefaultRenderResolution(KString& pCamName, KString& pResolutionMode, double& pW, double& pH);

	/** \enum EStreamOptionsGeneration Stream options identifiers.
	  * - \e eSTREAMOPTIONS_PARSE_FILE
	  * - \e eSTREAMOPTIONS_FAST		Do not parse the file.
	  */
	typedef enum 
	{
		eSTREAMOPTIONS_PARSE_FILE,
		eSTREAMOPTIONS_FAST,        // Do not parse the file
		eSTREAMOPTIONS_COUNT
	} EStreamOptionsGeneration;

    /** Read the currently opened file header to fetch information related to takes.
	  * \param pStreamOptionsGeneration Stream options identifier.
	  *	\return Pointer to file import options or \c NULL on failure.
	  * \remarks Caller gets ownership of the returned structure.
      */
	KStreamOptions* GetImportOptions(EStreamOptionsGeneration pStreamOptionsGeneration = eSTREAMOPTIONS_PARSE_FILE);	

    /** Read the currently opened file header to fetch information related to takes.
	  * \param pFbxObject Target FBX file.
	  *	\return Pointer to stream import options or \c NULL on failure.
	  * \remarks Caller gets ownership of the returned structure.
      */
	KStreamOptions* GetImportOptions(IFbxObject* pFbxObject);

    /** Import the currently opened file into a scene. 
      * \param pScene Scene to fill with file content.
	  * \param pStreamOptions Pointer to file import options.
	  *	\return \c true on success, \c false otherwise.
	  * To identify the error that occurred, call KFbxIo::GetLastErrorID().
	  * If the imported file is password protected and the password is not
	  * set or wrong, function KFbxIo::GetLastErrorID() returns 
	  * KFbxIo::ePASSWORD_ERROR.
      */
    bool Import(KFbxScene& pScene, KStreamOptions* pStreamOptions = NULL);	

    /** Import the FBX file into a scene. 
      * \param pScene Scene to fill with file content.
	  * \param pStreamOptions Pointer to file import options.
	  * \param pFbxObject Source FBX file.
	  *	\return \c true on success, \c false otherwise.
	  * To identify the error that occurred, call KFbxIo::GetLastErrorID().
	  * If the imported file is password protected and the password is not
	  * set or wrong, function KFbxIo::GetLastErrorID() returns 
	  * KFbxIo::ePASSWORD_ERROR.
      */
	bool Import(KFbxScene& pScene, KStreamOptions* pStreamOptions, IFbxObject* pFbxObject);

    /** Release the file import options. 
	  * \param pStreamOptions Pointer to file import options.
	  */
	void ReleaseImportOptions(KStreamOptions* pStreamOptions);

	//@}

	/** Set the password.
	  * All subsequently imported files are opened with the given password.
      * \param pPassword Password string.
      */
    void SetPassword(char* pPassword);

	/** 
	  * \name Take Description Access
	  */
	//@{
	
	/** Get the number of available takes in the file.
	  * \return Number of takes.
	  *	\remarks This function must be called after KFbxImporter::Initialize().
	  */
	int GetTakeCount();

	/** Get the take information about an available take.
	  * Use the returned reference to a KFbxTakeInfo object to set
	  * whether the indexed take is imported.
	  *	\param pIndex	Index of the requested take.
	  *	\return Take information or \c NULL if function failed.
	  *	\remarks This function must be called after KFbxImporter::Initialize().
	  */
	KFbxTakeInfo* GetTakeInfo(int pIndex);

	/** Return the current take name.
	  *	\return Current take name if there is one, otherwise returns an empty string.
	  *	\remarks This function must be called after KFbxImporter::Initialize().
	  */
	char* GetCurrentTakeName();

	//@}

	/** 
	  * \name Scene Description Access
	  */
	//@{

	/** Get the scene info.
	  * \return Pointer to the scene info or \c NULL if no scene information
	  * is available in the file.
	  */
	KFbxSceneInfo* GetSceneInfo();

	//@}


	/** 
	  * \name Import State Functions
	  * \remarks Only some parts of the scene can be imported by setting the import states.
	  */
	//@{

	/** \enum EExportState Import section identifiers.
	  - \e eIMPORT_MODEL
	  - \e eIMPPORT_MATERIAL
	  - \e eIMPORT_TEXTURE
	  - \e eIMPORT_LINK
	  - \e eIMPORT_SHAPE
	  - \e eIMPORT_GOBO
	  - \e eIMPORT_ANIMATION
	  - \e eIMPORT_CHARACTER
	  - \e eIMPORT_GLOBAL_SETTINGS
	  - \e eIMPORT_PIVOT
	  - \e eIMPORT_CONSTRAINTS
	  - \e eMERGE_LAYER_AND_TIMEWARP
	  */
	typedef enum 
	{
		eIMPORT_MODEL,
		eIMPORT_MATERIAL,
		eIMPORT_TEXTURE,
		eIMPORT_LINK,
		eIMPORT_SHAPE,
		eIMPORT_GOBO,
		eIMPORT_ANIMATION,
		eIMPORT_CHARACTER,
		eIMPORT_GLOBAL_SETTINGS,
		eIMPORT_PIVOT,
		eIMPORT_CONSTRAINTS,
		eMERGE_LAYER_AND_TIMEWARP,
		eIMPORT_STATE_COUNT
	} EImportState;

    /** Set the value of an import state.
	  * \param pState Import section identifier.
	  * \param pValue State of the flag. If set to \c true, the corresponding section
	  * is processed during import. Otherwise it is skipped.
	  */
    void SetState(EImportState pState, bool pValue);

    /** Get the value of an import state.
	  * \param pState Import section identifier.
	  * \return The current value of the Import section identifier.
	  */
    bool GetState(EImportState pState);

	/**	Reset all the import states to \c true, set the default take index to 0
	  * and disable the password.
	  */
    void ResetConfiguration();

#ifndef K_FBXSDK
	#include <kfbxio/kfbximporterinhouse.h>
#else
	typedef enum 
	{
		eFBX_BINARY,
		eFBX_ASCII,
		eFBX_ENCRYPTED,
		eAUTOCAD_DXF,		
		e3D_STUDIO_3DS,
		eALIAS_OBJ,
		eFILE_FORMAT_COUNT
	} EFileFormat;
#endif

	/** Set the imported file format.
	  *	\param pFileFormat	File format identifier.
	  */
	void SetFileFormat (EFileFormat pFileFormat);

	/** Get the format of the imported file.
	  *	\return File format identifier.
	  */
	EFileFormat GetFileFormat ();

	/** Return the file format count
	  *	\return count.
	  */
	static int GetFileFormatCount() {return eFILE_FORMAT_COUNT;}

	/** Return the number of importable file formats.
	  *	\return Number of supported importable file formats.
	  * \remark Multiple identifiers for the same format are counted as 
	  * a file format. For instance, eFBX_BINARY, eFBX_ASCII and eFBX_ENCRIPTED
	  * count for three file formats.
	  */
	static char* GetFileFormatDescription(EFileFormat pFileFormat);

	/** Get the file extension in a readable format.
	  *	\param pFileFormat	File format identifier.
	  *	\return Pointer to the character representation of the file extension.
	  */
	static char* GetFileFormatExtension(EFileFormat pFileFormat);
	
	//@}

///////////////////////////////////////////////////////////////////////////////
//
//  WARNING!
//
//	Anything beyond these lines may not be documented accurately and is 
// 	subject to change without notice.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef DOXYGEN_SHOULD_SKIP_THIS

		bool UpdateImportOptions(KStreamOptions* pStreamOptions);

protected:

	KFbxImporter(KFbxSdkManager& pManager);
	virtual ~KFbxImporter();

	void Reset();

    bool FileOpen();
	void FileClose();

	void SetImportOptionsInfo(KStreamOptionsFbx* pStreamOptionsFbx);
	void GetImportOptionsInfo(KStreamOptionsFbx* pStreamOptionsFbx);

    bool IsNativeExtension ();

	KFbxImporterImp* mImp;

	bool mImportOptionsDone;
	KString mPassword;
	KArrayTemplate<KFbxTakeInfo*> mTakeInfo;
	KFbxSceneInfo* mSceneInfo;
	KString mCurrentTakeName;
    bool mImportStates[eIMPORT_STATE_COUNT];
	int mMajorVersion;
	int mMinorVersion;
	int mRevisionVersion;

	// Used to retrieve the default render resolution
	// stored in the extended header
	bool    mDefaultCameraResolutionIsOK;
	KString mDefaultCameraName;
	KString mDefaultResolutionMode;
	double mDefaultResolutionW;
	double mDefaultResolutionH;


	#ifndef K_FBXSDK
		#include <kfbxio/kfbximporterinhouse.h>
	#endif

	friend class KFbxSdkManager;

#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

};

} // namespace
#endif // #ifndef _KFbxImporter_h


