#ifndef _KFbxColor_h
#define _KFbxColor_h

/**************************************************************************************

 Copyright  2000 - 2005 Systmes Alias Qubec Inc. and/or its licensors.  
 All rights reserved.

 The coded instructions, statements, computer programs, and/or related material 
 (collectively the "Data") in these files contain unpublished information proprietary
 to Systmes Alias Qubec Inc. and/or its licensors, which is protected by Canadian
 and US federal copyright law and by international treaties.

 The Data may not be disclosed or distributed to third parties, in whole or in part,
 without the prior written consent of Alias Systems Corp. (Alias).

 THE DATA IS PROVIDED AS IS AND WITHOUT WARRANTY. ALL WARRANTIES ARE EXPRESSLY
 EXCLUDED AND DISCLAIMED. ALIAS MAKES NO WARRANTY OF ANY KIND WITH RESPECT TO THE DATA,
 EXPRESS, IMPLIED OR ARISING BY CUSTOM OR TRADE USAGE, AND DISCLAIMS ANY IMPLIED
 WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 PURPOSE. IN NO EVENT SHALL ALIAS, ITS AFFILIATES, PARENT COMPANIES, LICENSORS OR 
 SUPPLIERS (ALIAS GROUP) BE LIABLE FOR ANY DAMAGES OR EXPENSES OF ANY KIND INCLUDING
 WITHOUT LIMITATION PUNITIVE OR MULTIPLE DAMAGES OR OTHER SPECIAL, DIRECT, INDIRECT, 
 EXEMPLARY, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY KIND, WHETHER OR NOT ALIAS HAS
 BEEN ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWSOEVER CAUSED AND ON ANY THEORY OF
 LIABILITY. 

**************************************************************************************/

#include <kaydaradef.h>
#ifndef KFBX_DLL 
	#define KFBX_DLL K_DLLIMPORT
#endif

#include <kaydara.h>
#include <kfbxplugins/kfbxnamespacedef.h>
namespace FBXSDK_NAMESPACE {

/** Simple class to hold RGBA values.
  * \nosubgrouping
  */
class KFBX_DLL KFbxColor
{
public:

    //! Constructor.
    KFbxColor();

	/** Constructor.
	  * \param pRed The Red component value.
	  * \param pGreen The Green component value.
	  * \param pBlue The Blue component value.
	  * \param pAlpha The alpha value of the color.
	  */
	KFbxColor(double pRed, double pGreen, double pBlue, double pAlpha = 1.0);

    //! Destructor.
    ~KFbxColor();

	/** Re-initialize the color object with their new values.
	  * \param pRed The Red component value.
	  * \param pGreen The Green component value.
	  * \param pBlue The Blue component value.
	  * \param pAlpha The alpha value of the color.
	  */
	void Set(double pRed, double pGreen, double pBlue, double pAlpha = 1.0);

	/** Indicate if all the members in the color objects are within their valid range.
	  * \return \c true if all the members are within their valid range.
	  */
	bool IsValid();

	/**
	  * \name Operators
	  */
	//@{

	//! Assignment operator.
    KFbxColor& operator=(const KFbxColor& pColor);

	//! Equality operator.
    bool operator==(const KFbxColor& pColor);

	//! Inequality operator.
    bool operator!=(const KFbxColor& pColor);
	//@}

	/**
	  * name Public Members
	  */
	//@{

	//! Valid range is from 0.0 to 1.0.
    double mRed;

	//! Valid range is from 0.0 to 1.0.
    double mGreen;

	//! Valid range is from 0.0 to 1.0.
    double mBlue;

	//! Valid range is from 0.0 to 1.0.
    double mAlpha;

	//@}
};

} // namespace
#endif // #ifndef _KFbxColor_h


