#ifndef _KFbxGenericNode_h
#define _KFbxGenericNode_h

/**************************************************************************************

 Copyright  2005 Systmes Alias Qubec Inc. and/or its licensors.  
 All rights reserved.

 The coded instructions, statements, computer programs, and/or related material 
 (collectively the "Data") in these files contain unpublished information proprietary
 to Systmes Alias Qubec Inc. and/or its licensors, which is protected by Canadian
 and US federal copyright law and by international treaties.

 The Data may not be disclosed or distributed to third parties, in whole or in part,
 without the prior written consent of Alias Systems Corp. (Alias).

 THE DATA IS PROVIDED AS IS AND WITHOUT WARRANTY. ALL WARRANTIES ARE EXPRESSLY
 EXCLUDED AND DISCLAIMED. ALIAS MAKES NO WARRANTY OF ANY KIND WITH RESPECT TO THE DATA,
 EXPRESS, IMPLIED OR ARISING BY CUSTOM OR TRADE USAGE, AND DISCLAIMS ANY IMPLIED
 WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 PURPOSE. IN NO EVENT SHALL ALIAS, ITS AFFILIATES, PARENT COMPANIES, LICENSORS OR 
 SUPPLIERS (ALIAS GROUP) BE LIABLE FOR ANY DAMAGES OR EXPENSES OF ANY KIND INCLUDING
 WITHOUT LIMITATION PUNITIVE OR MULTIPLE DAMAGES OR OTHER SPECIAL, DIRECT, INDIRECT, 
 EXEMPLARY, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY KIND, WHETHER OR NOT ALIAS HAS
 BEEN ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWSOEVER CAUSED AND ON ANY THEORY OF
 LIABILITY. 

**************************************************************************************/

#include <kaydaradef.h>
#ifndef KFBX_DLL 
	#define KFBX_DLL K_DLLIMPORT
#endif

#include <kaydara.h>

#include <kfbxplugins/kfbxtakenodecontainer.h>

#include <klib/kerror.h>

class KObject;

#include <kfbxplugins/kfbxnamespacedef.h>
namespace FBXSDK_NAMESPACE {

class KFbxSdkManager;
class KFbxGenericNodeProperties;

/** Empty node containing properties.
  *\ nosubgrouping
  */
class KFBX_DLL KFbxGenericNode : public KFbxTakeNodeContainer
{

public:

		/**
		  * \name Error Management
		  */
		//@{

		/** Retrieve error object.
		  *	\return Reference to error object.
		  */
		KError& GetError();

		/** \enum EError Error identifiers.
		  * - \e eERROR
		  * - \e eERROR_COUNT
		  */
		typedef enum 
		{
			eERROR,
			eERROR_COUNT
		} EError;	

		/** Get last error code.
		  *	\return Last error code.
		  */
		EError GetLastErrorID();

		/** Get last error string.
		  *	\return Textual description of the last error.
		  */
		char* GetLastErrorString();

		//@}

		/** Return the type ID of this class.
		  * \return Return KFbxObject::eGENERIC_NODE.
		  */
		virtual KFbxObject::ENameSpace GetNameSpace() const;

///////////////////////////////////////////////////////////////////////////////
//
//  WARNING!
//
//	Anything beyond these lines may not be documented accurately and is 
// 	subject to change without notice.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef DOXYGEN_SHOULD_SKIP_THIS
		virtual KObject*          GetPropertyHandler(); 

	protected:
		KFbxGenericNode(KFbxSdkManager& pManager, char* pName);
		virtual ~KFbxGenericNode();

		void        Destruct();

		KString		GetTypeName();
		KStringList	GetTypeFlags();

		// Entity manipulation
		bool SetEntity(KFbxEntity* pEntity);
		KFbxEntity* GetEntity();
		KFbxEntity* mEntity;
		void AddChannels(KFbxTakeNode *pTakeNode);
		
		
		virtual KFbxUserProperty* CreateKFbxUserProperty(KProperty* pProperty);

		KFbxGenericNodeProperties* mPH;
private:
		KError mError;
protected:

		friend class KFbxReaderFbx;
		friend class KFbxWriterFbx;
		friend class KFbxWriterFbx6;
		friend class KFbxSdkManager;
		friend class KFbxScene;

#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS
};

typedef KFbxGenericNode* HKFbxGenericNode;

} // namespace


#endif // _KFbxGenericNode_h
