#ifndef _KFbxGlobalTimeSettings_h
#define _KFbxGlobalTimeSettings_h

/**************************************************************************************

 Copyright  2001 - 2005 Systmes Alias Qubec Inc. and/or its licensors.  
 All rights reserved.

 The coded instructions, statements, computer programs, and/or related material 
 (collectively the "Data") in these files contain unpublished information proprietary
 to Systmes Alias Qubec Inc. and/or its licensors, which is protected by Canadian
 and US federal copyright law and by international treaties.

 The Data may not be disclosed or distributed to third parties, in whole or in part,
 without the prior written consent of Alias Systems Corp. (Alias).

 THE DATA IS PROVIDED AS IS AND WITHOUT WARRANTY. ALL WARRANTIES ARE EXPRESSLY
 EXCLUDED AND DISCLAIMED. ALIAS MAKES NO WARRANTY OF ANY KIND WITH RESPECT TO THE DATA,
 EXPRESS, IMPLIED OR ARISING BY CUSTOM OR TRADE USAGE, AND DISCLAIMS ANY IMPLIED
 WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 PURPOSE. IN NO EVENT SHALL ALIAS, ITS AFFILIATES, PARENT COMPANIES, LICENSORS OR 
 SUPPLIERS (ALIAS GROUP) BE LIABLE FOR ANY DAMAGES OR EXPENSES OF ANY KIND INCLUDING
 WITHOUT LIMITATION PUNITIVE OR MULTIPLE DAMAGES OR OTHER SPECIAL, DIRECT, INDIRECT, 
 EXEMPLARY, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY KIND, WHETHER OR NOT ALIAS HAS
 BEEN ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWSOEVER CAUSED AND ON ANY THEORY OF
 LIABILITY. 

**************************************************************************************/

#include <kaydaradef.h>
#ifndef KFBX_DLL 
	#define KFBX_DLL K_DLLIMPORT
#endif

#include <kaydara.h>

#include <klib/kstring.h>
#include <klib/karrayul.h>
#include <klib/ktime.h>
#include <klib/kerror.h>

#include <kfbxplugins/kfbxnamespacedef.h>
namespace FBXSDK_NAMESPACE {

class KFbxGlobalTimeSettingsProperties;

/** This class contains the global time settings.
  * \nosubgrouping
  */
class KFBX_DLL KFbxGlobalTimeSettings
{

public:

	//! Restore default settings.
	void RestoreDefaultSettings();

	/** Set time mode.
	  * \param pTimeMode One of the defined modes in class KTime.
	  */
	void SetTimeMode(KTime::ETimeMode pTimeMode);

	/** Get time mode.
	  * \return The currently set TimeMode.
	  */
	KTime::ETimeMode GetTimeMode();
	
	/** Set time protocol.
	  * \param pTimeProtocol One of the defined protocols in class KTime.
	  */
	void SetTimeProtocol(KTime::ETimeProtocol pTimeProtocol);

	/** Get time protocol.
	  * \return The currently set TimeProtocol.
	  */
	KTime::ETimeProtocol GetTimeProtocol();
	
	//! Snap on frame mode.
	typedef enum 
	{
		eNO_SNAP, 
		eSNAP_ON_FRAME, 
		ePLAY_ON_FRAME 
	} ESnapOnFrameMode;

	/** Set snap on frame mode.
	  * \param pSnapOnFrameMode One of the following values: \c eNO_SNAP, \c eSNAP_ON_FRAME or \c ePLAY_ON_FRAME.
	  */
	void SetSnapOnFrameMode(ESnapOnFrameMode pSnapOnFrameMode);

	/** Get snap on frame mode.
	  * \return The currently set FrameMode.
	  */
	ESnapOnFrameMode GetSnapOnFrameMode();

	/**
	  * \name Time Markers
	  */
	//@{

	struct KFbxTimeMarker
	{
		KFbxTimeMarker();
		KFbxTimeMarker(const KFbxTimeMarker& pTimeMarker);
		KFbxTimeMarker& operator=(const KFbxTimeMarker& pTimeMarker);

		KString mName; //! Marker name.
		KTime mTime; //! Marker time. 
		bool mLoop; //! Loop flag.
	};

	/** Get number of time markers.
	  * \return The number of time markers.
	  */
	int GetTimeMarkerCount();

	/** Set current time marker index.
	  * \param pIndex Current time marker index.
	  * \return If the index is valid, returns \c true otherwise returns \c false.
	  *	\remarks In case the return value is \false , KFbxGlobalTimeSettings::GetLastErrorID() returns eINDEX_OUT_OF_RANGE.
	  */
	bool SetCurrentTimeMarker(int pIndex);

	/** Get current time marker index.
	  * \return Current time marker index or -1 if it hasn't been set.
	  */
	int GetCurrentTimeMarker();

	/** Get time marker at given index.
	  * \param pIndex Time marker index.
	  * \return Pointer to time marker at given index or \c NULL if the index is out of range.
	  *	\remarks In case the returned value is \c NULL , KFbxGlobalTimeSettings::GetLastErrorID() returns eINDEX_OUT_OF_RANGE.
	  */
	KFbxTimeMarker* GetTimeMarker(int pIndex);

	/** Add a time marker.
	  * \param pTimeMarker New time marker.
	  */
	void AddTimeMarker(KFbxTimeMarker pTimeMarker);

	//! Remove all time markes and set current time marker to -1.
	void RemoveAllTimeMarkers();

	//@}

	//! Assignment operator.
	const KFbxGlobalTimeSettings& operator=(const KFbxGlobalTimeSettings& pGlobalTimeSettings); 

	/**
	  * \name Error Management
	  */
	//@{

	/** Retrieve error object.
	 *	\return Reference to error object.
	 */
	KError& GetError();
	
	/** Error identifiers.
	 *	\remarks Most of these are only used internally.
	 */
	typedef enum 
	{
		eINDEX_OUT_OF_RANGE,
		eERROR_COUNT
	} EError;
	
	/** Get last error code.
	 *	\return Last error code.
	 */
	EError GetLastErrorID();

	/** Get last error string.
	 *	\return Textual description of the last error.
	 */
	char* GetLastErrorString();

	//@}

	/**
	  * \name Obsolete Functions
	  * \remarks These functions still work but are not relevant anymore.
	  */
	//@{

	/** Set snap on frame flag.
	  * This function is replaced by KFbxGlobalTimeSettings::SetSnapOnFrameMode().
	  * \param pSnapOnFrame If \c true, snap on frame mode is set to 
	  * \c eSNAP_ON_FRAME. If \c false, snap on frame mode is set to \c eNO_SNAP.
	  */
	void SetSnapOnFrame(bool pSnapOnFrame);

	/** This function is replaced by KFbxGlobalTimeSettings::GetSnapOnFrameMode().
	  * \return \c true if the snap on frame mode is set to \c eSNAP_ON_FRAME 
	  * or \c ePLAY_ON_FRAME. Otherwise, if snap on frame mode is set to \c eNO_SNAP, 
      * \c false .
	  */
	bool GetSnapOnFrame();

	//@}

///////////////////////////////////////////////////////////////////////////////
//
//  WARNING!
//
//	Anything beyond these lines may not be documented accurately and is 
// 	subject to change without notice.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef DOXYGEN_SHOULD_SKIP_THIS

private:

	KFbxGlobalTimeSettings();
	~KFbxGlobalTimeSettings();

	KFbxGlobalTimeSettingsProperties* mPH;

	friend class KFbxScene;

#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS 

};

} // namespace
#endif // #ifndef _KFbxGlobalTimeSettings_h


