#ifndef _KFbxLight_h
#define _KFbxLight_h

/**************************************************************************************

 Copyright  2001 - 2005 Systmes Alias Qubec Inc. and/or its licensors.  
 All rights reserved.

 The coded instructions, statements, computer programs, and/or related material 
 (collectively the "Data") in these files contain unpublished information proprietary
 to Systmes Alias Qubec Inc. and/or its licensors, which is protected by Canadian
 and US federal copyright law and by international treaties.

 The Data may not be disclosed or distributed to third parties, in whole or in part,
 without the prior written consent of Alias Systems Corp. (Alias).

 THE DATA IS PROVIDED AS IS AND WITHOUT WARRANTY. ALL WARRANTIES ARE EXPRESSLY
 EXCLUDED AND DISCLAIMED. ALIAS MAKES NO WARRANTY OF ANY KIND WITH RESPECT TO THE DATA,
 EXPRESS, IMPLIED OR ARISING BY CUSTOM OR TRADE USAGE, AND DISCLAIMS ANY IMPLIED
 WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 PURPOSE. IN NO EVENT SHALL ALIAS, ITS AFFILIATES, PARENT COMPANIES, LICENSORS OR 
 SUPPLIERS (ALIAS GROUP) BE LIABLE FOR ANY DAMAGES OR EXPENSES OF ANY KIND INCLUDING
 WITHOUT LIMITATION PUNITIVE OR MULTIPLE DAMAGES OR OTHER SPECIAL, DIRECT, INDIRECT, 
 EXEMPLARY, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY KIND, WHETHER OR NOT ALIAS HAS
 BEEN ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWSOEVER CAUSED AND ON ANY THEORY OF
 LIABILITY. 

**************************************************************************************/

#include <kaydaradef.h>
#ifndef KFBX_DLL 
	#define KFBX_DLL K_DLLIMPORT
#endif

#include <kaydara.h>

#include <kfbxplugins/kfbxnodeattribute.h>
#include <kfbxplugins/kfbxnamespacedef.h>
namespace FBXSDK_NAMESPACE {


class KFbxColor;
class KFbxSdkManager;
class KFbxLightProperties;

//!	This node attribute contains the properties of a light.
class KFBX_DLL KFbxLight : public KFbxNodeAttribute
{

public:

	//! Return the type of node attribute which is EAttributeType::eLIGHT.
	virtual EAttributeType GetAttributeType();

	/**
	  * \name Light Properties
	  */
	//@{

    /** \enum ELightType Light types.
	  * - \e ePOINT
	  * - \e eDIRECTIONAL
	  * - \e eSPOT
	  */
    typedef enum  
    {
	    ePOINT, 
	    eDIRECTIONAL, 
	    eSPOT
    } ELightType;

    /** Set the light type. 
	  * \param pLightType The light type.
	  */
    void SetLightType(ELightType pLightType);

	/** Get the light type.
	  * \return The current type of this light.
	  */
	ELightType GetLightType();

	/** Activate or de-activate the light.
	  * \param pCastLight If \c true the light is active.
	  */
	void SetCastLight(bool pCastLight);

	/** Get light state.
	  * \return \c true if light is active.
	  */
	bool GetCastLight();

	//@}

	/**
	  * \name Gobo properties
	  */
	//@{
    /** Set the associated gobo file. 
      * \param pFileName The path of the gobo file.   
      * \return Return \c false if the pointer is null.
	  *	\remarks The gobo file name must be valid.
	  *	\remarks The associated gobo file must be an 8 bits grayscale TIFF image file.
	  * The height and width dimensions must equal a power of two and don't exceed 
	  * 256 pixels.
      */
    bool SetFileName(char* pFileName);
    
#ifdef KARCH_DEV_MACOSX_CFM
    bool SetFile(const FSSpec &pMacFileSpec);
    bool SetFile(const FSRef &pMacFileRef);
    bool SetFile(const CFURLRef &pMacURL);
#endif

    /** Get the associated gobo file path.
	  * \return The associated gobo file path.
	  * \return An empty string if KFbxGobo::SetFileName() has not been called before.
	  */
    char* GetFileName();

#ifdef KARCH_DEV_MACOSX_CFM
	bool GetFile(FSSpec &pMacFileSpec) const;
    bool GetFile(FSRef &pMacFileRef) const;
    bool GetFile(CFURLRef &pMacURL) const;
#endif

    /** Enable gobo ground projection.
	  * \param pEnable The ground projection enable flag state.
      */
    void SetGroundProjection (bool pEnable);

    /** Get gobo ground projection flag.
	  * \return \c true if the gobo ground projection is enabled.
	  */
    bool GetGroundProjection();

    /** Enable gobo volumetric light projection.
	  * \param pEnable The volumetric light projection enable flag state.
	  */
    void SetVolumetricProjection(bool pEnable);

    /** Get gobo volumetric light projection flag.
	  * \return \c true if gobo volumetric light projection is enabled.
	  */
    bool GetVolumetricProjection();

    /** Enable gobo front volumetric projection.
	  * \param pEnable The front volumetric projection enable state.
	  * \remarks This option is not supported in FiLMBOX.
	  */
    void SetFrontVolumetricProjection(bool pEnable);

    /** Get gobo front volumetric light projection flag.
	  * \return true if gobo front volumetric light projection is enabled.
	  * \remarks This option is not supported in MotionBuilder.
	  */
    bool GetFrontVolumetricProjection();

	//@}

	/**
	  * \name Default Animation Values
	  * \remarks This set of functions provide direct access to default
	  * animation values specific to a light. The default animation 
	  * values are found in the default take node of the associated node.
	  * Hence, these functions only work if the light has been associated
	  * with a node.
	  */
	//@{

	/** Set default color.
	  * \param pColor The color of the light. 
	  * This property's default value is white.
	  */
	void SetDefaultColor(KFbxColor& pColor);

	/** Get default color.
	  * \param pColor The color of the light.
	  * \return Input parameter filled with appropriate data.
	  * \remark This property's default value is white.
	  */
	KFbxColor& GetDefaultColor(KFbxColor& pColor);

	/** Set default intensity.
	  * \param pIntensity The intensity value of the light.
	  * This parameter range is from 0 to 200, 200 meaning full intensity. 
	  * The property's default value is 100.
	  */
	void SetDefaultIntensity(double pIntensity);

	/** Get default intensity.
	  * \return The intensity value of the light.
	  * This parameter range is from 0 to 200, 200 meaning full intensity. 
	  * The property's default value is 100.
	  */
	double GetDefaultIntensity();

	/** Set default cone angle in degrees.
	  * \param pConeAngle The cone angle value of the light.
	  * This parameter has range is from 0 to 160 degrees.
	  * This parameter has no effect if the light type is not set to \c eSPOT.
	  * The property's default value is 45 degrees.
	  */
	void SetDefaultConeAngle(double pConeAngle);

	/** Get default cone angle in degrees.
	  * \return The cone angle value of the light.
	  * This parameter range is from 0 to 160 degrees.
	  * This parameter has no effect if the light type is not set to \c eSPOT.
	  * The property's default value is 45 degrees.
	  */
	double GetDefaultConeAngle();

	/** Set default fog.
	  * \param pFog The fog value of the light.
	  * This parameter range is from 0 to 200, 200 meaning full fog opacity.
	  * This parameter has no effect if the light type is not set to \c eSPOT.
	  * The property's default value is 50.
	  */
	void SetDefaultFog(double pFog);

	/** Get default fog.
	  * \return The fog value of the light.
	  * This parameter range is from 0 to 200, 200 meaning full fog opacity.
	  * This parameter has no effect if the light type is not set to \c eSPOT.
	  * The property's default value is 50.
	  */
	double GetDefaultFog();

	//@}

	//! Assignment operator.
    KFbxLight& operator= (KFbxLight& pLight);

///////////////////////////////////////////////////////////////////////////////
//
//  WARNING!
//
//	Anything beyond these lines may not be documented accurately and is 
// 	subject to change without notice.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef DOXYGEN_SHOULD_SKIP_THIS

protected:

	KFbxLight(KFbxSdkManager& pManager);
	virtual ~KFbxLight();

	/** Add channels specific to a light attribute.
      * \param pTakeNode Take node to add specialized channels to.
	  */
	virtual void AddChannels(KFbxTakeNode *pTakeNode);

	/** Remove channels specific to a light attribute.
      * \param pTakeNode Take node to remove specialized channels from.
	  */
	virtual void RemoveChannels(KFbxTakeNode *pTakeNode);

	/**
	  *	Used to retrieve the KProperty list from an attribute.
	  */
	virtual void FillPropertyList(void* pPropertyList);
	KFbxLightProperties* mPH;

	KString		GetTypeName();
	KStringList	GetTypeFlags();

	KString mFileName;	

	friend class KFbxSdkManager;

#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

};

typedef KFbxLight* HKFbxLight;

} // namespace
#endif // #ifndef _KFbxLight_h



