#ifndef _KFbxMaterial_h
#define _KFbxMaterial_h

/**************************************************************************************

 Copyright  1999 - 2005 Systmes Alias Qubec Inc. and/or its licensors.  
 All rights reserved.

 The coded instructions, statements, computer programs, and/or related material 
 (collectively the "Data") in these files contain unpublished information proprietary
 to Systmes Alias Qubec Inc. and/or its licensors, which is protected by Canadian
 and US federal copyright law and by international treaties.

 The Data may not be disclosed or distributed to third parties, in whole or in part,
 without the prior written consent of Alias Systems Corp. (Alias).

 THE DATA IS PROVIDED AS IS AND WITHOUT WARRANTY. ALL WARRANTIES ARE EXPRESSLY
 EXCLUDED AND DISCLAIMED. ALIAS MAKES NO WARRANTY OF ANY KIND WITH RESPECT TO THE DATA,
 EXPRESS, IMPLIED OR ARISING BY CUSTOM OR TRADE USAGE, AND DISCLAIMS ANY IMPLIED
 WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 PURPOSE. IN NO EVENT SHALL ALIAS, ITS AFFILIATES, PARENT COMPANIES, LICENSORS OR 
 SUPPLIERS (ALIAS GROUP) BE LIABLE FOR ANY DAMAGES OR EXPENSES OF ANY KIND INCLUDING
 WITHOUT LIMITATION PUNITIVE OR MULTIPLE DAMAGES OR OTHER SPECIAL, DIRECT, INDIRECT, 
 EXEMPLARY, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY KIND, WHETHER OR NOT ALIAS HAS
 BEEN ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWSOEVER CAUSED AND ON ANY THEORY OF
 LIABILITY. 

**************************************************************************************/

#include <kaydaradef.h>
#ifndef KFBX_DLL 
	#define KFBX_DLL K_DLLIMPORT
#endif

#include <kaydara.h>

#include <kfbxplugins/kfbxtakenodecontainer.h>
#include <kfbxplugins/kfbxcolor.h>

#include <klib/kerror.h>

#include <kfbxplugins/kfbxnamespacedef.h>
namespace FBXSDK_NAMESPACE {

class KFbxMaterialProperties;

/** Material settings.
  * \nosubgrouping
  * A material is attached to an instance of class KFbxGeometry
  * by calling KFbxGeometry::AddMaterial(). Materials can be shared among 
  * many instances of class KFbxGeometry.
  */
class KFBX_DLL KFbxMaterial : public KFbxTakeNodeContainer
{

public:

	/**
	  * \name Material Properties
	  */
	//@{	

	//! Reset the material to default values.
	void Reset();
	
	/** Set material shading model.
	  * \param pShadingModel Shading model for the material.
	  * \remarks "phong" is the default shading model.
	  */
	void SetShadingModel(const char* pShadingModel);

	/** Get material shading model.
	  * \return The shading model type string.
	  */
	char* GetShadingModel();

	//@}	

	/**
	  * \name Default Animation Values
	  * \remarks This set of functions provides direct access to default
	  * animation values in the default take node. 
	  */
	//@{

	/** Set default material ambient color.
	  * \param pColor Material ambient color.
	  */
	void SetDefaultAmbientColor(KFbxColor& pColor);
	
	/** Get default material ambient color.
	  * \param pColor Material ambient color.
	  * \return Input parameter filled with appropriate data.
	  */
	KFbxColor& GetDefaultAmbientColor(KFbxColor& pColor);

	/** Set default material diffuse color.
	  * \param pColor Material diffuse color.
      */
	void SetDefaultDiffuseColor(KFbxColor& pColor);
	
	/** Get default material diffuse color.
	  * \param pColor Material diffuse color.
	  * \return Input parameter filled with appropriate data.
	  */
	KFbxColor& GetDefaultDiffuseColor(KFbxColor& pColor);

	/** Set default material specular color.
	  * \param pColor Material specular color.
	  */
	void SetDefaultSpecularColor(KFbxColor& pColor);
	
	/** Get default material specular color.
	  * \param pColor Material specular color.
	  * \return Input parameter filled with appropriate data.
	  */
	KFbxColor& GetDefaultSpecularColor(KFbxColor& pColor);

	/** Set default material emissive color.
	  * \param pColor Material emissive color.
	  */
	void SetDefaultEmissiveColor(KFbxColor& pColor);
	
	/** Get default material emissive color.
	  * \param pColor Material emissive color.
	  * \return Input parameter filled with appropriate data.
	  */
	KFbxColor& GetDefaultEmissiveColor(KFbxColor& pColor);

	/** Set default material opacity.
	  * \param pOpacity Opacity value from 0 to 1, 0 meaning 100% transparent.
	  */
	void SetDefaultOpacity(double pOpacity);

	/** Get default material opacity.
	  * \return The material opacity. The value is in the interval [0, 1], 
	    0 meaning 100% transparent.
	  */
	double	GetDefaultOpacity();
	
	/** Set default material shininess.
	  * \param pShininess Shininess value from 0 to 1, 0 meaning 100% matte.
	  */
	void SetDefaultShininess(double pShininess);

	/** Get default material shininess.
	  * \return The material shininess. The value is in the interval [0, 1],
	  0 meaning 100% matte.
	  */
	double	GetDefaultShininess();

	/** Set default material reflectivity.
	  * \param pReflectivity Reflectivity value from 0 to 1, 0 meaning no reflectivity.
	  */
	void SetDefaultReflectivity(double pReflectivity);

	/** Get default material reflectivity.
	  * \return The material reflectivity. The value is in the interval [0,1],
	  0 meaning no reflectivity.
	  */
	double	GetDefaultReflectivity();

	/**	Set multi layer state flag.
	  * \param pMultiLayer If \c true, multi-layering is enabled.
	  */
	void SetMultiLayer(bool pMultiLayer);

	/**	Get multilayer state.
	  * \return The state of the multy-layer flag.
	  */
	bool GetMultiLayer();

	//@}

	/**
	  * \name Obsolete Functions
	  * \remarks This set of functions is obsolete since animated parameters
	  * are now supported. Use functions found in section "Default Animation 
	  * Values" instead.
	  */
	//@{

	//! Set material ambient color.
	void SetAmbient(KFbxColor pColor);
	
	//! Get material ambient color.
	KFbxColor GetAmbient();

	//! Set material diffuse color.
	void SetDiffuse(KFbxColor pColor);
	
	//! Get material diffuse color.
	KFbxColor GetDiffuse();

	//! Set material specular color.
	void SetSpecular(KFbxColor pColor);
	
	//! Get material specular color.
	KFbxColor GetSpecular();

	//! Set material emissive color.
	void SetEmissive(KFbxColor pColor);
	
	//! Get material emissive color.
	KFbxColor GetEmissive();

	//! Set material opacity on a scale of 0 to 1, 0 meaning transparent.
	void SetOpacity(double pOpacity);

	//! Get material opacity on a scale of 0 to 1, 0 meaning transparent.
	double	GetOpacity();
	
	//! Set material shininess on a 0 to 1 scale, 0 meaning matte.
	void SetShininess(double pShininess);

	//! Get material shininess on a 0 to 1 scale, 0 meaning matte.
	double	GetShininess();

	//! Set material reflectivity on a 0 to 1 scale, 0 meaning no reflexivity.
	void SetReflectivity(double pReflectivity);

	//! Get material reflectivity on a 0 to 1 scale, 0 meaning no reflexivity.
	double	GetReflectivity();

	//@}

	//! Assignment operator.
    KFbxMaterial& operator=(KFbxMaterial& pMaterial);

	/**
	  * \name Error Management
	  */
	//@{

	/** Retrieve error object.
	  *	\return Reference to error object.
	  */
	KError& GetError();

	/** \enum EError Error identifiers.
	  * - \e eTAKE_NODE_ERROR
	  * - \e eERROR_COUNT
	  */
	typedef enum 
	{
		eTAKE_NODE_ERROR,
		eERROR_COUNT
	} EError;	

	/** Get last error code.
	  *	\return Last error code.
	  */
	EError GetLastErrorID();

	/** Get last error string.
	  *	\return Textual description of the last error.
	  */
	char* GetLastErrorString();

	//@}

///////////////////////////////////////////////////////////////////////////////
//
//  WARNING!
//
//	Anything beyond these lines may not be documented accurately and is 
// 	subject to change without notice.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef DOXYGEN_SHOULD_SKIP_THIS

	static int GetClassID();
	virtual KObject* GetPropertyHandler();

protected:

	KFbxMaterial(KFbxSdkManager& pManager, char* pName);
	virtual ~KFbxMaterial();

	// From KFbxPlug
	void Destruct();

	// From KFbxObject
	KFbxObject::ENameSpace GetNameSpace() const; 
	KString		GetTypeName();
	KStringList	GetTypeFlags();

	// From KFbxTakeNodeContainer
	void AddChannels(KFbxTakeNode *pTakeNode);

	virtual KFbxUserProperty* CreateKFbxUserProperty(KProperty* pProperty);

	// Properties Handler
	KFbxMaterialProperties* mPH;

	// Local
	bool operator==(KFbxMaterial& pMaterial);
	void Init();

	KString mShadingModel;
	KError  mError;
	bool    mMultiLayer;

	friend class KFbxSdkManager;
	friend class KFbxWriterFbx;
	friend class KFbxWriterFbx6;
	friend class KFbxReaderFbx;

#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS 

};

typedef KFbxMaterial* HKFbxMaterial;

} // namespace
#endif


