#ifndef _KFbxObject_h
#define _KFbxObject_h

/**************************************************************************************

 Copyright  2004 - 2005 Systmes Alias Qubec Inc. and/or its licensors.  
 All rights reserved.

 The coded instructions, statements, computer programs, and/or related material 
 (collectively the "Data") in these files contain unpublished information proprietary
 to Systmes Alias Qubec Inc. and/or its licensors, which is protected by Canadian
 and US federal copyright law and by international treaties.

 The Data may not be disclosed or distributed to third parties, in whole or in part,
 without the prior written consent of Alias Systems Corp. (Alias).

 THE DATA IS PROVIDED AS IS AND WITHOUT WARRANTY. ALL WARRANTIES ARE EXPRESSLY
 EXCLUDED AND DISCLAIMED. ALIAS MAKES NO WARRANTY OF ANY KIND WITH RESPECT TO THE DATA,
 EXPRESS, IMPLIED OR ARISING BY CUSTOM OR TRADE USAGE, AND DISCLAIMS ANY IMPLIED
 WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 PURPOSE. IN NO EVENT SHALL ALIAS, ITS AFFILIATES, PARENT COMPANIES, LICENSORS OR 
 SUPPLIERS (ALIAS GROUP) BE LIABLE FOR ANY DAMAGES OR EXPENSES OF ANY KIND INCLUDING
 WITHOUT LIMITATION PUNITIVE OR MULTIPLE DAMAGES OR OTHER SPECIAL, DIRECT, INDIRECT, 
 EXEMPLARY, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY KIND, WHETHER OR NOT ALIAS HAS
 BEEN ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWSOEVER CAUSED AND ON ANY THEORY OF
 LIABILITY. 

**************************************************************************************/

#include <kaydaradef.h>
#ifndef KFBX_DLL 
	#define KFBX_DLL K_DLLIMPORT
#endif

#include <kaydara.h>

#include <klib/kstring.h>
#include <klib/kstringlist.h>
#include <klib/kname.h>
#include <klib/karrayul.h>

#include <kfbxplugins/kfbxplug.h>
#include <kfbxplugins/kfbxuserproperty.h>

class KObject;
class KProperty;

#include <kfbxplugins/kfbxnamespacedef.h>
namespace FBXSDK_NAMESPACE {

class KFbxSdkManager;

/** Basic class for object type identification and instance naming.
  * \nosubgrouping
  */
class KFBX_DLL KFbxObject : public KFbxPlug
{

public:

	/**
	  * \name Object Name Management
	  */
	//@{

	/** Set the name of the object. 
	  * \param pName A \c NULL terminated string.
	  */
	void SetName(char* pName);

	/** Return the full name of the object.
	  * \return Return a \c NULL terminated string.
	  */
	char* GetName();

	/** Return the name of the object without the namespace qualifier.
	  * \return Return the name in a temporary string.
	  */
	KString GetNameWithoutNameSpacePrefix();

	/** Set the initial name of the object. 
	  * \param pName A \c NULL terminated string.
	  */
	void SetInitialName(char* pName);

	/** Return the initial name of the object.
	  * \return Return a \c NULL terminated string.
	  */
	char* GetInitialName();

	//@}

	/**
	  * \name Properties access
	  */
	//@{

	/** Return the number of property defining this object.
	  */
	int GetPropertyCount();

	/** Return the number of property with the specified flag set 
	  * defining this object.
	  * \param pHasFlag The flag set to base the search on. 
	  * \return Return the number of property having at least one of 
	  * the specified flag.
	  */
	int GetPropertyCount(KFbxUserProperty::EFlags pHasFlag);

	/** Add the specified property to the object.
	  * \param pProperty A pointer to the property to be added.
	  */
	void AddProperty(KFbxUserProperty* pProperty);

	/** Remove the specified property from the object.
	  * \param pProperty A pointer to the property to be removed.
	  */
	void RemoveProperty(KFbxUserProperty* pProperty);

	/** Get the property at the given index.
	  * \param pIndex The index of the requested property. This value
	  * must be between 0 and the number of properties returned by
	  * GetPropertyCount().
	  * \return Return the property at the given index, or \c NULL if \param pIndex
	  * is out of bound.
	  */
	KFbxUserProperty* GetProperty(int pIndex);

	/** Find a property using its name.
	  * \param pName The name of the property as a \c NULL terminated string.
	  * \return Return a pointer to the property found or \c NULL if the property was not found.
	  */
	KFbxUserProperty* FindPropertyByName(char* pName);

	//@}

	/**
	  * \name User-specific Data
	  */
	//@{

	/** Store a pointer to an application-specific memory region.
	  * \param pUserData A pointer to be carry over by the KFbxObject.
	  * \remarks The FBX SDK make no use of this pointer, it's the application
	  * responsability to allocate/free the memory referenced by this pointer.
	  */
	void SetUserData(void* pUserData);

	/** Retrieve the application-specific pointer stored in this KFbxObject.
	  * \return Return the pointer previously stored by SetUserData(), or \c NULL if the
	  * pointer was not set.
	  */
	void* GetUserData();

	//@}

	//! Assignment operator.
	KFbxObject& operator=(KFbxObject& pObject);
   
///////////////////////////////////////////////////////////////////////////////
//
//  WARNING!
//
//	Anything beyond these lines may not be documented accurately and is 
// 	subject to change without notice.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef DOXYGEN_SHOULD_SKIP_THIS
	

public:

	typedef enum 
	{
		eNODE             = 0x00000001,
		eTEXTURE          = 0x00000002,
		eMATERIAL         = 0x00000003,
		eGOBO	          = 0x00000004,
		eMEDIA	          = 0x00000005,
		eACTOR	          = 0x00000006,
		eMARKER_SET       = 0x00000007,
		eCHARACTER        = 0x00000008,
		eCHARACTER_POSE	  = 0x00000009,
		eGENERIC_NODE     = 0x0000000a,
		ePOSE             = 0x0000000b,
		eCONTROLSET_PLUG  = 0x0000000c,
		eCONSTRAINT		  = 0x0000000d
	} ENameSpace;

	virtual ENameSpace GetNameSpace() const = 0;
	KString GetNameSpacePrefix();

	static KString GetDefaultNameSpacePrefix(ENameSpace pNameSpace);
	static KString RemovePrefix(char* pName);

	KFbxUserProperty* GetProperty(KProperty* pKProperty);
	virtual KObject*  GetPropertyHandler();
	virtual void      FillPropertyList(void* pPropertyList);

protected:

    KFbxObject(KFbxSdkManager& pManager, char* pName);
    virtual ~KFbxObject();

	virtual KString		GetTypeName();
	virtual KStringList	GetTypeFlags();

	virtual void PropertyAdded(KFbxUserProperty* pProperty);
	virtual void PropertyRemoved(KFbxUserProperty* pProperty);	
	
    KName                             mName;
	KArrayTemplate<KFbxUserProperty*> mProperties;
	void*                             mUserData;
	KFbxSdkManager*                   mManager;

	friend class KFbxEntity;
	friend class KFbxReaderFbx;
	friend class KFbxWriterFbx6;

#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

};

typedef KFbxObject* HKFbxObject;

} // namespace
#endif // #ifndef _KFbxObject_h


