#ifndef _KString_h
#define _KString_h

/**************************************************************************************

 Copyright  1995 - 2005 Systmes Alias Qubec Inc. and/or its licensors.  
 All rights reserved.

 The coded instructions, statements, computer programs, and/or related material 
 (collectively the "Data") in these files contain unpublished information proprietary
 to Systmes Alias Qubec Inc. and/or its licensors, which is protected by Canadian
 and US federal copyright law and by international treaties.

 The Data may not be disclosed or distributed to third parties, in whole or in part,
 without the prior written consent of Alias Systems Corp. (Alias).

 THE DATA IS PROVIDED AS IS AND WITHOUT WARRANTY. ALL WARRANTIES ARE EXPRESSLY
 EXCLUDED AND DISCLAIMED. ALIAS MAKES NO WARRANTY OF ANY KIND WITH RESPECT TO THE DATA,
 EXPRESS, IMPLIED OR ARISING BY CUSTOM OR TRADE USAGE, AND DISCLAIMS ANY IMPLIED
 WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 PURPOSE. IN NO EVENT SHALL ALIAS, ITS AFFILIATES, PARENT COMPANIES, LICENSORS OR 
 SUPPLIERS (ALIAS GROUP) BE LIABLE FOR ANY DAMAGES OR EXPENSES OF ANY KIND INCLUDING
 WITHOUT LIMITATION PUNITIVE OR MULTIPLE DAMAGES OR OTHER SPECIAL, DIRECT, INDIRECT, 
 EXEMPLARY, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY KIND, WHETHER OR NOT ALIAS HAS
 BEEN ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWSOEVER CAUSED AND ON ANY THEORY OF
 LIABILITY. 

**************************************************************************************/

#include <kaydaradef.h>
#ifndef KBASELIB_DLL 
	#define KBASELIB_DLL K_DLLIMPORT
#endif

#include <kaydara.h>

#if defined(KARCH_ENV_WIN32)
// WideChar to UTF8,ANSI and UTF8 to WideChar
	#define UTF8_to_WIDECHAR( inUTF8,outWideChar ) \
		wchar_t outWideChar[1024]; \
		MultiByteToWideChar( CP_UTF8/*CP_ACP*/, 0, inUTF8, -1, outWideChar, 1024 );

	#define WIDECHAR_to_UTF8( inWideChar,outUTF8 ) \
		char outUTF8[1024]; \
		WideCharToMultiByte( CP_UTF8,0,inWideChar,-1,outUTF8,1024,NULL,NULL);

	#define WIDECHAR_to_ANSI( inWideChar,outANSI ) \
		char outANSI[1024]; \
		WideCharToMultiByte( CP_ACP,0,inWideChar,-1,outANSI,1024,NULL,NULL);
#endif

KBASELIB_DLL void UTF8_to_ANSI( const char* pSrc, char* pDest, int pDestSize, char pSubstitute = '.' );

#define KStringStaticSize 64

/**	Utility class to manipulate strings.
  * \nosubgrouping
  */
class KBASELIB_DLL KString
{
  public:
	/**
	  * \name Constructors and Destructor
	  */
	//@{
	//! Create an instance.
	static KString		*Create();
	//! Create an instance if not already allocated ( is null )
	static KString		*Create(KString *pString);
	//! Destroy an allocated version of the string
	void				Destroy();
	//! Destroy the allocated space if empty
	static KString* DestroyIfEmpty(KString *pString);
	//! Destroy the allocated space if empty
	static KString* StringOrEmpty(KString *pString);
	//@}

  public:

	/**
	  * \name Constructors and Destructor
	  */
	//@{

	//! Default constructor.
	KString();

	//! Copy constructor.
	KString(const KString& pStr);

	//! String constructor.
	KString(const char* pStr);

	//! Character constructor.
	KString(char pChar, kUInt pNbRepeat = 1);

	//! String constructor with maximum length.
	KString(const char* pCharPtr, kUInt pLength);

	//! Int constructor.
	KString(const int pValue);

	//! Float constructor.
	KString(const float pValue);

	//! Double constructor.
	KString(const double pValue);

	//! Destructor.
	~KString();

	//@}

	/**
	  * \name Instance Validation.
	  */
	//@{

	//! Return \c true if string is valid.
	bool IsOK() const;

	//! Invalidate string.
	KString &Invalidate();

	//! Get string length like "C" strlen().
	kUInt GetLen() const;

	//! Return \c true if string is of length 0.
	bool IsEmpty() const;

	//! Discard the content of the object.
	KString  &Empty();   

	//@}

	/**
	  * \name Buffer Access
	  */
	//@{

	//! Access by reference.
	char& operator[](kUInt pIndex);

	//! Access by copy.
	char operator[](kUInt pIndex) const;    

	//! Cast operator.
	operator const char*() const;

	//! Uniform non-UNICODE support.
	char* ToChars() const;

	//! Non-const buffer access.
	char* Buffer();     

	//! Non-const buffer access, same as Buffer().
	char* GetBuffer();

	//@}

	/**
	  * \name Assignement Operators
	  */
	//@{

	//! KString assignment operator.
	const KString& operator=(const KString& pStr);

	//! Character assignment operator.
	const KString& operator=(char pChar);

	//! String assignment operator.
	const KString& operator=(const char* pStr);

	//! Int assignment operator.
	const KString& operator=(int  pValue);

	//! Float assignment operator.
	const KString& operator=(float pValue);

	//! Double assignment operator.
	const KString& operator=(double pValue);

	//! String assignment function.
	const KString& Copy(int pLen, const char* pStr);

	//@}

	/**
	  * \name Append and Concatenation
	  */
	//@{

	//! Append as "C" strcat().
	const KString& Append(const char* pStr); 

	//! Append as "C" strncat().
	const KString& AppendN(const char* pStr, kUInt pLength); 

	//! KString append.
	const KString& operator+=(const KString& pKStr);

	//! Character append.
	const KString& operator+=(char pChar);

	//! String append.
	const KString& operator+=(const char* pStr);

	//! Int append.
	const KString& operator+=(int pValue);

	//! Float append.
	const KString& operator+=(float pValue);

	//! KString concatenation.
	friend KBASELIB_DLL KString operator+(const KString& pKStr1, const KString& pKStr2);

	//! Character concatenation.
	friend KBASELIB_DLL KString operator+(const KString& pKStr, char pChar);

	//! String concatenation.
	friend KBASELIB_DLL KString operator+(const KString& pKStr, const char* pStr);

	//! Int concatenation.
	friend KBASELIB_DLL KString operator+(const KString& pKStr, int pValue);

	//! Float concatenation.
	friend KBASELIB_DLL KString operator+(const KString& pKStr, float pValue);

	//@}

	/**
	  * \name String Comparison
	  */
	//@{
	
	//! Compare as "C" strcmp().
	int Compare(const char * pStr) const;        

	//! Compare as "C" stricmp().
	int CompareNoCase( const char * pStr ) const;   

	//! Equality operator.
	bool operator==(const KString &pStr) const;

	//! Inequality operator.
	bool operator!=(const KString &pStr) const;

	//! Inferior to operator.
	bool operator< (const KString &pStr) const;

	//! Inferior or equal to operator.
	bool operator<=(const KString &pStr) const;

	//! Superior or equal to operator.
	bool operator>=(const KString &pStr) const;

	//! Superior to operator.
	bool operator> (const KString &pStr) const;

	//! Equality operator.
	bool operator==(const char *pStr) const;

	//! Inequality operator.
	bool operator!=(const char *pStr) const;

	//! Inferior to operator.
	bool operator< (const char *pStr) const;

	//! Inferior or equal to operator.
	bool operator<=(const char *pStr) const;

	//! Superior or equal to operator.
	bool operator>=(const char *pStr) const;

	//! Superior to operator.
	bool operator> (const char *pStr) const;

	//@}

	/**
	  * \name Substring Extraction
	  */
	//@{

	//! Extract middle string for a given length.
	KString Mid(kUInt pFirst, kUInt pCount) const;

	//! Extract middle string up to the end.
	KString Mid(kUInt pFirst) const;

	//! Extract left string.
	KString Left(kUInt pCount) const;

	//! Extract right string.
	KString Right(kUInt pCount) const;

	//@}

	/**
	  * \name Padding
	  */
	//@{

	//! Padding types.
	enum PaddingType { eRight, eLeft, eBoth };

	//! Add padding characters.
	KString Pad(PaddingType pPadding, kUInt pLen, char pCar = ' ') const;

	//! Remove padding characters.
	KString UnPad(PaddingType pPadding) const;

	//@}

	/**
	  * \name Conversion
	  */
	//@{

	//! Uppercase conversion
	KString Upper() const;

	//! Lowercase conversion
	KString Lower() const;

	//! Reverse conversion
	KString Reverse() const;

	//! Convert to Unix, changes \\r\\n characters for a single \\n
	KString ConvertToUnix() const;

	//! Convert to Windows, changes \\n character for both \\r\\n
	KString ConvertToWindows() const;

	//@}

	/**
	  * \name Search
	  */
	//@{

	/** Look for a single character match, like "C" strchr().
	  * \return Index or -1 if not found.
	  */
	int Find(char pChar, kUInt pStartPosition = 0) const; 

	/** Look for a substring match, like "C" strstr().
	  * \return Starting index or -1 if not found.
	  */
	int Find(const char* pStrSub, kUInt pStartPosition = 0) const;

	/** Look for a single character match, like "C" strrchr().
	  * \return Index or -1 if not found.
	  */
	int ReverseFind(char pChar) const;

	/** Look for a single character match, like "C" strpbrk().
	  * \return Index or -1 if not found.
	  */
	int FindOneOf(const char * pStrCharSet, kUInt pStartPosition = 0) const;

	/** Replace a substring.
	  * \return \c true if substring found and replaced.
	  */
	bool FindAndReplace(const char* pFind, const char* pReplaceBy, kUInt pStartPosition = 0);

	//@}

	/**
	  * \name Token Extraction
	  */
	//@{

	//! Get number of tokens.
	int GetTokenCount(const char* pSpans);

	//! Get token at given index.
	KString GetToken(int pTokenIndex, const char* pSpans);

	//@}

///////////////////////////////////////////////////////////////////////////////
//
//  WARNING!
//
//	Anything beyond these lines may not be documented accurately and is 
// 	subject to change without notice.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef DOXYGEN_SHOULD_SKIP_THIS

private:

	// Lengths/sizes in characters. 
	// Note: an extra character is always allocated.
	char *mData; // Actual string (zero terminated).
	kUInt mDataLen; // Does not include terminating 0.
	kUInt mAllocLen; // Does not include terminating 0.
	char mStaticData[KStringStaticSize]; // Actual static string (zero terminated) used as long as the string is smaller than KStringStaticSize

	void Init();
	void FreeBuffer();
	void FreeBuffer(char *&pOldData);

	bool AllocCopy(KString& pDest, kUInt pCopyLen, kUInt pCopyIndex) const;
	bool AllocBuffer(kUInt pLen);
	bool AllocBuffer(kUInt pLen, char *&pOldData);

	bool AssignCopy(kUInt pSrcLen, const char * pSrcData);
	bool ConcatCopy(kUInt pSrc1Len, const char * pSrc1Data, 
					   kUInt pSrc2Len, const char * pSrc2Data);
	bool ConcatInPlace(kUInt pSrcLen, const char * pSrcData);

	bool IsIn(char pChar, const char* pString);

#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

};

//! KString concatenation.
KBASELIB_DLL KString operator+(const KString& pKStr1, const KString& pKStr2);

//! Character concatenation.
KBASELIB_DLL KString operator+(const KString& pKStr, char pChar);

//! String concatenation.
KBASELIB_DLL KString operator+(const KString& pKStr, const char* pStr);

//! Int concatenation.
KBASELIB_DLL KString operator+(const KString& pKStr, int pValue);

//! Float concatenation.
KBASELIB_DLL KString operator+(const KString& pKStr, float pValue);


// Strip the first and last character of a string assuming they are quotation marks.
inline void StripQuotationMarks(KString& str)
{
	str = str.Mid( 1, str.GetLen() - 2 );
}

inline void RemoveChar( KString& pString, char lToRemove )
{
    int lPos = pString.ReverseFind( lToRemove );

    while( lPos >= 0 ) 
    {
        pString = pString.Left( lPos ) + pString.Mid( lPos + 1 );
        lPos = pString.ReverseFind( lToRemove );
    }
}

#endif // #define _KString_h

