#ifndef NX_COLLISION_CONTROLLER
#define NX_COLLISION_CONTROLLER
/*----------------------------------------------------------------------------*\
|
|								NovodeX Technology
|
|							     www.novodex.com
|
\*----------------------------------------------------------------------------*/

/* Exclude from documentation */
/** \cond */

//#include "NxAllocateable.h"
#include "NxController.h"

class NxScene;
class NxActor;
class NxBounds3;
class ControllerManager;
class SweptVolume;

class Controller //: public NxAllocateable
	{
	public:
										Controller(const NxControllerDesc& desc, NxScene* scene);
	virtual								~Controller();

	virtual	bool						getWorldBox(NxExtendedBounds3& box)	const	= 0;
	virtual	NxController*				getController()						= 0;
	virtual	NxActor*					getActor()					const	{ return kineActor; };

			NxControllerType			type;

	// User params
			NxHeightFieldAxis			upDirection;
			NxF32						slopeLimit;
			NxF32						skinWidth;
			NxF32						stepOffset;
			NxUserControllerHitReport*	callback;
			void*						userData;

	// Internal data
			void*						cctModule;			// Internal CCT object. Optim test for Ubi.
			NxActor*					kineActor;			// Associated kinematic actor
			NxExtendedVec3				position;			// Current position
			NxExtendedVec3				filteredPosition;	// Current position after feedback filter
			NxExtendedVec3				exposedPosition;	// Position visible from the outside at any given time
			Extended					memory;				// Memory variable for feedback filter
			NxScene*					scene;				// Handy scene owner
			Controller*					next;				// Linked list of controllers
			ControllerManager*			manager;			// Owner manager
			bool						handleSlope;		// True to handle walkable parts according to slope

	protected:
	// Internal methods
			bool						setPos(const NxExtendedVec3& pos);
			void						setCollision(bool enabled);
			void						move(SweptVolume& volume, const NxVec3& disp, NxU32 activeGroups, NxF32 minDist, NxU32& collisionFlags, NxF32 sharpness);
	};

/** \endcond */

#endif
