/****************************************************************************\
*                                                                            *
* 19.03.2006                                                                 *
* CQTSceneGraph - Header File                                                *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __CQTSCENEGRAPH_H__
#define __CQTSCENEGRAPH_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <Memory/mmgr.h>
#include <list>
#include <Memory/TSmartPointer.h>
#include <Math/CMatrix44.h>
#include "IRepresentation.h"
#include "ISceneGraph.h"
#include "GraphStates/IStateNode.h"

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/

template<typename T>
class TQTNode;

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class CQTSceneGraph : public ISceneGraph
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        CQTSceneGraph() {Clear();}
        
        virtual bool Create(float fSizeX, float fSizeZ, float fMinY, float fMaxY, word wGraphDepth);
        virtual void Render(bool bVerify);

        virtual void AddObject(const TSmartPointer<IRepresentation> &pRepresentation);
        virtual void RemoveObject(const TSmartPointer<IRepresentation> &pRepresentation);
        virtual void PickObject(const CVector3f &vRayOrigin, const CVector3f &vRayDir,
                                TSmartPointer<IRepresentation> &pRepresentation, CVector3f &vHitPos);

		virtual TQTNode<IRepresentation> * const CreateSpecialRootNode(TSmartPointer<IStateNode> pStateNode);

        virtual void Destroy();

        virtual ~CQTSceneGraph() {Destroy();}
        
    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        void Clear();

        //Private copy constructor and assignment operator to prevent mistakes:
        CQTSceneGraph(const CQTSceneGraph &);
        CQTSceneGraph &operator =(const CQTSceneGraph &);
        
    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
        TQTNode<IRepresentation> *m_pRootNode;

		std::vector<TQTNode<IRepresentation> *> m_vSpecialRootNodes;

		float m_fSizeX;
		float m_fSizeZ;
		float m_fMinY;
		float m_fMaxY;
		word  m_wGraphDepth;
       
};

#endif
