/****************************************************************************\
*                                                                            *
* 26.04.2006                                                                 *
* CTransformableObj - Source File                                            *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CTransformableObj.h"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
CTransformableObj::CTransformableObj()
{
    SetPos(CVector3f(0.0f, 0.0f, 0.0f));
    SetRot(CVector3f(0.0f, 0.0f, 0.0f));
    SetScale(CVector3f(1.0f, 1.0f, 1.0f));
}

void CTransformableObj::SetPos(const CVector3f &vPos)
{
    m_vPos = vPos;
    m_bMatrixValid = false;
}

void CTransformableObj::SetRot(const CVector3f &vRot)
{
    m_vRot = vRot;
    m_bVectorsValid = m_bMatrixValid = false;
}

void CTransformableObj::SetScale(const CVector3f &vScale)
{
    m_vScale = vScale;
    m_bMatrixValid = false;
}

void CTransformableObj::LookAt(const CVector3f &vPoint)
{
	if(!m_bMatrixValid) //To make sure that the matrix created below is valid
	{
		Update();
	}
	m_vIn = vPoint - GetPos();
	float fLen = m_vIn.Length();
	if(fLen > 0.001f) //Prevent division by zero
	{
		m_vIn /= fLen;
		if(m_vIn.m_Y > 0.999f || m_vIn.m_Y < -0.999f) //Prevent up and in vector being the same
		{
			m_vUp.Set(0.0f, 0.0f, 1.0f);
		}
		else
		{
			m_vUp.Set(0.0f, 1.0f, 0.0f);
		}
		m_vRight = m_vIn * m_vUp;
		m_vRight.Normalize();
		m_vUp = m_vRight * m_vIn;
		//Ok, we got the vectors! Lets create the matrix
		m_Transformation._11 = m_vRight.m_X;
		m_Transformation._21 = m_vRight.m_Y;
		m_Transformation._31 = m_vRight.m_Z;
		m_Transformation._12 = m_vUp.m_X;
		m_Transformation._22 = m_vUp.m_Y;
		m_Transformation._32 = m_vUp.m_Z;
		//invert the in vector
		m_Transformation._13 = -m_vIn.m_X;
		m_Transformation._23 = -m_vIn.m_Y;
		m_Transformation._33 = -m_vIn.m_Z;

		//Extract rotation
		m_vRot = m_Transformation.GetZXYRotationVector();
		m_bMatrixValid = m_bVectorsValid = true;
	}
}

void CTransformableObj::LookAt(const CVector3f &vPoint, const CVector3f &vUp)
{
    //ToDo
    LOG(TL_ERROR, "Error: CTransformableObj::LookAt not implemented");
}

void CTransformableObj::MoveIn(float length)
{
    if(!m_bVectorsValid)
        Update();

    m_vPos += m_vIn * length;

    m_bMatrixValid = false;
}

void CTransformableObj::MoveUp(float length)
{
    if(!m_bVectorsValid)
        Update();

    m_vPos += m_vUp * length;

    m_bMatrixValid = false;
}

void CTransformableObj::MoveRight(float length)
{
    if(!m_bVectorsValid)
        Update();

    m_vPos += m_vRight * length;

    m_bMatrixValid = false;
}

void CTransformableObj::MoveX(float length)
{
    m_vPos.m_X += length;

    m_bMatrixValid = false;
}

void CTransformableObj::MoveY(float length)
{
    m_vPos.m_Y += length;

    m_bMatrixValid = false;
}

void CTransformableObj::MoveZ(float length)
{
    m_vPos.m_Z += length;

    m_bMatrixValid = false;
}

void CTransformableObj::Move(const CVector3f &vDistance)
{
    m_vPos += vDistance;

    m_bMatrixValid = false;
}

void CTransformableObj::RotateX(float angle)
{
    m_vRot.m_X += angle;

    if(m_vRot.m_X > MY_PI / 2)
        m_vRot.m_X = MY_PI / 2;
    else if(m_vRot.m_X < -MY_PI / 2)
        m_vRot.m_X = -MY_PI / 2;

    m_bVectorsValid = m_bMatrixValid = false;
}

void CTransformableObj::RotateY(float angle)
{
    m_vRot.m_Y += angle;

    if(m_vRot.m_Y >= MY_PI * 2)
        m_vRot.m_Y -= MY_PI * 2;
    else if(m_vRot.m_Y <= -MY_PI * 2)
        m_vRot.m_Y += MY_PI * 2;

    m_bVectorsValid = m_bMatrixValid = false;
}

void CTransformableObj::RotateZ(float angle)
{
    m_vRot.m_Z += angle;

    if(m_vRot.m_Z >= MY_PI * 2)
        m_vRot.m_Z -= MY_PI * 2;
    else if(m_vRot.m_Z <= -MY_PI * 2)
        m_vRot.m_Z += MY_PI * 2;

    m_bVectorsValid = m_bMatrixValid = false;
}

const CVector3f &CTransformableObj::GetIn()
{
    if(!m_bVectorsValid)
        Update();
    return m_vIn;
}

const CVector3f &CTransformableObj::GetUp()
{
    if(!m_bVectorsValid)
        Update();
    return m_vUp;
}

const CVector3f &CTransformableObj::GetRight()
{
    if(!m_bVectorsValid)
        Update();
    return m_vRight;
}


const CMatrix44 &CTransformableObj::GetTransformation()
{
    if(!m_bMatrixValid)
    {
        Update();
    }

    return m_Transformation;
}

CTransformableObj::~CTransformableObj()
{

}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
void CTransformableObj::Update()
{
/*
	CMatrix44 rot, trans;
	CMatrix44 rotY, rotX;
	rotX.RotationX(m_vRot.m_X);
	rotY.RotationY(m_vRot.m_Y);
	rot.Multiply(rotY, rotX);
	trans.Translation(m_vPos);
	m_Transformation.Multiply(rot, trans);
*/
    CMatrix44 tmp, tmp2;
    m_Transformation.Scaling(m_vScale);
    tmp2.RotationZXY(m_vRot.m_X, m_vRot.m_Y, m_vRot.m_Z);
    tmp.Multiply(tmp2, m_Transformation); //First scaling, then rotation...
    tmp2.Translation(m_vPos);
    m_Transformation.Multiply(tmp2, tmp); //... then translation

    //Get the view vectors
    m_vRight.m_X = m_Transformation._11;
    m_vRight.m_Y = m_Transformation._21;
    m_vRight.m_Z = m_Transformation._31;

    m_vUp.m_X = m_Transformation._12;
    m_vUp.m_Y = m_Transformation._22;
    m_vUp.m_Z = m_Transformation._32;

    //...and invert the in vector
    m_vIn.m_X = -m_Transformation._13;
    m_vIn.m_Y = -m_Transformation._23;
    m_vIn.m_Z = -m_Transformation._33;

    m_bVectorsValid = m_bMatrixValid = true;
}