/****************************************************************************\
*                                                                            *
* 19.03.2006                                                                 *
* CTransformableObj - Header File                                            *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __CTRANSFORMABLEOBJ_H__
#define __CTRANSFORMABLEOBJ_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <Memory/mmgr.h>
#include <Math/TVector3.h>
#include <Math/CMatrix44.h>

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class CTransformableObj
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        CTransformableObj();

        //State setting
        void SetPos(const CVector3f &vPos);
        void SetRot(const CVector3f &vRot);
        void SetScale(const CVector3f &vScale);
		void LookAt(const CVector3f &vPoint);
        void LookAt(const CVector3f &vPoint, const CVector3f &vUp);

        void MoveIn(float length);
        void MoveUp(float length);
        void MoveRight(float length);
        void MoveX(float length);
        void MoveY(float length);
        void MoveZ(float length);
        void Move(const CVector3f &vDistance);

        //Rotations (use Move_ before Rotate_ to prevent multiple calculations of matrices)
        void RotateX(float angle);
        void RotateY(float angle);
        void RotateZ(float angle);

        //State request:
        inline const CVector3f &GetPos() const {return m_vPos;}
        inline const CVector3f &GetRot() const {return m_vRot;}
        inline const CVector3f &GetScale() const {return m_vScale;}
        const CVector3f &GetIn();
        const CVector3f &GetUp();
        const CVector3f &GetRight();

        const CMatrix44 &GetTransformation();
        
        virtual ~CTransformableObj();
        
    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        void Update();

        //Private copy constructor and assignment operator to prevent mistakes:
        CTransformableObj(const CTransformableObj &);
        CTransformableObj &operator =(const CTransformableObj &);
        
    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:
    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
        CVector3f m_vPos, m_vRot, m_vScale;
        CMatrix44 m_Transformation;

        //Helpers
        CVector3f m_vIn, m_vRight, m_vUp;
        bool m_bMatrixValid;
        bool m_bVectorsValid;
};

#endif
