/****************************************************************************\
*                                                                            *
* 29.03.2006                                                                 *
* CFrustum - Header File                                                     *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __CFRUSTUM_H__
#define __CFRUSTUM_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <Memory/mmgr.h>
#include <Math/TPlane.h>
#include <Math/CMatrix44.h>
#include <Math/TAABox3.h>
#include <Math/TSphere.h>

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/
#define FRUSTUM_OUTSIDE    -1
#define FRUSTUM_INSIDE      1 
#define FRUSTUM_INTERSECTS  0

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class CFrustum
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        CFrustum() {}
        
        void CalculateFrustum(const CMatrix44 &projection, const CMatrix44 &view);  //Calculates the frustum

        sbyte SphereInFrustum(const CSpheref &sphere) const;
        bool BoxInFrustum(const CAABox3f &box) const;
        bool BoxInFrustum(const CBox3f &box) const;
        
        virtual ~CFrustum() {}
        
    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        //Private copy constructor and assignment operator to prevent mistakes:
        CFrustum(const CFrustum &);
        CFrustum &operator =(const CFrustum &);
        
    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
        CPlanef m_Planes[6];        
};

#endif
