/****************************************************************************\
*                                                                            *
* 16.04.2006                                                                 *
* CScrollXZCamCtrl - Source File                                                  *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CScrollXZCamCtrl.h"
#include "CCamera.h"
#include "CEngineFactory.h"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
CScrollXZCamCtrl::CScrollXZCamCtrl(float fScrollFrame, CAABox3f scrollArea, float fSpeed,
                                   const TSmartPointer<CTimer> &pTimer, word wInputPriority)
{
    assert((bool) pTimer && "Error: Timer is NULL in CScrollXZCamCtrl::CScrollXZCamCtrl");

    m_fScrollFrame = fScrollFrame;
    m_ScrollArea = scrollArea;
    m_fSpeed = fSpeed;

    m_pTimer = pTimer;
    m_pCamera = NULL;
    m_bForward = false;
    m_bBackward = false;
    m_bLeft = false;
    m_bRight = false;

    SetPriority(wInputPriority);
}

void CScrollXZCamCtrl::Enable(const TSmartPointer<CCamera> &pCamera)
{
    if(!IsEnabled())
    {
        CEngineFactory::InstancePtr()->GetInputMgr()->RegisterHandler(this);
        m_bForward = false;
        m_bBackward = false;
        m_bLeft = false;
        m_bRight = false;
    }
    ICamCtrl::Enable(pCamera);

    //Set view angle
	pCamera->SetPos(CVector3f(m_ScrollArea.SizeX() / 2.0f, 50, m_ScrollArea.SizeZ() / 2.0f));
    pCamera->SetRot(CVector3f(3.0f*MY_PI/8.0f, 0.0f, 0.0f));
}

void CScrollXZCamCtrl::Update()
{
    if(IsEnabled())
    {
        word wWidth, wHeight;
        CEngineFactory::InstancePtr()->GetGuiMgr()->GetVScreenSize(wWidth, wHeight);
        CVector2f vPos = CEngineFactory::InstancePtr()->GetGuiMgr()->GetCursorPos();
        CVector3f vDir;
        float dt = (float) m_pTimer->GetFrameDuration();
        if(m_bLeft || (vPos.m_X <= m_fScrollFrame && vPos.m_X >= 0))
        {
            //Get movement direction and remove y-value (only move in XZ-Direction)
            vDir = -m_pCamera->GetRight();
            vDir.m_Y = 0.0f;
            if(vDir.m_X != 0.0f || vDir.m_Z != 0.0f)
            {
                vDir.Normalize();
                m_pCamera->Move(vDir * m_fSpeed * dt);
            }
        }
        if(m_bRight || (((float) wWidth) - vPos.m_X <= m_fScrollFrame + 1 && vPos.m_X <= wWidth)) // + 1 is a fix for the rounding error introduced by locking the cursor
        {
            //Get movement direction and remove y-value (only move in XZ-Direction)
            vDir = m_pCamera->GetRight();
            vDir.m_Y = 0;
            if(vDir.m_X != 0.0f || vDir.m_Z != 0.0f)
            {
                vDir.Normalize();
                m_pCamera->Move(vDir * m_fSpeed * dt);
            }
        }

        if(m_bForward || (vPos.m_Y <= m_fScrollFrame && vPos.m_Y >= 0))
        {
            //Get movement direction and remove y-value (only move in XZ-Direction)
            vDir = m_pCamera->GetUp();
            vDir.m_Y = 0;
            if(vDir.m_X != 0.0f || vDir.m_Z != 0.0f)
            {
                vDir.Normalize();
                m_pCamera->Move(vDir * m_fSpeed * dt);
            }
        }
        if(m_bBackward || (((float) wHeight) - vPos.m_Y <= m_fScrollFrame + 1 && vPos.m_Y <= wHeight)) // + 1 is a fix for the rounding error introduced by locking the cursor
        {
            //Get movement direction and remove y-value (only move in XZ-Direction)
            vDir = -m_pCamera->GetUp();
            vDir.m_Y = 0;
            if(vDir.m_X != 0.0f || vDir.m_Z != 0.0f)
            {
                vDir.Normalize();
                m_pCamera->Move(vDir * m_fSpeed * dt);
            }
        }

        vDir = m_pCamera->GetPos();
        if(vDir.m_X < m_ScrollArea.m_MinX)
        {
            vDir.m_X = m_ScrollArea.m_MinX;
        }
        else if(vDir.m_X > m_ScrollArea.m_MaxX)
        {
            vDir.m_X = m_ScrollArea.m_MaxX;
        }

        if(vDir.m_Y < m_ScrollArea.m_MinY)
        {
            vDir.m_Y = m_ScrollArea.m_MinY;
        }
        else if(vDir.m_Y > m_ScrollArea.m_MaxY)
        {
            vDir.m_Y = m_ScrollArea.m_MaxY;
        }

        if(vDir.m_Z < m_ScrollArea.m_MinZ)
        {
            vDir.m_Z = m_ScrollArea.m_MinZ;
        }
        else if(vDir.m_Z > m_ScrollArea.m_MaxZ)
        {
            vDir.m_Z = m_ScrollArea.m_MaxZ;
        }

        m_pCamera->SetPos(vDir);
        
    }
}

void CScrollXZCamCtrl::Disable()
{
    if(IsEnabled())
    {
        CEngineFactory::InstancePtr()->GetInputMgr()->UnregisterHandler(this);
    }
    ICamCtrl::Disable();
    m_bForward = false;
    m_bBackward = false;
    m_bLeft = false;
    m_bRight = false;
}

CScrollXZCamCtrl::~CScrollXZCamCtrl()
{
    m_pTimer = NULL;
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
bool CScrollXZCamCtrl::OnKeyDown(dword dwKey)
{
    if(!IsEnabled())
        return false;

    switch(dwKey)
    {
        case cg_dwCursorUp:
            m_bForward = true;
            return true;
        case cg_dwCursorDown:
            m_bBackward = true;
            return true;
        case cg_dwCursorLeft:
            m_bLeft = true;
            return true;
        case cg_dwCursorRight:
            m_bRight = true;
            return true;
    }

    return false;

}

bool CScrollXZCamCtrl::OnKeyUp(dword dwKey)
{
    if(!IsEnabled())
        return false;

    switch(dwKey)
    {
        case cg_dwCursorUp:
            m_bForward = false;
            return true;
        case cg_dwCursorDown:
            m_bBackward = false;
            return true;
        case cg_dwCursorLeft:
            m_bLeft = false;
            return true;
        case cg_dwCursorRight:
            m_bRight = false;
            return true;
    }

    return false;
}
