/****************************************************************************\
*                                                                            *
* 24.03.2006                                                                 *
* CGuiContainer - Header File                                                *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __CGUICONTAINER_H__
#define __CGUICONTAINER_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <Memory/mmgr.h>
#include "IGuiObject.h"

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class CGuiContainer : public IGuiObject
{
    friend class CGuiMgr;
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        virtual void Update(float dt);

        virtual bool OnGetFocus();

        virtual bool OnMouseMove(float fX, float fY, sdword sdwZ);
        virtual bool OnMouseKeyDown(byte bKey);
        virtual bool OnMouseKeyUp(byte bKey);
        virtual bool OnChar(char c);
        virtual bool OnKeyDown(dword dwKey);
        virtual bool OnKeyUp(dword dwKey);

        virtual void AddChild(const TSmartPointer<IGuiObject> &pObject);
        virtual void RemoveChild(const TSmartPointer<IGuiObject> &pObject);
        virtual bool Contains(const TSmartPointer<IGuiObject> &pObject) const;

        virtual void OnChildPosUpdate() {m_bResortChildren = true;}

        virtual void Destroy();

        virtual ~CGuiContainer() {Destroy();}
        
    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
        CGuiContainer() : IGuiObject() {Clear();}

        void ResortChildren();
        virtual void RenderSub();

    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        void Clear();

        //Private copy constructor and assignment operator to prevent mistakes:
        CGuiContainer(const CGuiContainer &);
        CGuiContainer &operator =(const CGuiContainer &);
        
    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
        typedef std::vector< TSmartPointer<IGuiObject> > ObjectList;
        ObjectList m_Children;
        bool m_bResortChildren;
        
};

#endif
