/****************************************************************************\
*                                                                            *
* 16.04.2006                                                                 *
* IRepresentation - Source File                                              *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <CEngineFactory.h>
#include "IRepresentation.h"
#include "ISceneGraph.h"
#include "TSceneNode.h"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
void IRepresentation::ObjTransformationChanged()
{
    m_bSphereValid = false;
    m_bBoxValid = false;
    if(m_pParentNode)
    {
        m_pParentNode->ObjectChanged(this);
    }
}

const CSpheref &IRepresentation::GetBoundingSphere()
{
    if(!m_bSphereValid)
    {
        m_CurrBoundingSphere = (m_BoundingSphere * GetScale()).GetRotatedSphere(GetRot()) + GetPos();
        m_bSphereValid = true;
    }

    return m_CurrBoundingSphere;
}

const CBox3f &IRepresentation::GetBoundingBox()
{
    if(!m_bBoxValid)
    {
        m_CurrBoundingBox = (m_AABoundingBox * GetScale()).GetRotatedBox(GetRot()) + GetPos();
        m_bBoxValid = true;
    }

    return m_CurrBoundingBox;
}

#include <gl/GL.h> //Debug
dword IRepresentation::ms_dwRendered = 0;   //Debug
dword IRepresentation::ms_dwVerified = 0;   //Debug
dword IRepresentation::ms_dwBothTests = 0;   //Debug

void IRepresentation::Render(const CFrustum &frustum, bool bVerify)
{
    bool bRender = true;
    if(bVerify)
    {
        ++ms_dwVerified;
        sbyte val = frustum.SphereInFrustum(GetBoundingSphere());
        if(val == FRUSTUM_INSIDE)
        {
            bVerify = false;
        }
        else if(val == FRUSTUM_INTERSECTS && frustum.BoxInFrustum(GetBoundingBox()))
        {
            ++ms_dwBothTests; //Debug
        }
        else
        {
            bRender = false;
        }
    }

    if(bRender)
//    if(frustum.BoxInFrustum(GetAABoundingBox()))
    {
        ++ms_dwRendered;   //Debug
        IRenderer *pRenderer = CEngineFactory::InstancePtr()->GetRenderer();

        pRenderer->SetMatrixMode(EMM_MODELVIEW);
/*
        //Debug: Render bounding box
        CBox3f box = GetBoundingBox();
        glBegin(GL_LINES);
        glColor3f(1.0f, 0.0f, 0.0f);
        glVertex3fv(&box.m_vPoint.m_X);
        glVertex3fv(&((box.m_vPoint + box.m_vA).m_X));

		glVertex3fv(&box.m_vPoint.m_X);
		glVertex3fv(&((box.m_vPoint + box.m_vB).m_X));

		glVertex3fv(&box.m_vPoint.m_X);
		glVertex3fv(&((box.m_vPoint + box.m_vC).m_X));

		glVertex3fv(&((box.m_vPoint + box.m_vA).m_X));
		glVertex3fv(&((box.m_vPoint + box.m_vA + box.m_vB).m_X));

		glVertex3fv(&((box.m_vPoint + box.m_vA).m_X));
		glVertex3fv(&((box.m_vPoint + box.m_vA + box.m_vC).m_X));

		glVertex3fv(&((box.m_vPoint + box.m_vB).m_X));
		glVertex3fv(&((box.m_vPoint + box.m_vB + box.m_vA).m_X));

		glVertex3fv(&((box.m_vPoint + box.m_vB).m_X));
		glVertex3fv(&((box.m_vPoint + box.m_vB + box.m_vC).m_X));

		glVertex3fv(&((box.m_vPoint + box.m_vC).m_X));
		glVertex3fv(&((box.m_vPoint + box.m_vC + box.m_vA).m_X));

		glVertex3fv(&((box.m_vPoint + box.m_vC).m_X));
		glVertex3fv(&((box.m_vPoint + box.m_vC + box.m_vB).m_X));

		glVertex3fv(&((box.m_vPoint + box.m_vA + box.m_vB + box.m_vC).m_X));
		glVertex3fv(&((box.m_vPoint + box.m_vA + box.m_vB).m_X));

		glVertex3fv(&((box.m_vPoint + box.m_vA + box.m_vB + box.m_vC).m_X));
		glVertex3fv(&((box.m_vPoint + box.m_vA + box.m_vC).m_X));

		glVertex3fv(&((box.m_vPoint + box.m_vA + box.m_vB + box.m_vC).m_X));
		glVertex3fv(&((box.m_vPoint + box.m_vB + box.m_vC).m_X));
        glEnd();
        //End Debug
*/

/*
        //Debug: Render bounding sphere
        CSpheref sphere = GetBoundingSphere();
        glBegin(GL_LINES);
        glColor3f(0.0f, 0.0f, 1.0f);
        glVertex3f(sphere.m_vCenter.m_X - sphere.m_Radius, sphere.m_vCenter.m_Y, sphere.m_vCenter.m_Z);
        glVertex3f(sphere.m_vCenter.m_X + sphere.m_Radius, sphere.m_vCenter.m_Y, sphere.m_vCenter.m_Z);

        glVertex3f(sphere.m_vCenter.m_X, sphere.m_vCenter.m_Y - sphere.m_Radius, sphere.m_vCenter.m_Z);
        glVertex3f(sphere.m_vCenter.m_X, sphere.m_vCenter.m_Y + sphere.m_Radius, sphere.m_vCenter.m_Z);

        glVertex3f(sphere.m_vCenter.m_X, sphere.m_vCenter.m_Y, sphere.m_vCenter.m_Z - sphere.m_Radius);
        glVertex3f(sphere.m_vCenter.m_X, sphere.m_vCenter.m_Y, sphere.m_vCenter.m_Z + sphere.m_Radius);

        glEnd();
        //End Debug
*/

        pRenderer->SaveMatrix();
        pRenderer->MultMatrix(GetTransformation());
		RenderSub(frustum, bVerify);
        pRenderer->RestoreMatrix();
    }
}

void IRepresentation::Destroy()
{
	if(GetRefCount() > 0)
	{
		m_pSceneGraph->RemoveObject(this);
	}
}
/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/


