/****************************************************************************\
*                                                                            *
* 19.03.2006                                                                 *
* ISceneGraph - Header File                                                  *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __ISCENEGRAPH_H__
#define __ISCENEGRAPH_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <Memory/mmgr.h>
#include <Memory/IManagedObject.h>
#include <Display/Gui/CGuiMgr.h>
#include "IRepresentation.h"
#include "Camera/CCamera.h"
#include <Math/CMatrix44.h>

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/
class IRepresentation;

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class ISceneGraph : public IManagedObject
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        ISceneGraph() {}

        virtual void Render(bool bVerify) = 0;
        virtual void AddObject(const TSmartPointer<IRepresentation> &pRepresentation) = 0;
        virtual void RemoveObject(const TSmartPointer<IRepresentation> &pRepresentation) = 0;
        virtual void PickObject(const CVector3f &vRayOrigin, const CVector3f &vRayDir,
                                TSmartPointer<IRepresentation> &pRepresentation, CVector3f &vHitPos) = 0;


        inline void SetCamera(const TSmartPointer<CCamera> &pCamera) {m_pCamera = pCamera;}
        inline TSmartPointer<CCamera> GetCamera() {return m_pCamera;}

		inline void SetEnvironment(TSmartPointer<IRepresentation> pEnvironment){m_pEnvironment = pEnvironment;}
		inline TSmartPointer<IRepresentation> GetEnvironment() {return m_pEnvironment;}
		inline void SetSunDir(const CVector3f &vDir) {m_vSunDir = vDir;}

		virtual void Destroy() = 0;        
        virtual ~ISceneGraph() {}
        
    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        //Private copy constructor and assignment operator to prevent mistakes:
        ISceneGraph(const ISceneGraph &);
        ISceneGraph &operator =(const ISceneGraph &);
        
    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:
        TSmartPointer<CCamera> m_pCamera;
		TSmartPointer<IRepresentation> m_pEnvironment;
		CVector3f m_vSunDir;
		CMatrix44 m_CameraRotM;

    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
};

#endif
