/****************************************************************************\
*                                                                            *
* 22.02.2006                                                                 *
* CConsoleWindow - Source File                                               *
*                                                                            *
* (c) Raphael Charwot                                                        *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CConsoleWindow.h"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
bool CConsoleWindow::Create(const std::string &sTitle, word wWidth, word wHeight, sword swPosX, sword swPosY)
{
    if(!AllocConsole())
    {
        LOG(TL_USER, "Error: AllocConsole failed in CConsoleWindow::Create (%lu)", GetLastError());
        return false;
    }

    if(!SetConsoleTitle(sTitle.c_str()))
    {
        LOG(TL_ERROR, "Error: SetConsoleTitle failed in CConsoleWindow::Create (%lu)", GetLastError());
        return false;
    }

    m_hBuffer = CreateConsoleScreenBuffer(GENERIC_READ | GENERIC_WRITE,	FILE_SHARE_READ | FILE_SHARE_WRITE,	NULL, CONSOLE_TEXTMODE_BUFFER, NULL);

    if(m_hBuffer == INVALID_HANDLE_VALUE)
    {
        LOG(TL_ERROR, "Error: CreateConsoleScreenBuffer failed in CConsoleWindow::Create (%lu)", GetLastError());
        return false;
    }
    
    if(!SetConsoleActiveScreenBuffer(m_hBuffer))
    {
        LOG(TL_ERROR, "Error: SetConsoleActiveScreenBuffer failed in CConsoleWindow::Create (%lu)", GetLastError());
        return false;
    }

    COORD coord;
    coord.X = wWidth;
    coord.Y = wHeight;

    if(!SetConsoleScreenBufferSize(m_hBuffer, coord))
    {
        LOG(TL_ERROR, "Error: SetConsoleScreenBufferSize failed in CConsoleWindow::Create (%lu)", GetLastError());
        return false;
    }

    m_pInFile = freopen("CONIN$","rt",stdin);
    m_pOutFile = freopen("CONOUT$", "wt", stdout);
    m_pErrFile = freopen("CONOUT$", "wt", stderr);

    if(!m_pInFile || !m_pOutFile || !m_pErrFile)
    {
        LOG(TL_ERROR, "Error: Files could not be created in CConsoleWindow::Create");
        return false;
    }

    //Move window
    SetWindowPos(GetConsoleWindow(), 0, swPosX, swPosY, 0, 0, SWP_NOSIZE | SWP_NOZORDER | SWP_NOACTIVATE);

    m_wWidth = wWidth;
    m_wHeight = wHeight;

    return true;
}

void CConsoleWindow::Destroy()
{
    if(m_pInFile)
        fclose(m_pInFile);
    if(m_pOutFile)
        fclose(m_pOutFile);
    if(m_pErrFile)
        fclose(m_pErrFile);

    if(m_hBuffer != INVALID_HANDLE_VALUE)
        CloseHandle(m_hBuffer);
    FreeConsole();
    Clear();
}
/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
void CConsoleWindow::Clear()
{
    m_wWidth = 0;
    m_wHeight = 0;

    m_hBuffer = INVALID_HANDLE_VALUE;

    m_pInFile = NULL;
    m_pOutFile = NULL;
    m_pErrFile = NULL;
}

