/****************************************************************************\
*                                                                            *
* 16.02.2006                                                                 *
* CW32Window - Header File                                                   *
*                                                                            *
* (c) Raphael Charwot                                                        *
*                                                                            *
\****************************************************************************/

#ifndef __CW32WINDOW_H__
#define __CW32WINDOW_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <map>

#include <Memory/mmgr.h>
#include "IWindow.h"

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class CW32Window : public IWindow
{
   /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        CW32Window() {Clear();};
        
        virtual bool Create(HINSTANCE hInstance, const std::string &sTitle,
                            word wWidth, word wHeight, bool bFullscreen,
                            sword swXPos, sword swYPos, byte bBitsPerPixel);
        virtual void SetPos(sword swXPos, sword swYPos);
        virtual void SetSize(word wWidth, word wHeight);
        virtual void SetTitle(const std::string &sTitle);
        virtual void SetFullscreen(bool bFullscreen, word wWidth, word wHeight);
        virtual void Show(bool bShow);
        virtual bool IsCreated() const {return m_hWnd != 0;}
        virtual bool IsFullscreen() const {return m_bFullscreen;}
        virtual byte GetBitsPerPixel() const {return m_bBitsPerPixel;}
        virtual void GetSize(word &wWidth, word &wHeight) const {wWidth = m_wWidth; wHeight = m_wHeight;}
        virtual HWND GetWindowHandle() const {return m_hWnd;}
        virtual void Destroy();

        virtual ~CW32Window() {}
        
    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        virtual void Clear();
        void RestoreMode();
        static LRESULT CALLBACK StaticWindowProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);
                                            //Durch static trotzdem public
        LRESULT WindowProc(UINT msg, WPARAM wParam, LPARAM lParam);

        //Private copy constructor and assignment operator to prevent mistakes:
        CW32Window(const CW32Window &);
        CW32Window &operator =(const CW32Window &);
        
    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
        HWND m_hWnd;
        HINSTANCE m_hInstance;
        bool m_bDestroy;    //Should the window be destroyed after next update?

        //Window status flags
        bool m_bFullscreen;
        byte m_bBitsPerPixel;
        word m_wWidth, m_wHeight;
        word m_wDefaultWidth, m_wDefaultHeight;
        sword m_swDefaultXPos, m_swDefaultYPos;

        //Saving old display state:
        int m_iOldWidth, m_iOldHeight, m_iOldDepth, m_iOldRefresh;

        static bool sm_bWndClassRegistered;

        typedef std::map< HWND, CW32Window *> WindowMap;
        typedef WindowMap::value_type WindowMapValType;
        typedef WindowMap::iterator WindowMapIterator;
        static WindowMap sm_WindowMap;      //Durch static trotzdem public
};

#endif
