/****************************************************************************\
*                                                                            *
* 16.02.2006                                                                 *
* IWindow - Header File                                                      *
*                                                                            *
* (c) Raphael Charwot                                                        *
*                                                                            *
\****************************************************************************/

#ifndef __IWINDOW_H__
#define __IWINDOW_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>
#include <list>

#include <Memory/mmgr.h>
#include <Memory/TSmartPointer.h>

#include <Display/Windows/IWindowHandler.h>

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class IWindow : public IManagedObject
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        IWindow() {}

        virtual bool Create(HINSTANCE hInstance, const std::string &sTitle,
                            word wWidth = 640, word wHeight = 480, bool bFullscreen = false,
                            sword swXPos = 0, sword swYPos = 0, byte bBitsPerPixel = 32) = 0;

        virtual void SetPos(sword swXPos, sword swYPos) = 0;
        virtual void SetSize(word wWidth, word wHeight) = 0;
        virtual void SetTitle(const std::string &sTitle) = 0;
        virtual void SetFullscreen(bool bFullscreen, word wWidth, word wHeight) = 0;
        virtual void Show(bool bShow) = 0;

        virtual bool IsCreated() const = 0;
        virtual bool IsFullscreen() const = 0;
        virtual byte GetBitsPerPixel() const = 0;
        virtual void GetSize(word &wWidth, word &wHeight) const = 0;

        virtual HWND GetWindowHandle() const = 0; //Not beautiful, but neccessary
    
        virtual void Destroy()
        {
            m_WindowHandlers.clear();
        };

        virtual void RegisterHandler(IWindowHandler *pHandler)
        {
            m_WindowHandlers.push_back(pHandler);
        }

        virtual void UnregisterHandler(IWindowHandler *pHandler)
        {
            HandlerListIterator i(m_WindowHandlers.begin());

            while(i != m_WindowHandlers.end() && !((*i) == pHandler))
            {
                ++i;
            }
            if(i != m_WindowHandlers.end())
                m_WindowHandlers.erase(i);
        }

        virtual ~IWindow() {Destroy();};
        
    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        //Private copy constructor and assignment operator to prevent mistakes:
        IWindow(const IWindow &);
        IWindow &operator =(const IWindow &);

    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:
        typedef std::list<IWindowHandler *> HandlerList;
        typedef HandlerList::iterator HandlerListIterator;

        HandlerList m_WindowHandlers;
};

#endif
