/****************************************************************************\
*                                                                            *
* 15.03.2006                                                                 *
* CIniFile - Header File                                                     *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __CINIFILE_H__
#define __CINIFILE_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/

#include <Definitions.h>
#include <Memory/mmgr.h>

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/

struct Entry{
	std::string name;
	std::string value;
};
struct Category{
	std::string name;
	std::vector<Entry> entries;
};
typedef std::vector<Category> CategoryList;


//structs parser
struct AddCategory{
	CategoryList * m_file;
	void operator() (char const* str, char const* end) const
	{
		std::string name(str, end);
		Category cat;
		cat.name = name;
		m_file->push_back(cat);
	}
};
struct AddName{
	CategoryList * m_file;
	void operator() (char const* str, char const* end) const
	{
		std::string name(str, end);
		int last_cat_id = (int)m_file->size() - 1;
		if (last_cat_id == -1)
		{
			Category cat;
			cat.name = "Globals";
			m_file->push_back(cat);
			last_cat_id = 0;
		}
		Entry entry;
		entry.name = name;
		m_file->at(last_cat_id).entries.push_back(entry);
	}
};
struct AddValue{
	CategoryList * m_file;
	void operator() (char const* str, char const* end) const
	{
		std::string value(str, end);
		int last_cat_id = (int)m_file->size() - 1;
		int last_entry_id = (int)m_file->at(last_cat_id).entries.size() - 1;
		if (last_entry_id == -1)
		{
			Entry entry;
			entry.name = "";
			m_file->at(last_cat_id).entries.push_back(entry);
			last_entry_id = 0;
		};
		m_file->at(last_cat_id).entries[last_entry_id].value = value;
	}
};

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class CIniFile{

	/**************************************************************************\
	* PRIVATE CLASS MEMBERS                                                    *
	\**************************************************************************/
private:
	
	CategoryList m_file;
	int m_crt_category;
	int m_crt_entry;

	/**************************************************************************\
	* PRIVATE CLASS FUNCTIONS                                                  *
	\**************************************************************************/
private:
	
	int b_CategoryIsValid();
	int b_EntryIsValid();

	/**************************************************************************\
	* PUBLIC CLASS MEMBERS                                                     *
	\**************************************************************************/
public:
	
	int m_status;

	/**************************************************************************\
	* PUBLIC CLASS FUNCTIONS                                                   *
	\**************************************************************************/
public:
	
	CIniFile();
	void ClearAll();
	void LoadFile(std::string filename);
	void SaveToFile(std::string filename);

	int GetNumCategories();
	void SetCategory(int cat);
	int SetCategory(std::string name);
	std::string GetCategoryName();
	void AddCategoryUnique(std::string name);
	void AddCategory(std::string name);

	int GetNumEntries();
	void SetEntry(int entry);
	int SetEntry(std::string name);
	int SetEntryByValue(std::string value);

	std::string GetEntryName();
	std::string GetEntryValue();
	std::string GetEntryValueByName(std::string name);
	void AddEntryUnique(std::string name, std::string value);
	void AddEntry(std::string name, std::string value);
	void ChangeEntry(std::string name, std::string value);
	void SetValue(std::string value);

	void DeleteEntry(std::string name);
	void DeleteEntry(int entry);
	void DeleteByValue(std::string value);

	void Print();

};

#endif
