/****************************************************************************\
*                                                                            *
* 02.07.2003                                                                 *
* CStdFile - Source File                                                     *
*                                                                            *
* (c) Raphael Charwot                                                        *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CStdFile.h"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
bool CStdFile::Create(const std::string &sFileName, EAccess access, ECreation creation, ESharing sharing, dword dwFlags)
{
  assert(!IsOpen() && "Error: Trying to call CStdFile::Create on an open file!");

  m_hFileHandle = ::CreateFile(sFileName.c_str(), access, sharing, NULL, creation, dwFlags, NULL);
  
  return IsOpen();
}

dword CStdFile::GetSize() const
{
  assert(IsOpen() && "Error: CStdFile::GetSize called on a closed file!");
  return ::GetFileSize(m_hFileHandle, NULL);
}

bool CStdFile::Seek(sdword sdwOffset, ESeek seek)
{
  assert(IsOpen() && "Error: CStdFile::Seek called on a closed file!");
  return ::SetFilePointer(m_hFileHandle, sdwOffset, NULL, seek) != 0xFFFFFFFF;
}

dword CStdFile::GetSeekPos()
{
  assert(IsOpen() && "Error: CStdFile::GetSeekPos called on closed file!");
  return ::SetFilePointer(m_hFileHandle, 0, NULL, SEEK_FROM_CURRENT);
}

dword CStdFile::Read(void *pBuffer, dword dwSize) const
{
  assert(IsOpen() && "Error: CStdFile::Read called on a closed file!");

  if(dwSize)
  {
    dword dwRead = 0;
    if(::ReadFile(m_hFileHandle, pBuffer, dwSize, &dwRead, NULL))
    {
      return dwRead;
    }
    return 0;
  }
  return 1; //To prevent errors
}

dword CStdFile::Write(const void *pBuffer, dword dwSize)
{
  assert(IsOpen() && "Error: CStdFile::Write called on a closed file!");

  if(dwSize)
  {
    dword dwWritten = 0;
    return ::WriteFile(m_hFileHandle, pBuffer, dwSize, &dwWritten, NULL) && dwWritten;
  }
  return 1; //To prevent errors
}

dword CStdFile::Print(const char* sText, ...)
{
  if(sText && sText[0] != '\0')
  {
    va_list args;
    char sPrintText[1024];
    dword dwSize;

    va_start(args, sText);
    vsprintf(sPrintText, sText, args);

    dwSize = (dword) strlen(sPrintText);
    if(dwSize)
    {
      return Write((void *) sPrintText, dwSize);
    }
  }  
  return 0;
}

void CStdFile::Destroy()
{
  if(IsOpen())
  {
    ::CloseHandle(m_hFileHandle);
    m_hFileHandle = INVALID_HANDLE_VALUE;
  }
  Clear();
}