/****************************************************************************\
*                                                                            *
* 21.02.2006                                                                 *
* CDirectInputMgr - Header File                                              *
*                                                                            *
* (c) Raphael Charwot                                                        *
*                                                                            *
\****************************************************************************/

#ifndef __CDIRECTINPUTMGR_H__
#define __CDIRECTINPUTMGR_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <Memory/mmgr.h>
#include <Display/Windows/IWindowHandler.h>
#include "IInputMgr.h"

#pragma comment(lib, "dinput8.lib")
#pragma comment(lib, "dxguid.lib")

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/
struct IDirectInput8;
struct IDirectInputDevice8;

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class CDirectInputMgr : public IInputMgr, public IWindowHandler
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        CDirectInputMgr() {Clear();}
        
        virtual bool Create(HINSTANCE hInstance, const TSmartPointer<IWindow> &pWindow);
        virtual void Update();
        virtual void GetCursorPos(sdword &sdwPosX, sdword &sdwPosY) const;
        virtual void GetNormalizedCursorPos(float &fPosX, float & fPosY) const;
        virtual void LockCursor(bool bLock);
        virtual void ShowCursor(bool bShow);
        virtual void Destroy();
        
        virtual ~CDirectInputMgr() {Destroy();}

        virtual void OnWndChar(char c);             //from IWindowHandler. Used to get all printable chars.
                                                    //easier than with DI
		virtual void OnWndActivate(bool bActive);	//from IWindowHandler. Used to re-lock the mouse.
        
    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
        void UpdateMouse();
        void UpdateKeyboard();

        bool VerifyMouse();
        bool VerifyKeyboard();
        
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        void Clear();
        //Private copy constructor and assignment operator to prevent mistakes:
        CDirectInputMgr(const CDirectInputMgr &);
        CDirectInputMgr &operator =(const CDirectInputMgr &);
        
    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
        IDirectInput8       *m_pDI;                       //Needed for DirectInput
        IDirectInputDevice8 *m_pDIMouse;                  //The mouse
        IDirectInputDevice8 *m_pDIKeyboard;               //The keyboard

        TSmartPointer<IWindow> m_pWindow;                //The window for which we are receiving input

        bool m_bMouseStatus[8];                         //Holds current mouse status (true = pressed, false = normal)
        bool m_bKeyboardStatus[256];                    //Holds current keyboard status
        bool m_bVerifyMouse;                            //Should the current status of the mouse device be verified?
        bool m_bVerifyKeyboard;                         //Should the current status of the keyboard device be verified?

        bool m_bLockCursor;
        bool m_bShowCursor;
};

#endif
