/****************************************************************************\
*                                                                            *
* 15.04.2006                                                                 *
* CTerrainBuilder - Header File                                              *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __CTERRAINBUILDER_H__
#define __CTERRAINBUILDER_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <Memory/mmgr.h>
#include <Memory/IManagedObject.h>
#include <Memory/TSmartPointer.h>

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/
class ITextureLoader;
class CGeometryInfo;
struct SGeoGroup;
class CTerrainInfo;

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class CTerrainBuilder : public IManagedObject
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        CTerrainBuilder(const TSmartPointer<ITextureLoader> &pTextureLoader) {m_pTextureLoader = pTextureLoader;}
        
        bool BuildTerrain(const std::string &sLevelName, CTerrainInfo *pTerrainInfo,
                          std::vector< TSmartPointer<CGeometryInfo> > &geometryInfos) const;
                                                            //Creates the terrain, stores the height info in terrainInfo
                                                            //and the terrain parts in the vector
        
        virtual ~CTerrainBuilder() {m_pTextureLoader = NULL;}
        
    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        //Private copy constructor and assignment operator to prevent mistakes:
        CTerrainBuilder(const CTerrainBuilder &);
        const CTerrainBuilder &operator =(const CTerrainBuilder &);
        
    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
        TSmartPointer<ITextureLoader> m_pTextureLoader;
        
};

#endif
