/****************************************************************************\
*                                                                            *
* 19.02.2006                                                                 *
* CMatrix44 - Header File                                                     *
*                                                                            *
* (c) Raphael Charwot                                                        *
*                                                                            *
* If you are wondering about all the consts and references, read             *
* "Effective C++" and "More Effective C++" by Scott Meyers                   *
\****************************************************************************/

#ifndef __CMATRIX44_H__
#define __CMATRIX44_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>
#include "TVector3.h"
#include "TVector4.h"

/****************************************************************************\
* DEFINITIONS	                                                               *
\****************************************************************************/

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
#pragma pack(push, 1)    //Ensures that the floats are tightly packed
class CMatrix44
{
  /**************************************************************************\
  * PUBLIC CLASS FUNCTIONS                                                   *
  \**************************************************************************/
  public:
    CMatrix44() {};
    CMatrix44(const float &f);
    CMatrix44(const float &f11, const float &f12, const float &f13, const float &f14,
              const float &f21, const float &f22, const float &f23, const float &f24,
              const float &f31, const float &f32, const float &f33, const float &f34,
              const float &f41, const float &f42, const float &f43, const float &f44);
    CMatrix44(const CMatrix44 &other);

    const CMatrix44 &operator=(const CMatrix44 &other);
    const CMatrix44 operator*(const CMatrix44 &other) const; //target.Multiply(this, other) is most likely faster
    const CMatrix44 &operator*=(const CMatrix44 &other); //Slower than target.Multiply(this, other), so use the faster one, if possible!
                                                         //Edit: Still true? Benchmark necessary
    const CVector3f operator*(const CVector3f &v) const; //"Dirty"-Multiplication (ignoring the last matrix row)

    bool operator==(const CMatrix44 &other) const;
    bool operator!=(const CMatrix44 &other) const;

    void Set(const float &f);
    void Set(const float &f11, const float &f12, const float &f13, const float &f14,
             const float &f21, const float &f22, const float &f23, const float &f24,
             const float &f31, const float &f32, const float &f33, const float &f34,
             const float &f41, const float &f42, const float &f43, const float &f44);

	float Determinant();

    void Identity();
    void Invert();
    void Transpose();
    void TransposeFrom(const CMatrix44 &m);

    //Don't use "first.Multiply(first, second)" or "second.Multiply(first, second)" This gives wrong results!
    //Use "first *= second" instead
    void Multiply(const CMatrix44 &first, const CMatrix44 &second); //Matrix contains "first" x "second" --> "second" is applied before "first"

    void Scaling(const float &fX, const float &fY, const float &fZ);
    void Scaling(const CVector3f &vScale);
    void Translation(const float &fX, const float &fY, const float &fZ);
    void Translation(const CVector3f &vTranslation);
    void RotationX(const float &fRad);
    void RotationY(const float &fRad);
    void RotationZ(const float &fRad);
	//ToDo: Pfusch! Msste eigentlich YXZ sein!! (Inverse Rotationsreihenfolge fr die Kamera!)
    void RotationZYX(const float &fX, const float &fY, const float &fZ);
    void RotationZYX(const CVector3f &vRotation);
	void RotationZXY(const float &fX, const float &fY, const float &fZ);
	void RotationZXY(const CVector3f &vRotation);
	const CVector3f GetZXYRotationVector();
    void Orthogonal(const float &fWidth, const float &fHeight, const float &fNearZ, const float &fFarZ);
    void Perspective(const float &fFov, const float &fAspectRatio, const float &fNearZ, const float &fFarZ);

    void Log(word wTargetLog) const;

    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
    
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:

    /**************************************************************************\
    * PUBLIC CLASS MEMBERS                                                     *
    \**************************************************************************/
    public:
        //Column major fr OpenGL (grrrr), dies betrifft aber _nur_ die Ausrichtung im SPEICHER
        //Logisch liegen die Elemente trotzdem row major (wie in der Mathematik blich)
        struct
        {
            float _11, _21, _31, _41;
            float _12, _22, _32, _42;
            float _13, _23, _33, _43;
            float _14, _24, _34, _44;
        };
/*
        //Row major, so wie es sich gehrt (seufz)
        union
        {
            struct
            {
                float _11, _12, _13, _14;
                float _21, _22, _23, _24;
                float _31, _32, _33, _34;
                float _41, _42, _43, _44;
            };
            float m[4][4];
        };
*/
};
#pragma pack(pop)

CVector4f operator*(const CVector4f &v, const CMatrix44 &m);
CVector4f &operator*=(CVector4f &v, const CMatrix44 &m);

#endif