/****************************************************************************\
*                                                                            *
* 28.03.2006                                                                 *
* TAABox3 - Header File                                                        *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __TAABox3_H__
#define __TAABox3_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <Memory/mmgr.h>
#include "TVector3.h"
#include "TBox3.h"

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
template<typename T>
class TAABox3
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        TAABox3() {}
        TAABox3(const T &minX, const T &minY, const T &minZ,
              const T &maxX, const T &maxY, const T &maxZ) : m_MinX(minX), m_MinY(minY), m_MinZ(minZ),
                                                             m_MaxX(maxX), m_MaxY(maxY), m_MaxZ(maxZ) {}
        TAABox3<T>(const TAABox3<T> &other) : m_MinX(other.m_MinX), m_MinY(other.m_MinY), m_MinZ(other.m_MinZ),
                                          m_MaxX(other.m_MaxX), m_MaxY(other.m_MaxY), m_MaxZ(other.m_MaxZ) {}
        const TAABox3<T> &operator =(const TAABox3<T> &other) {m_MinX = other.m_MinX; m_MinY = other.m_MinY; m_MinZ = other.m_MinZ;
                                                           m_MaxX = other.m_MaxX; m_MaxY = other.m_MaxY; m_MaxZ = other.m_MaxZ; return *this;}

        const TAABox3<T> operator+(const TVector3<T> &vec) const {return TAABox3<T>(m_MinX + vec.m_X, m_MinY + vec.m_Y, m_MinZ + vec.m_Z,
                                                                          m_MaxX + vec.m_X, m_MaxY + vec.m_Y, m_MaxZ + vec.m_Z);}
        const TAABox3<T> &operator+=(const TVector3<T> &vec) {m_MinX += vec.m_X; m_MaxX += vec.m_X; m_MinY += vec.m_Y; m_MaxY += vec.m_Y;
                                                            m_MinZ += vec.m_Z; m_MaxZ += vec.m_Z; return *this;}

        const TAABox3<T> operator*(const TVector3<T> &vec) const {return TAABox3<T>(m_MinX * vec.m_X, m_MinY * vec.m_Y, m_MinZ * vec.m_Z,
                                                                          m_MaxX * vec.m_X, m_MaxY * vec.m_Y, m_MaxZ * vec.m_Z);}
        const TAABox3<T> &operator*=(const TVector3<T> &vec) {m_MinX *= vec.m_X; m_MaxX *= vec.m_X; m_MinY *= vec.m_Y; m_MaxY *= vec.m_Y;
                                                            m_MinZ *= vec.m_Z; m_MaxZ *= vec.m_Z; return *this;}
        const CBox3f GetRotatedBox(const CVector3f &vRotation) const;

        void SetSafePointWidth(const T &x, const T &y, const T &z,      //makes sure that min < max
                               const T &xWidth, const T &yWidth, const T &zWidth);
        void SetSafePointPoint(const T &x1, const T &y1, const T &z1,      //makes sure that min < max
                               const T &x2, const T &y2, const T &z2);

        T SizeX() const {return m_MaxX - m_MinX;}
        T SizeY() const {return m_MaxY - m_MinY;}
        T SizeZ() const {return m_MaxZ - m_MinZ;}
        const TVector3<T> Size() const {return TVector3<T>(m_MaxX - m_MinX, m_MaxY - m_MinY, m_MaxZ - m_MinZ);}
		const TVector3<T> Center() const {return TVector3<T>((m_MaxX + m_MinX) / 2.0f, (m_MaxY + m_MinY) / 2.0f, (m_MaxZ + m_MinZ) / 2.0f);}

        //More to come (intersection tests, etc.)
        
        
    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        
    /**************************************************************************\
    * PUBLIC CLASS MEMBERS                                                  *
    \**************************************************************************/
    public:
        T m_MinX, m_MinY, m_MinZ,
          m_MaxX, m_MaxY, m_MaxZ;
};


template<typename T>
const CBox3f TAABox3<T>::GetRotatedBox(const CVector3f &vRotation) const
{
    CMatrix44 mat;
    mat.RotationZXY(vRotation);

    return CBox3f(mat * CVector3f(m_MinX, m_MinY, m_MinZ), mat * CVector3f(m_MaxX - m_MinX, 0.0f, 0.0f),
                  mat * CVector3f(0.0f, m_MaxY - m_MinY, 0.0f), mat * CVector3f(0.0f, 0.0f, m_MaxZ - m_MinZ));
}

template<typename T>
void TAABox3<T>::SetSafePointWidth(const T &x, const T &y, const T &z,
                                 const T &xWidth, const T &yWidth, const T &zWidth)
{
    if(xWidth >= 0)
    {
        m_MinX = x;
        m_MaxX = x + xWidth;
    }
    else
    {
        m_MaxX = x;
        m_MinX = x + xWidth;
    }

    if(yWidth >= 0)
    {
        m_MinY = y;
        m_MaxY = y + yWidth;
    }
    else
    {
        m_MaxY = y;
        m_MinY = y + yWidth;
    }

    if(zWidth >= 0)
    {
        m_MinZ = z;
        m_MaxZ = z + zWidth;
    }
    else
    {
        m_MaxZ = z;
        m_MinZ = z + zWidth;
    }
}

template<typename T>
void TAABox3<T>::SetSafePointPoint(const T &x1, const T &y1, const T &z1,
                                 const T &x2, const T &y2, const T &z2)
{
    if(x2 >= x1)
    {
        m_MinX = x1;
        m_MaxX = x2;
    }
    else
    {
        m_MaxX = x1;
        m_MinX = x2;
    }

    if(y2 >= y1)
    {
        m_MinY = y1;
        m_MaxY = y2;
    }
    else
    {
        m_MaxY = y1;
        m_MinY = y2;
    }

    if(z2 >= z1)
    {
        m_MinZ = z1;
        m_MaxZ = z2;
    }
    else
    {
        m_MaxZ = z1;
        m_MinZ = z2;
    }
}

typedef TAABox3<float> CAABox3f;

#endif
