/****************************************************************************\
*                                                                            *
* 22.04.2006                                                                 *
* TSphere - Header File                                                      *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __TSPHERE_H__
#define __TSPHERE_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <Memory/mmgr.h>

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/
#ifndef min
    #define min(x,y) ((x) < (y) ? (x) : (y))
#endif

#ifndef max
    #define max(x,y) ((x) > (y) ? (x) : (y))
#endif

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
template<typename T>
class TSphere
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        TSphere() {}
        TSphere(CVector3f vCenter, T radius) : m_vCenter(vCenter), m_Radius(radius) {}
        TSphere(const TSphere &other) : m_vCenter(other.m_vCenter), m_Radius(other.m_Radius) {}
        const TSphere &operator =(const TSphere &other) {m_vCenter = other.m_vCenter; m_Radius = other.m_Radius; return *this;}

        const TSphere<T> operator+(const TVector3<T> &vec) const {return TSphere<T>(m_vCenter + vec, m_Radius);}
        const TSphere<T> &operator+=(const TVector3<T> &vec) {m_Center += vec; return *this;}

        const TSphere<T> operator*(const TVector3<T> &vec) const {return TSphere<T>(m_vCenter, m_Radius * max(vec.m_X, max(vec.m_Y, vec.m_Z)));}
        const TSphere<T> &operator*=(const TVector3<T> &vec) {m_Radius *= max(vec.m_X, max(vec.m_Y, vec.m_Z)); return *this;}
        const TSphere<T> operator*(const T &value) const {return TSphere<T>(m_vCenter, m_Radius * value);}
        const TSphere<T> &operator*=(const T &value) {m_Radius *= value; return *this;}

        const TSphere<T> GetRotatedSphere(const CVector3f &vRotation) const;
        bool RayIntersection(const CVector3f &vRayOrigin, const CVector3f &vRayDir, float &fDistance) const;
		bool Intersects(const TSphere<T> &other) const;

        void Log(word wTargetLog) const {LOG(wTargetLog, "Center: (%.2f, %.2f, %.2f), R: %.2f",
                                         (float) m_vCenter.m_X, (float) m_vCenter.m_Y, (float) m_vCenter.m_Z, (float) m_Radius);}

    /**************************************************************************\
    * PUBLIC CLASS MEMBERS                                                     *
    \**************************************************************************/
    public:
        TVector3<T> m_vCenter;
        T           m_Radius;
        
};

typedef TSphere<float> CSpheref;

template<typename T>
const TSphere<T> TSphere<T>::GetRotatedSphere(const CVector3f &vRotation) const
{
    CMatrix44 mat;
    mat.RotationZXY(vRotation);

    return TSphere<T>(mat * m_vCenter, m_Radius);
}

template<typename T>
bool TSphere<T>::RayIntersection(const CVector3f &vRayOrigin, const CVector3f &vRayDir, float &fDistance) const
{
    CVector3f eo(m_vCenter - vRayOrigin);

    float v = eo.Dot(vRayDir);
    float disc = m_Radius * m_Radius - (eo.Dot(eo) - v * v);
    if(disc < 0.0f)
        return false;
    
    fDistance = v - sqrt(disc);
	return true;
/*
    CVector3f dst(vRayOrigin - m_vCenter);
    float B = dst.Dot(vRayDir);
    float C = dst.Dot(dst) - m_Radius;
    float D = B*B - C;
    return D > 0 ? -B - sqrt(D) : -1.0f;
*/
}

template<typename T>
bool TSphere<T>::Intersects(const TSphere<T> &other) const
{
	return (other.m_vCenter - m_vCenter).Length() < other.m_Radius + m_Radius;
}

#endif
