/****************************************************************************\
*                                                                            *
* 19.02.2006                                                                 *
* TVector4 - Header File                                                     *
*                                                                            *
* (c) Raphael Charwot                                                        *
*                                                                            *
* If you are wondering about all the consts and references, read             *
* "Effective C++" and "More Effective C++" by Scott Meyers                   *
\****************************************************************************/

#ifndef __TVECTOR4_H__
#define __TVECTOR4_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <Memory/mmgr.h>
#include <math.h>

/****************************************************************************\
* DEFINITIONS	                                                               *
\****************************************************************************/

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
template<typename T>
struct TVector4
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        TVector4() {}     //No initialization for speed purposes
        TVector4(const T &x, const T &y, const T &z, const T &w) {m_X = x; m_Y = y; m_Z = z; m_W = w;}
        TVector4(const TVector4<T> &other) {m_X = other.m_X; m_Y = other.m_Y; m_Z = other.m_Z; m_W = other.m_W;}

        const TVector4<T> &operator=(const TVector4<T> &other) {m_X = other.m_X; m_Y = other.m_Y; m_Z = other.m_Z; m_W = other.m_W; return *this;}

        const TVector4<T> operator+(const TVector4<T> &other) const {return TVector4<T>(m_X + other.m_X, m_Y + other.m_Y, m_Z + other.m_Z, m_W + other.m_W);}
        const TVector4<T> &operator+=(const TVector4<T> &other) {m_X += other.m_X; m_Y += other.m_Y; m_Z += other.m_Z; m_W += other.m_W; return *this;}

        const TVector4<T> operator-(const TVector4<T> &other) const {return TVector4<T>(m_X - other.m_X, m_Y - other.m_Y, m_Z - other.m_Z, m_W - other.m_W);}
        const TVector4<T> &operator-=(const TVector4<T> &other) {m_X -= other.m_X; m_Y -= other.m_Y; m_Z -= other.m_Z; m_W -= other.m_W; return *this;}

        const TVector4<T> operator*(const T &factor) const {return TVector4<T>(m_X * factor, m_Y * factor, m_Z * factor, m_W * factor);}
        const TVector4<T> &operator*=(const T &factor) {m_X *= factor; m_Y *= factor; m_Z *= factor; m_W *= factor; return *this;}

        const TVector4<T> operator/(const T &quot) const {return TVector4<T>(m_X / quot, m_Y / quot, m_Z / quot, m_W / quot);}
        const TVector4<T> &operator/=(const T &quot) {m_X /= quot; m_Y /= quot; m_Z /= quot; m_W /= quot; return *this;}

        bool operator==(const TVector4<T> &other) {return m_X == other.m_X && m_Y == other.m_Y && m_Z == other.m_Z && m_W == other.m_W;}
        bool operator!=(const TVector4<T> &other) {return m_X != other.m_X || m_Y != other.m_Y || m_Z != other.m_Z || m_W != other.m_W;}

        T &operator[](const byte &i) {assert(i < 4 && "Error: Invalid index for TVector4::operator[]!");
                                    return *(&m_X + i);}

        const T &operator[](const byte &i) const {assert(i < 4 && "Error: Invalid index for TVector4::operator[]!");
                                                return *(&m_X + i);}

        void Set(const T &x, const T &y, const T &z, const T &w) {m_X = x; m_Y = y; m_Z = z; m_W = w;}

        const float Length() const {return sqrt(m_X * m_X + m_Y * m_Y + m_Z * m_Z + m_W * m_W);}
        const float LengthSqr() const {return m_X * m_X + m_Y * m_Y + m_Z * m_Z + m_W * m_W;}

        void Normalize() {assert(LengthSqr() && "Error: TVector4::Normalize called for a zero vector!");
                        float fL = 1 / Length(); m_X *= fL; m_Y *= fL; m_Z *= fL; m_W *= fL;}

        const T Dot(const TVector4<T> &other) const {return m_X * other.m_X + m_Y * other.m_Y + m_Z * other.m_Z + m_W * other.m_W;}
		const TVector4<T> ElementMultiply(const TVector4<T> &other) const {return TVector4<T>(m_X * other.m_X, m_Y * other.m_Y, m_Z * other.m_Z, m_W * other.m_W);}

        void Log(word wTargetLog) const {LOG(wTargetLog, "(%.2f, %.2f, %.2f, %.2f)", m_X, m_Y, m_Z, m_W);}

    /**************************************************************************\
    * PUBLIC CLASS MEMBERS                                                     *
    \**************************************************************************/
    public:
        T m_X, m_Y, m_Z, m_W;
};

typedef TVector4<float> CVector4f;
typedef TVector4<sword> CVector4sw;
typedef TVector4<sdword> CVector4sdw;

#endif
