/****************************************************************************\
*                                                                            *
* 18.06.2003                                                                 *
* CLogSystem - Source File                                                   *
*                                                                            *
* (c) Raphael Charwot                                                        *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <stdarg.h>
#include "CLogSystem.h"
#include <FileSystem/CStdFile.h>

/****************************************************************************\
* PRIVATE DEFINITIONS	                                                       *
\****************************************************************************/
#define ERROR_LOGFILE   "Error.log"
#define DEBUG_LOGFILE   "Debug.log"

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
void CLogSystem::ClearLogs()
{
  remove(ERROR_LOGFILE);
  remove(DEBUG_LOGFILE);
}

void CLogSystem::Log(word wTargetLog, const char* sText, ...)
{
  if(sText && sText[0] != '\0')
  {
    va_list args;
    CStdFile file;
    char sLogText[1024];
    dword dwSize;

    va_start(args, sText);
    vsprintf(sLogText, sText, args);

    dwSize = (dword) strlen(sLogText);
    if(dwSize)
    {
      if(wTargetLog & TL_ERROR)
      {
        file.Create(ERROR_LOGFILE, ACCESS_WRITE_ONLY, CREATION_OPEN_ALWAYS);
        file.Seek(0, SEEK_FROM_END);
        file.Write((void *) sLogText, dwSize);
        file.Write((void *) "\n", 1);
        file.Destroy();
      }
    #ifdef _DEBUG
      if(wTargetLog & TL_DEBUG)
      {
        file.Create(DEBUG_LOGFILE, ACCESS_WRITE_ONLY, CREATION_OPEN_ALWAYS);
        file.Seek(0, SEEK_FROM_END);
        file.Write((void *) sLogText, dwSize);
        file.Write((void *) "\n", 1);
        file.Destroy();
      }
    #endif
      if(wTargetLog & TL_USER && dwSize)
      {
        MessageBox(NULL, sLogText, "Information", MB_OK | MB_SETFOREGROUND | MB_TOPMOST | MB_TASKMODAL);
      }
      if(wTargetLog & TL_STDOUT || wTargetLog & TL_DEBUG)
      {
        fprintf(stdout, "%s\n", sLogText);
      }
      if(wTargetLog & TL_STDERR || wTargetLog & TL_ERROR)
      {
          fprintf(stderr, "%s\n", sLogText);
      }
    }
  }
}