/****************************************************************************\
*                                                                            *
* 30.07.2005                                                                 *
* Functors - Header File                                                     *
*                                                                            *
* (c) Raphael Charwot                                                        *
*                                                                            *
\****************************************************************************/

#ifndef __FUNCTORS_H__
#define __FUNCTORS_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>
#include <Memory/mmgr.h>
#include <Memory/IManagedObject.h>
#include <Memory/TSmartPointer.h>

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class IFunctor : public IManagedObject
{
  /**************************************************************************\
  * PUBLIC CLASS FUNCTIONS                                                   *
  \**************************************************************************/
  public:
    virtual dword operator ()() = 0;

    virtual ~IFunctor() {};
};

template <class T>
class TFunctor : public IFunctor
{
    typedef dword (T::*FuncType)();

    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        TFunctor(T *o, FuncType f) {assert(o && f && "Error: TFunctor construced with NULL pointer(s)");
                                    m_pObj = o; m_pFunc = f;}

        dword operator ()() {return (m_pObj->*m_pFunc)();}

        virtual ~TFunctor() {m_pObj = NULL;}

    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        //Private copy constructor and assignment operator to prevent mistakes:
        TFunctor(const TFunctor &);
        TFunctor &operator =(const TFunctor &);

    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
        T *m_pObj;
        FuncType m_pFunc;
};

template <class T>
class TManagedFunctor : public IFunctor
{
    typedef dword (T::*FuncType)();

    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        TManagedFunctor(T *o, FuncType f) {assert(o && f && "Error: TManagedFunctor construced with NULL pointer(s)");
                                           m_pObj = o; m_pFunc = f;}

        int operator ()() {return (m_pObj->*m_pFunc)();}

        virtual ~TManagedFunctor() {m_pObj = NULL;}

    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        //Private copy constructor and assignment operator to prevent mistakes:
        TManagedFunctor(const TManagedFunctor &);
        TManagedFunctor &operator =(const TManagedFunctor &);

    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
        TSmartPointer<T> m_pObj;
        FuncType m_pFunc;
};

class IParamFunctor : public IManagedObject
{
  /**************************************************************************\
  * PUBLIC CLASS FUNCTIONS                                                   *
  \**************************************************************************/
  public:
    virtual dword operator ()(dword dwValue) = 0;

    virtual ~IParamFunctor() {};
};

template <class T>
class TParamFunctor : public IParamFunctor
{
    typedef dword (T::*FuncType)(dword);

    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        TParamFunctor(T *o, FuncType f) {assert(o && f && "Error: TParamFunctor construced with NULL pointer(s)");
                                         m_pObj = o; m_pFunc = f;}

        dword operator ()(dword dwValue) {return (m_pObj->*m_pFunc)(dwValue);}

        virtual ~TParamFunctor() {m_pObj = NULL;}

    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        //Private copy constructor and assignment operator to prevent mistakes:
        TParamFunctor(const TParamFunctor &);
        TParamFunctor &operator =(const TParamFunctor &);

    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
        T *m_pObj;
        FuncType m_pFunc;
};

template <class T>
class TManagedParamFunctor : public IParamFunctor
{
    typedef dword (T::*FuncType)(dword);

    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        TManagedParamFunctor(T *o, FuncType f) {assert(o && f && "Error: TManagedParamFunctor construced with NULL pointer(s)");
                                                m_pObj = o; m_pFunc = f;}

        int operator ()(dword dwValue) {return (m_pObj->*m_pFunc)(dwValue);}

        virtual ~TManagedParamFunctor() {m_pObj = NULL;}

    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        //Private copy constructor and assignment operator to prevent mistakes:
        TManagedParamFunctor(const TManagedParamFunctor &);
        TManagedParamFunctor &operator =(const TManagedParamFunctor &);

    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
        TSmartPointer<T> m_pObj;
        FuncType m_pFunc;
};
#endif
