/****************************************************************************\
*                                                                            *
* 01.06.2006                                                                 *
* COGLCGProgram - Header File                                               *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __COGLCGPROGRAM_H__
#define __COGLCGPROGRAM_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>
#include <Memory/mmgr.h>
#include <Memory/IManagedObject.h>
#include <Math/TVector3.h>
#include <Math/TVector2.h>
#include <Math/TVector4.h>
#include <glew.h>
#include <CG/cg.h>
#include <CG/cgGL.h>
#include <map>


/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/

class COGLTriangleGroup;
class COGLRenderer;

enum EProgramType{
	ePROGRAM_VERTEX,
	ePROGRAM_FRAGMENT
};

struct SProgramInfo{

	SProgramInfo(){m_referenceCount = 0;}

	bool Release(){
		if(!--m_referenceCount){
			cgDestroyProgram(m_Program);
			return true;
		}
		return false;
	}

	void IncReference(){m_referenceCount++;}

	std::string      m_fileName;
	EProgramType     m_eType;
	CGprogram        m_Program;
	CGprofile        m_Profile;
	int              m_referenceCount;
};

enum EShaderType{
	eSHADER_VERTEX,
	eSHADER_FRAGMENT,
	eSHADER_VERTEX_FRAGMENT,
	eSHADER_NOT_SET
};

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class COGLCGProgram : public IManagedObject
{
    /**************************************************************************\
	* PUBLIC CLASS FUNCTIONS                                                   *
	\**************************************************************************/
public:
	COGLCGProgram(COGLTriangleGroup *pTri, COGLRenderer *pRenderer);

	virtual void SetupAndBind() = 0;
	virtual void Unbind() = 0;

	SProgramInfo const * const getVertexProgramInfo(){return m_pVertexProgInfo;}
	SProgramInfo const * const getFragmentProgramInfo(){return m_pFragmentProgInfo;}

	virtual ~COGLCGProgram();

	/**************************************************************************\
	* PROTECTED CLASS FUNCTIONS                                                *
	\**************************************************************************/
protected:

	SProgramInfo * CompileProgram(EProgramType eType,const std::string & fileName,const std::string & entryFunc);
	void CalculateVertexTangents(dword vertexCount,dword *pIndices,CVector3f *pCoords,CVector3f * pNormals,CVector2f * pUvs,CVector4f * pTangent);
		
	virtual void Destroy(){}

	/**************************************************************************\
	* PRIVATE CLASS FUNCTIONS                                                  *
	\**************************************************************************/
private:
	//Private copy constructor and assignment operator to prevent mistakes:
	COGLCGProgram(const COGLCGProgram &);
	COGLCGProgram &operator =(const COGLCGProgram &);

	/**************************************************************************\
	* PROTECTED CLASS MEMBERS                                                  *
	\**************************************************************************/
protected:

	EShaderType         m_shaderType;
	SProgramInfo *		m_pVertexProgInfo;
	SProgramInfo *      m_pFragmentProgInfo;
	COGLRenderer *      m_pRenderer;
	COGLTriangleGroup * m_pGeometry;

	/**************************************************************************\
	* PRIVATE CLASS MEMBERS                                                    *
	\**************************************************************************/
private:
	
	static std::map<std::string,SProgramInfo *> s_VertexPrograms;
	static std::map<std::string,SProgramInfo *> s_FragmentPrograms;
	static CGcontext                            s_context;
};

#endif