/****************************************************************************\
*                                                                            *
* 13.06.2006                                                                 *
* COGLGlobalRenderSettings - Source File                                     *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CEngineFactory.h"
#include <Renderer/COGLGlobalRenderSettings.h>
#include <Renderer/COGLRenderer.h>
#include <Renderer//COGLModelFactory.h>
#include <gl/gl.h>

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
COGLGlobalRenderSettings::COGLGlobalRenderSettings(IRenderer *pRenderer){

	m_pRenderer = (COGLRenderer *)pRenderer;

	m_globalDrawColor.Set(1.0f,1.0f,1.0f,1.0f);

	m_GlobalRenderState = m_StoredGlobalRenderState = 0xFFFFFFFF; 
	m_bGlobalRenderStateIsActive = false; 

	m_GlobalTextureState = OGL_TQ_MIPMAPPING | OGL_TQ_TRILINEAR;
	m_bGlobalTextureStateIsActive = false;

	m_RenderModeOverride = OGL_RENDER_MODE_MEGASLOW;
	m_bGlobalRenderModeStateIsActive = false;

	m_CacheModeOverride = OGL_CACHE_MODE_DISABLED;
	m_bGlobalCacheModeStateIsActive = false;

	m_bUseCurrentOverride = false;
}

bool COGLGlobalRenderSettings::IsGlobalRenderStateSet(){
	return m_bGlobalRenderStateIsActive || m_bGlobalTextureStateIsActive;
}

void COGLGlobalRenderSettings::DisableAllRenderFeatures(){
	m_GlobalRenderState = 0; 
	m_bGlobalRenderStateIsActive = true;
}

void COGLGlobalRenderSettings::EnableAllRenderFeatures(){
	m_GlobalRenderState  = 0xFFFFFFFF; 
	m_bGlobalRenderStateIsActive = false;
}

void COGLGlobalRenderSettings::StoreGlobalRenderState(){
	m_StoredGlobalRenderState = m_GlobalRenderState;
}

void COGLGlobalRenderSettings::RestoreGlobalRenderState(){
	m_GlobalRenderState = m_StoredGlobalRenderState;
	m_bGlobalRenderStateIsActive = ~m_GlobalRenderState > 0;
}

void COGLGlobalRenderSettings::EnableVertexColors(bool bEnable){
	if(bEnable){
		m_GlobalRenderState |= OGL_RENDER_STATE_USE_VERTEXCOLORS;
	}else{
		m_GlobalRenderState &= ~OGL_RENDER_STATE_USE_VERTEXCOLORS;
	}
	m_bGlobalRenderStateIsActive = ~m_GlobalRenderState > 0;
}

void COGLGlobalRenderSettings::EnableLighting(bool bEnable){
	if(bEnable){
		m_GlobalRenderState |= OGL_RENDER_STATE_USE_LIGHTING;
	}else{
		m_GlobalRenderState &= ~OGL_RENDER_STATE_USE_LIGHTING;
	}
	m_bGlobalRenderStateIsActive = ~m_GlobalRenderState > 0;
}

void COGLGlobalRenderSettings::EnableTexturing(bool bEnable){
	if(bEnable){
		m_GlobalRenderState |= OGL_RENDER_STATE_USE_TEXTURING;
	}else{
		m_GlobalRenderState &= ~OGL_RENDER_STATE_USE_TEXTURING;
	}
	m_bGlobalRenderStateIsActive = ~m_GlobalRenderState > 0;
}

void COGLGlobalRenderSettings::EnableShader(bool bEnable){
	if(bEnable){
		m_GlobalRenderState |= OGL_RENDER_STATE_USE_SHADER;
	}else{
		m_GlobalRenderState &= ~OGL_RENDER_STATE_USE_SHADER;
	}
	m_bGlobalRenderStateIsActive = ~m_GlobalRenderState > 0;
}

void COGLGlobalRenderSettings::SetMipMappingOverride(bool bEnable){
	if(bEnable){
		m_GlobalTextureState |= OGL_TQ_MIPMAPPING;
		((COGLModelFactory*)(IModelFactory*)CEngineFactory::InstancePtr()->GetModelFactory())->SetGlobalMipMapping(true);
	}else{
		m_GlobalTextureState &= ~OGL_TQ_MIPMAPPING;
		m_bGlobalTextureStateIsActive = true;
		((COGLModelFactory*)(IModelFactory*)CEngineFactory::InstancePtr()->GetModelFactory())->SetGlobalMipMapping(false);
	}
}

void COGLGlobalRenderSettings::SetTextureQualityOverride(EOGLTextureQuality eQuality,bool bEnable){
	if(bEnable){
		if(eQuality <= OGL_TQ_TRILINEAR){
			dword mipMapping = m_GlobalTextureState & OGL_TQ_MIPMAPPING; 
			m_GlobalTextureState = eQuality | mipMapping;
		 	m_bGlobalTextureStateIsActive = true;
			((COGLModelFactory*)(IModelFactory*)CEngineFactory::InstancePtr()->GetModelFactory())->SetGlobalTextureQuality(eQuality);
		}
	}else{
		dword mipMapping = m_GlobalTextureState & OGL_TQ_MIPMAPPING;
		m_GlobalTextureState = 0 | mipMapping;
		m_bGlobalTextureStateIsActive = (m_GlobalTextureState == 0);
	}
}

bool COGLGlobalRenderSettings::SetRenderModeOverride(EOGLRenderMode mode, bool bEnable){

	if(bEnable){

		//this override can only be set, when not in mixed or execute mode - just when cachmode override disables all displaylists
		if((mode == OGL_RENDER_MODE_VERTEXBUFFEROBJECT || mode == OGL_RENDER_MODE_UNKNOWN) &&
			!(m_bGlobalCacheModeStateIsActive && m_CacheModeOverride == OGL_CACHE_MODE_DISABLED))
		{
			return false;
		}

		if(mode < OGL_RENDER_MODE_UNKNOWN){
			
			if(m_bGlobalCacheModeStateIsActive){

				switch(m_CacheModeOverride){
					//Compile all the Models with RenderMode "mode"
					case OGL_CACHE_MODE_EXECUTE:
						if(((COGLModelFactory *)(IModelFactory*)CEngineFactory::InstancePtr()->GetModelFactory())->RecompileAll(mode)){
							LOG(TL_ERROR,"Error occured in COGLGlobalRenderSettings::SetRenderModeOverride()");
						}
						break;
					case OGL_CACHE_MODE_DISABLED:
						//The Renderer will take care of this situation
						if(mode == OGL_RENDER_MODE_VERTEXBUFFEROBJECT){
							if(((COGLModelFactory *)(IModelFactory*)CEngineFactory::InstancePtr()->GetModelFactory())->RecompileAll(mode)){
								LOG(TL_ERROR,"Error occured in COGLGlobalRenderSettings::SetRenderModeOverride()");
							}
						}
						break;
				}

			}else{

				//Compile only the cached Objects
				if(((COGLModelFactory *)(IModelFactory*)CEngineFactory::InstancePtr()->GetModelFactory())->Recompile(mode)){
					LOG(TL_ERROR,"Error occured in COGLGlobalRenderSettings::SetRenderModeOverride()");
				}

			}
			m_RenderModeOverride = mode;
			m_bGlobalRenderModeStateIsActive = true;
			return true;

		}else{
			LOG(TL_ERROR,"Unknown RenderModeOverride in COGLGlobalRenderSettings::SetRenderModeOverride()");
			return false;
		}

 	}else{

		if(m_bGlobalCacheModeStateIsActive){

			switch(m_CacheModeOverride){
				//Compile all the Models with their original RenderModes
					case OGL_CACHE_MODE_EXECUTE:
						if(((COGLModelFactory *)(IModelFactory*)CEngineFactory::InstancePtr()->GetModelFactory())->RecompileAll(OGL_RENDER_MODE_UNKNOWN)){
							LOG(TL_ERROR,"Error occured in COGLGlobalRenderSettings::SetRenderModeOverride()");
						}
						break;
					case OGL_CACHE_MODE_DISABLED:
						//The Renderer will take care of this situation
						break;
			}
			

		}else{

			//Compile only the cached Objects
			if(((COGLModelFactory *)(IModelFactory*)CEngineFactory::InstancePtr()->GetModelFactory())->Recompile(OGL_RENDER_MODE_UNKNOWN)){
				LOG(TL_ERROR,"Error occured in COGLGlobalRenderSettings::SetRenderModeOverride()");
			}
		}
		m_RenderModeOverride = mode;
		m_bGlobalRenderModeStateIsActive = false;
		return true;
	}
}

bool COGLGlobalRenderSettings::SetCacheModeOverride(EOGLCacheMode mode, bool bEnable){

	if((mode == OGL_CACHE_MODE_EXECUTE || mode == OGL_CACHE_MODE_UNKNOWN) &&
		!(m_bGlobalRenderModeStateIsActive &&
		(m_RenderModeOverride == OGL_RENDER_MODE_VERTEXARRAY || m_RenderModeOverride == OGL_RENDER_MODE_MEGASLOW)))
	{
		return false;
	}

	if(bEnable){

		switch(mode){
			case OGL_CACHE_MODE_EXECUTE:

				if(m_bGlobalRenderModeStateIsActive){
					if(((COGLModelFactory *)(IModelFactory*)CEngineFactory::InstancePtr()->GetModelFactory())->RecompileAll(m_RenderModeOverride)){
						LOG(TL_ERROR,"Error occured in COGLGlobalRenderSettings::SetCacheModeOverride()");
					}
				}else{
					if(((COGLModelFactory *)(IModelFactory*)CEngineFactory::InstancePtr()->GetModelFactory())->RecompileAll(OGL_RENDER_MODE_UNKNOWN)){
						LOG(TL_ERROR,"Error occured in COGLGlobalRenderSettings::SetCacheModeOverride()");
					}
				}
				break;

			case OGL_CACHE_MODE_DISABLED:
				//The Renderer will take care of this situation
				break;
		}
		if(m_bGlobalRenderModeStateIsActive){
			if(((COGLModelFactory *)(IModelFactory*)CEngineFactory::InstancePtr()->GetModelFactory())->RecompileAll(m_RenderModeOverride)){
				LOG(TL_ERROR,"Error occured in COGLGlobalRenderSettings::SetCacheModeOverride()");
			}
		}
		m_bGlobalCacheModeStateIsActive = true;
		m_CacheModeOverride = mode;
		return true;

	}else{

		if(m_bGlobalRenderModeStateIsActive){
			if(((COGLModelFactory *)(IModelFactory*)CEngineFactory::InstancePtr()->GetModelFactory())->Recompile(m_RenderModeOverride)){
				LOG(TL_ERROR,"Error occured in COGLGlobalRenderSettings::SetCacheModeOverride()");
			}
		}else{
			if(((COGLModelFactory *)(IModelFactory*)CEngineFactory::InstancePtr()->GetModelFactory())->Recompile(OGL_RENDER_MODE_UNKNOWN)){
				LOG(TL_ERROR,"Error occured in COGLGlobalRenderSettings::SetCacheModeOverride()");
			}
		}
		m_bGlobalCacheModeStateIsActive = false;
 		m_CacheModeOverride = OGL_CACHE_MODE_UNKNOWN;
		return true;
	}
}

void COGLGlobalRenderSettings::SetWireFrame(bool bEnable,bool bUniformColor){
	
	if(bEnable){
		if(bUniformColor){
			DisableAllRenderFeatures();
			m_globalDrawColor = CVector4f(0.6f,0.2f,0.8f,1.0f);
		}
		glClearColor(0.3f,0.3f,0.3f,1.0f);
		glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
	}else{
		glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
		m_globalDrawColor = CVector4f(1.0f,1.0f,1.0f,1.0f);
		glClearColor(m_bgColor.m_X,m_bgColor.m_Y,m_bgColor.m_Z,m_bgColor.m_W);
		EnableAllRenderFeatures();
	}
}

void COGLGlobalRenderSettings::SetStipplePolygon(bool bEnable){
	if(bEnable){
		glEnable(GL_POLYGON_STIPPLE);
	}else{
		glDisable(GL_POLYGON_STIPPLE);
	}
}

void COGLGlobalRenderSettings::SetDrawColor(CVector4f &color){
	//IF OGL_RENDER_STATE_COLOR IS DISABLED - USE THIS
	m_globalDrawColor = color;
}

void COGLGlobalRenderSettings::EnableBlendingEffects(bool bEnable){
	((COGLRenderer *)(IRenderer *)CEngineFactory::InstancePtr()->GetRenderer())->EnableBlendEffects(bEnable);
}
/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/