/****************************************************************************\
*                                                                            *
* 18.03.2006                                                                 *
* COGLModelFactory - Header File                                                *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __COGLMODELFACTORY_H__
#define __COGLMODELFACTORY_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Renderer/IModelFactory.h>
#include <Renderer/COGLModelGeometry.h>

#include <map>
/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/
class COGLStockGeometry;

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class COGLModelFactory : public IModelFactory
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
		COGLModelFactory(TSmartPointer<IModelLoader> ,TSmartPointer<ITextureLoader>, TSmartPointer<CTerrainBuilder> tb);

        virtual TSmartPointer<IModelGeometry> getModel(const std::string &);
		
		virtual TSmartPointer<IStockGeometry> getStockObject(ESTOCK_OBJECT eType, const std::string & texFileName,
															 float topLeftX, float topLeftY, float bottomRightX, float bottomRightY,
															 const CVector4f *color, EBlendingMode blendingMode);

        virtual bool CreateTerrain(const std::string &sLevel, CTerrainInfo *pTerrainInfo,
                                   std::vector< TSmartPointer<IModelGeometry> > &models);

		virtual bool CreateEnv(const std::string *tex,byte nmb);

		virtual dword RecompileAll(EOGLRenderMode eRenderMode);
		virtual dword Recompile(EOGLRenderMode eRenderMode);

		virtual void SetGlobalTextureQuality(EOGLTextureQuality quality);
		virtual void SetGlobalMipMapping(bool bEnable);
			
		virtual ~COGLModelFactory();

	/**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        //Private copy constructor and assignment operator to prevent mistakes:
        COGLModelFactory(const COGLModelFactory &);
        COGLModelFactory &operator =(const COGLModelFactory &);

		COGLModelGeometry * BuildMG(CGeometryInfo *pGeoInfo);
		bool CreateTexture(COGLTexture * tex, const std::string &fileName);
		word BuildTexture2D(CImageInfo *);
        
    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:
		
    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
		std::map<std::string,TSmartPointer<CImageInfo> > m_currentTextures;
		std::map<std::string,TSmartPointer<CImageInfo> >::iterator mT_it;

		std::map<std::string,TSmartPointer<COGLModelGeometry> > m_currentMG;
		std::map<std::string,TSmartPointer<COGLModelGeometry> >::iterator mg_it;

		std::vector<TSmartPointer<COGLModelGeometry> > m_vCurrentTerrainMG;
		std::vector<TSmartPointer<COGLModelGeometry> >::iterator mgT_it;

        COGLReflectionMapTexLayer m_currentEnvironment;
};

#endif
