/****************************************************************************\
*                                                                            *
* 24.05.2006                                                                 *
* COGLRenderInfo - Header File                                                *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __COGLRENDERINFO_H__
#define __COGLRENDERINFO_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>
#include <Memory/mmgr.h>
#include <Memory/TSmartPointer.h>
#include <Renderer/coglcg/COGLCGProgram.h>
/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/
class COGLRenderer;
class COGLTriangleGroup;
class COGLModelFactory;

enum EOGLRenderMode{
	OGL_RENDER_MODE_MEGASLOW,
	OGL_RENDER_MODE_VERTEXARRAY,
	OGL_RENDER_MODE_VERTEXBUFFEROBJECT,
	OGL_RENDER_MODE_SHADER,
	OGL_RENDER_MODE_UNKNOWN
};

enum EOGLCacheMode{
	OGL_CACHE_MODE_EXECUTE                = 1,
	OGL_CACHE_MODE_DISABLED               = 2,
	OGL_CACHE_MODE_DO_COMPILE             = 4, 
	OGL_CACHE_MODE_DO_COMPILE_AND_EXECUTE = 8,
	OGL_CACHE_MODE_NO_LIST                = 16,
	OGL_CACHE_MODE_COMPILE_ERROR          = 32,
	OGL_CACHE_MODE_NOT_COMPILABLE         = 64,
	OGL_CACHE_MODE_UNKNOWN                = 128
};

enum EOGLTextureQuality{
	OGL_TQ_LINEAR     = 1,
	OGL_TQ_BILINEAR   = 2,
	OGL_TQ_TRILINEAR  = 4,
	OGL_TQ_MIPMAPPING = 8,
	OGL_TQ_UNKNOWN    = 16,
};

enum EOGLRenderState{
	OGL_RENDER_STATE_USE_VERTEXCOLORS = 1,
	OGL_RENDER_STATE_USE_LIGHTING     = 2,
	OGL_RENDER_STATE_USE_COLOR        = 4,
	OGL_RENDER_STATE_USE_TEXTURING    = 8,
	OGL_RENDER_STATE_USE_SHADER       = 16
};

enum EOGLShadeState{
	OGL_SHADE_STATE_VERTEXCOLOR           = OGL_RENDER_STATE_USE_VERTEXCOLORS,
	OGL_SHADE_STATE_VERTEXCOLOR_TEXTURING = OGL_RENDER_STATE_USE_VERTEXCOLORS | OGL_RENDER_STATE_USE_TEXTURING,
	OGL_SHADE_STATE_LIGHTING              = OGL_RENDER_STATE_USE_LIGHTING,
	OGL_SHADE_STATE_LIGHTING_TEXTURING    = OGL_RENDER_STATE_USE_LIGHTING | OGL_RENDER_STATE_USE_TEXTURING,
	OGL_SHADE_STATE_COLOR                 = OGL_RENDER_STATE_USE_COLOR,
	OGL_SHADE_STATE_COLOR_TEXTURING       = OGL_RENDER_STATE_USE_COLOR | OGL_RENDER_STATE_USE_TEXTURING
};

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/

class COGLRenderInfo{

	friend class COGLRenderer;
	friend class COGLTriangleGroup;
	friend class COGLModelFactory;

public:
	COGLRenderInfo(){
		m_eRenderMode  = OGL_RENDER_MODE_MEGASLOW;
		m_eCacheMode   = OGL_CACHE_MODE_NO_LIST;
		m_dispListNmb  = 0;
		m_shadingState = OGL_SHADE_STATE_LIGHTING;
		m_pCGProgram   = NULL;
		m_bIsVertexBuffered = false;

		m_eCurrentRenderMode = OGL_RENDER_MODE_MEGASLOW;
		m_eCurrentCacheMode  = OGL_CACHE_MODE_NO_LIST;
	}

	void setCGShader(COGLCGProgram *pProg){
		m_pCGProgram = pProg;
	}

	EOGLRenderMode getRenderMode(){return m_eRenderMode;}
	EOGLCacheMode  getCacheMode(){return m_eCacheMode;}

	~COGLRenderInfo(){m_pCGProgram = NULL;}

private:
	EOGLRenderMode m_eRenderMode;
	EOGLCacheMode  m_eCacheMode;
	word           m_dispListNmb;
	word           m_shadingState;
	EOGLRenderMode m_eCurrentRenderMode;
	EOGLCacheMode  m_eCurrentCacheMode;
	bool           m_bIsVertexBuffered;

	TSmartPointer<COGLCGProgram> m_pCGProgram;
};

#endif