/****************************************************************************\
*                                                                            *
* 27.03.2006                                                                 *
* IModelFactory - Header File                                                *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __IMODELFACTORY_H__
#define __IMODELFACTORY_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>
#include <Memory/mmgr.h>
#include <Memory/IManagedObject.h>
#include <Memory/TSmartPointer.h>
#include <Loader/IModelLoader.h>
#include <Loader/ITextureLoader.h>
#include <Loader/CTerrainBuilder.h>
#include <Renderer/IModelGeometry.h>
#include <Renderer/IStockGeometry.h>

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/
class CTerrainInfo;

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class IModelFactory : public IManagedObject
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
		IModelFactory(TSmartPointer<IModelLoader> mL ,TSmartPointer<ITextureLoader> tL,
                      TSmartPointer<CTerrainBuilder> pTerrainBuilder){
			m_pModelLoader = mL;
			m_pTextureLoader = tL;
            m_pTerrainBuilder = pTerrainBuilder;
		}

		virtual TSmartPointer<IModelGeometry> getModel(const std::string &) = 0;
		
		virtual TSmartPointer<IStockGeometry> getStockObject(ESTOCK_OBJECT eType, const std::string & texFileName,
															 float topLeftX, float topLeftY, float bottomRightX, float bottomRightY,
															 const CVector4f *color, EBlendingMode blendingMode) = 0; 
															 	
        virtual bool CreateTerrain(const std::string &sLevel, CTerrainInfo *pTerrainInfo,
                                   std::vector< TSmartPointer<IModelGeometry> > &models) = 0;

		virtual bool CreateEnv(const std::string *texNames, byte nmb) = 0;

		virtual ~IModelFactory(){
			m_pModelLoader = NULL;
			m_pTextureLoader = NULL;
            m_pTerrainBuilder = NULL;
		}
        
    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        //Private copy constructor and assignment operator to prevent mistakes:
        IModelFactory(const IModelFactory &);
        IModelFactory &operator =(const IModelFactory &);
        
    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:
		TSmartPointer<IModelLoader> m_pModelLoader;
		TSmartPointer<ITextureLoader> m_pTextureLoader;
        TSmartPointer<CTerrainBuilder> m_pTerrainBuilder;
    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
        
};

#endif
