/****************************************************************************\
*                                                                            *
* 18.03.2006                                                                 *
* IRenderer - Header File                                                    *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __IRENDERER_H__
#define __IRENDERER_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>
#include <Memory/mmgr.h>

#include <Display/Windows/IWindowHandler.h>
#include <Display/Windows/IWindow.h>
#include <Memory/IManagedObject.h>
#include <Memory/TSmartPointer.h>
#include <Renderer/IGlobalRenderSettings.h>
#include <Math/CMatrix44.h>

#include "IFont.h"

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/
enum EMatrixMode
{
    EMM_PROJECTION,
    EMM_MODELVIEW,
};

/****************************************************************************\
* ABSTRACT CLASS                                                             *
\****************************************************************************/
class IRenderer : public IManagedObject
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
		IRenderer(){};
        virtual ~IRenderer(){};
		
		virtual bool Create(const TSmartPointer<IWindow> &pWindow) = 0;
		
		virtual TSmartPointer<IFont> GetFontObject(const std::string &sFontName, byte bSize, bool bBold, bool bItalic) = 0;
		virtual TSmartPointer<IGlobalRenderSettings> GetGlobalSettings() = 0;
      
        virtual void SetBgColor(dword dwColor) = 0;
        virtual void SetMatrixMode(EMatrixMode mode) = 0;
        virtual void SetMatrix(const CMatrix44 &mat) = 0;
        virtual void MultMatrix(const CMatrix44 &mat) = 0;
        virtual void SaveMatrix() = 0;
        virtual void RestoreMatrix() = 0;
        virtual void EnableZTest(float fEnable) = 0;
		virtual void EnableZWrite(float fEnable) = 0;
		virtual void EnableAlphaTest(float fEnable) = 0;
		virtual void EnableBlending(float fEnable) = 0;

		virtual void SetLightDir(const CVector3f &vDir) = 0;
		virtual const CVector4f &GetLightDir() const = 0;
		virtual void SetLightColor(byte lightNmb,const CVector4f & ambientColor,const CVector4f &diffuseColor) = 0;
		virtual CVector4f const & getAmbientLightColor(byte lightNmb) = 0; 

        virtual void StartRendering() = 0;
        virtual void EndRendering() = 0;

		virtual void Destroy() = 0;

	/**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
		     
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        //Private copy constructor and assignment operator to prevent mistakes:
        IRenderer(const IRenderer &);
        IRenderer &operator =(const IRenderer &);
       
    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:
			
    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
        
};

#endif
