/****************************************************************************\
*                                                                            *
* 24.02.2006                                                                 *
* CTimer - Source File                                                       *
*                                                                            *
* (c) Raphael Charwot                                                        *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CTimer.h"
#include "CClock.h"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
CTimer::CTimer(CClock * const pClock) :
    m_pClock(pClock),
    m_dCurrentTime(0),
    m_dFrameTime(0),
    m_bEnabled(true),
    m_fScale(1.0f)
{
    m_pClock->AddObserver(this);
}

double CTimer::GetFrameDuration() const
{
    if(m_bEnabled)
    {
        return m_dFrameTime;
    }
    return 0.0;
}

float CTimer::GetFrameRate() const
{
    return m_pClock->GetFrameRate();
}

void CTimer::Reset()
{
    m_dCurrentTime = 0.0;
}

CTimer::~CTimer()
{
    if(m_pClock)
    {
        m_pClock->RemoveObserver(this);
        m_pClock = NULL;
    }
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
void CTimer::Notify()
{
    if(m_bEnabled)
    {
        m_dFrameTime = m_pClock->GetFrameDuration() * m_fScale;
        m_dCurrentTime += m_dFrameTime;
    }
}
