/****************************************************************************\
*                                                                            *
* 12.04.2006                                                                 *
* CGameCtrl - Header File                                                    *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __CGAMECTRL_H__
#define __CGAMECTRL_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <Memory/mmgr.h>
#include <Memory/TSmartPointer.h>
#include <Display/ISceneGraph.h>
#include <Display/Gui/CGuiMgr.h>
#include <Renderer/IRenderer.h>
#include <Renderer/IModelFactory.h>
#include <Input/IInputMgr.h>
#include <Timer/CClock.h>
#include <Display/Camera/ICamCtrl.h>
#include "Objects/Terrain/CTerrain.h"
#include "Objects/Environment/CEnvironment.h"
#include <Renderer/IGlobalRenderSettings.h>



/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/
class IGameState;
class CBuildState;
class CBattleState;
class CPostGameState;
class CCamera;
class CBuildingObj;
class COGLMultiPass;

enum ECollisionGroup
{
	ECG_TERRAIN,
	ECG_PLAYER,
	ECG_ENEMY,
	ECG_BUILDING,
	ECG_PLAYER_PROJECTILE,
	ECG_ENEMY_PROJECTILE,
	ECG_TOWER,
	ECG_TOWER_PROJECTILE,
	ECG_VEGETATION,
};

typedef std::vector< TSmartPointer<IGameObj> > GameObjectList;

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class CGameCtrl /*Debug*/ : private IInputHandler
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
		CGameCtrl(){Clear();}
        
        bool Create(const TSmartPointer<CClock> &pClock);
        bool Update();
        void UpdateGameObjects(float dt);
        void Destroy();

        //Access functions
        inline const TSmartPointer<CClock> &GetClock() const {return m_pClock;}
		inline const TSmartPointer<IRenderer> &GetRenderer() const {return m_pRenderer;}
		inline const TSmartPointer<CGuiMgr> &GetGuiMgr() const {return m_pGuiMgr;}

        void SetCurrentState(IGameState *pState);
        inline IGameState *GetMainMenuState() const {return m_pMainMenuState;}
        inline IGameState *GetNewLevelState() const {return m_pNewLevelState;}
        IGameState *GetBuildState() const;
        IGameState *GetBattleState() const;
		inline IGameState *GetPostLevelState() const {return m_pPostLevelState;}
		inline CPostGameState *GetPostGameState() const {return m_pPostGameState;}

        void SetSceneGraph(const TSmartPointer<ISceneGraph> &pSceneGraph);
        inline const TSmartPointer<ISceneGraph> &GetSceneGraph() {return m_pSceneGraph;}
		
		void SetCamCtrl(const TSmartPointer<ICamCtrl> &pCamCtrl);
		inline void SetCameraPos(const CVector3f &vPos) {m_pCamera->SetPos(vPos);}
        inline const CVector3f GetCameraPos() const {return m_pCamera->GetPos();}
        const CVector3f GetCameraRay() const;
		CCamera * const getCamera(){return m_pCamera;}
		inline const CVector3f &GetPlayerStartPos() const {return m_vPlayerStartPos;}

        void SetTerrain(const TSmartPointer<CTerrain> &pTerrain);
        inline const TSmartPointer<CTerrain> &GetTerrain() {return m_pTerrain;}

		void SetEnvironment(const TSmartPointer<CEnvironment> &pEnvironment);
		inline const TSmartPointer<CEnvironment> &GetEnvironment() {return m_pEnvironment;}

		inline void SetLevel(word wLevel) {m_wLevel = wLevel;}
        inline word GetLevel() const {return m_wLevel;}
        inline void SetLevelName(const std::string &sLevelName) {m_sLevelName = sLevelName;}
        inline const std::string &GetLevelName() const {return m_sLevelName;}
        void ResetGame();
		void LevelDone();

        void AddObject(const TSmartPointer<IGameObj> &pObj);
		inline GameObjectList *GetObjectList() {return &m_GameObjects;} //Dirty, aber muss sein

		void AddMoney(float fAmount);
		void RemoveMoney(float fAmount);
		void AddScore(float fAmount);
		inline float GetScore() const {return m_fScore;}

		CBuildingObj *GetRandomBuilding();

        virtual ~CGameCtrl() {Destroy();}
        
    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        void Clear();
        //Debug
        virtual bool OnKeyDown(dword dwKey);
        //End Debug
        //Private copy constructor and assignment operator to prevent mistakes:
        CGameCtrl(const CGameCtrl &);
        const CGameCtrl &operator =(const CGameCtrl &);

    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
        TSmartPointer<CClock> m_pClock;

        //Game states
        IGameState		*m_pMainMenuState;
        IGameState		*m_pNewLevelState;
        CBuildState		*m_pBuildState;
        CBattleState	*m_pBattleState;
		IGameState		*m_pPostLevelState;
		CPostGameState	*m_pPostGameState;

        IGameState *m_pCurrState;

        TSmartPointer<ISceneGraph> m_pSceneGraph;
        TSmartPointer<CCamera>     m_pCamera;
        TSmartPointer<ICamCtrl>    m_pCamCtrl;

        word m_wLevel;
        std::string m_sLevelName;

        TSmartPointer<CTerrain>     m_pTerrain;
		TSmartPointer<CEnvironment> m_pEnvironment;

        GameObjectList m_GameObjects;
		GameObjectList m_ObjectsToAdd;

		float m_fScore;
		CVector3f m_vPlayerStartPos;

        //For quicker access...
        TSmartPointer<IRenderer>             m_pRenderer;
		TSmartPointer<IGlobalRenderSettings> m_pGlobalRenderSettings;
		TSmartPointer<IModelFactory>         m_pModelFactory;
        TSmartPointer<IInputMgr>             m_pInputMgr;
        TSmartPointer<CGuiMgr>               m_pGuiMgr;

		//Kein SmartPointer
		COGLMultiPass * m_pMultiPass;
		bool m_bRenderMultiPass;
		
		//Global States
		bool m_bGRenderWireframe;
		bool m_bGRenderGuiInfo;
		bool m_bGRenderMipMapping;
		EOGLTextureQuality m_eGTexQuality;
		EOGLRenderMode m_eGRenderMode;
		EOGLCacheMode  m_eGCacheMode;
		bool m_bGUseVFC;
		bool m_bGRenderTransparent;
		bool m_bUseFrustumCulling;
		bool m_bEnableTransparency;
};

#endif
