/****************************************************************************\
*                                                                            *
* 18.04.2006                                                                 *
* CBirdRep - Source File                                                     *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CBirdRep.h"
#include <Display/ISceneGraph.h>
#include <Renderer/IModelGeometry.h>
#include <CEngineFactory.h>
#include "CBirdObj.h"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
CBirdRep::CBirdRep() : m_pBirdObj(NULL)
{
}

bool CBirdRep::Create(ISceneGraph *pSceneGraph, CBirdObj *pBirdObj)
{
    m_pBirdObj = pBirdObj;
    m_pModelGeometry = CEngineFactory::InstancePtr()->GetModelFactory()->getModel("Data\\models\\bird.fbx");
    if(!m_pModelGeometry)
    {
        LOG(TL_ERROR, "Error: getModel failed in CBirdRep::Create");
        return false;
    }

    IRepresentation::Create(pSceneGraph, m_pModelGeometry->GetBoundingSphere(), m_pModelGeometry->GetAABoundingBox());

    return true;
}

const CVector3f &CBirdRep::GetPos() const
{
    return m_pBirdObj->GetPos();
}

const CVector3f &CBirdRep::GetRot() const
{
    return m_pBirdObj->GetRot();
}

const CVector3f &CBirdRep::GetScale() const
{
    return m_pBirdObj->GetScale();
}

const CMatrix44 &CBirdRep::GetTransformation()
{
    return m_pBirdObj->GetTransformation();
}

IGameObj * const CBirdRep::GetGameObj() const
{
    return m_pBirdObj;
}

bool CBirdRep::RayIntersection(const CVector3f &vOrigin, const CVector3f &vDir, CVector3f &vIntersectionPos, float &fIntersectionDistance) const
{
    return m_pModelGeometry->RayIntersection(GetPos(), GetRot(), vOrigin, vDir, vIntersectionPos, fIntersectionDistance);
}

void CBirdRep::Show(bool bShow)
{
    if(bShow)
    {
        m_pSceneGraph->AddObject(this);
    }
    else
    {
        m_pSceneGraph->RemoveObject(this);
    }
}

CBirdRep::~CBirdRep()
{
    m_pBirdObj = NULL;
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
void CBirdRep::RenderSub(const CFrustum &frustum, bool bVerify)
{
    m_pModelGeometry->Render();
}

