/****************************************************************************\
*                                                                            *
* 30.04.2006                                                                 *
* CAiTowerCtrl - Source File                                                 *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CAiTowerCtrl.h"
#include "CEngineFactory.h"
#include "CTowerObj.h"
#include "CGameCtrl.h"
#include "Objects/Birds/CBirdObj.h"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/
const float cg_fAttackDistanceSqr = 2500.0f;

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
CAiTowerCtrl::CAiTowerCtrl(CGameCtrl *pGameCtrl) : m_pGameCtrl(pGameCtrl), m_pNextTarget(NULL), m_vRot(0.0f, 0.0f, 0.0f)
{
}

void CAiTowerCtrl::Update(float dt)
{
	if(m_pNextTarget && m_pNextTarget->GetState() != EBS_LIVING)
	{
		m_pNextTarget->RemoveDeleteObserver(this);
		m_pNextTarget = NULL;
	}
	CBirdObj *pBird = NULL;
	GameObjectList *pObjectList = m_pGameCtrl->GetObjectList();
	GameObjectList::iterator it;
	float fMinDistance;
	float fDistance;
	for(it = pObjectList->begin(); it != pObjectList->end(); ++it)
	{
		if((*it)->GetType() == EOT_BIRD)
		{
			if(((CBirdObj *)((IGameObj *)(*it)))->GetState() == EBS_LIVING)
			{
				fDistance = ((*it)->GetPos() - m_pTower->GetShootingPos()).LengthSqr();
				if(pBird == NULL || fDistance < fMinDistance)
				{
					pBird = (CBirdObj *)((IGameObj *)(*it));
					fMinDistance = fDistance;
				}
			}
		}
	}
	if(pBird == NULL)
		return;
	if(pBird != m_pNextTarget)
	{
		if(m_pNextTarget)
			m_pNextTarget->RemoveDeleteObserver(this);
		m_pNextTarget = pBird;
		m_pNextTarget->AddDeleteObserver(this);
	}
	//"Look" at target
    CVector3f vDir(m_pNextTarget->GetPos() - m_pTower->GetShootingPos());
    fDistance = vDir.Length();
    //Add the target's In-vector to "predict" movement, depending on distance
    vDir += m_pNextTarget->GetIn() * fDistance / 3.0f;
    //Adding some height offset, depending on distance
	vDir.m_Y += fDistance / 5.0f;
	m_vRot.LookAlongRot(vDir);
}

bool CAiTowerCtrl::GetRot(CVector3f &vRot)
{
	vRot.ShortestRot(m_pTower->GetRot(), m_vRot);

    return true;
}

bool CAiTowerCtrl::Shoot()
{
	if(m_pNextTarget && (m_pNextTarget->GetPos() - m_pTower->GetPos()).LengthSqr() <= cg_fAttackDistanceSqr)
		return true;

	return false;
}

CAiTowerCtrl::~CAiTowerCtrl()
{
	if(m_pNextTarget)
	{
		m_pNextTarget->RemoveDeleteObserver(this);
		m_pNextTarget = NULL;
	}
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
void CAiTowerCtrl::Notify()
{
	//Ok, our target has been destroyed
	m_pNextTarget = NULL;
}
