/****************************************************************************\
*                                                                            *
* 01.05.2006                                                                 *
* CDummyRep - Source File                                              *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Objects/CDummyRep.h>
#include <CEngineFactory.h>

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
CDummyRep::CDummyRep(const std::string & modelName, const CVector3f &vPos)
{

	m_pModelGeometry = CEngineFactory::InstancePtr()->GetModelFactory()->getModel(modelName);
	if(!m_pModelGeometry){
		LOG(TL_ERROR,"Failed to load Model : \"%s\" in CDummyRep::CDummyRep()");
	}

	Create(NULL,m_pModelGeometry->GetBoundingSphere(),m_pModelGeometry->GetAABoundingBox());

	SetPickable(false);

	m_pos = vPos;
	m_rot = CVector3f(0,0,0);
	m_scale = CVector3f(1,1,1);

	CMatrix44 tmp1;
	CMatrix44 tmp2;
	CMatrix44 tmp3;

	tmp1.Scaling(m_scale);
	tmp2.RotationZXY(m_rot);
	tmp3.Translation(m_pos);

	m_transform = tmp3 * tmp2 * tmp1;
}

const CVector3f & CDummyRep::GetPos() const{
	return m_pos;
}

const CVector3f & CDummyRep::GetRot() const{
	return m_rot;
}

const CVector3f & CDummyRep::GetScale() const{
	return m_scale;
}

const CMatrix44 & CDummyRep::GetTransformation(){
	return m_transform;
}

IGameObj * const CDummyRep::GetGameObj() const
{
    LOG(TL_DEBUG, "CDummyRep");
    return NULL;
}

bool CDummyRep::RayIntersection(const CVector3f &vOrigin, const CVector3f &vDir, CVector3f &vIntersectionPos, float &fIntersectionDistance) const
{
    return false;
}

void CDummyRep::Destroy(){
	
}

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
void CDummyRep::RenderSub(const CFrustum &frustum, bool bVerify){
	m_pModelGeometry->Render();
}
/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/