/****************************************************************************\
*                                                                            *
* 22.05.2006                                                                 *
* CProjectileRep - Source File                                               *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CProjectileRep.h"
#include <Renderer/IModelGeometry.h>
#include <Display/ISceneGraph.h>
#include <CEngineFactory.h>
#include "CProjectileObj.h"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
CProjectileRep::CProjectileRep() : m_pProjectileObj(NULL), m_pModelGeometry(NULL)
{

}

bool CProjectileRep::Create(ISceneGraph *pSceneGraph, const std::string &sModel, CProjectileObj *pProjectileObj)
{
	m_pProjectileObj = pProjectileObj;
	m_pModelGeometry = CEngineFactory::InstancePtr()->GetModelFactory()->getModel(sModel);
	if(!m_pModelGeometry)
	{
		LOG(TL_ERROR, "Error: getModel failed in CProjectileRep::Create");
		return false;
	}

	IRepresentation::Create(pSceneGraph, m_pModelGeometry->GetBoundingSphere(), m_pModelGeometry->GetAABoundingBox());

	pSceneGraph->AddObject(this);

	return true;
}

const CVector3f &CProjectileRep::GetPos() const
{
	return m_pProjectileObj->GetPos();
}

const CVector3f &CProjectileRep::GetRot() const
{
	return m_pProjectileObj->GetRot();
}

const CVector3f &CProjectileRep::GetScale() const
{
	return m_pProjectileObj->GetScale();
}

const CMatrix44 &CProjectileRep::GetTransformation()
{
	return m_pProjectileObj->GetTransformation();
}

bool CProjectileRep::RayIntersection(const CVector3f &vOrigin, const CVector3f &vDir, CVector3f &vIntersectionPos, float &fIntersectionDistance) const
{
	return m_pModelGeometry->RayIntersection(GetPos(), GetRot(), vOrigin, vDir, vIntersectionPos, fIntersectionDistance);
}

IGameObj * const CProjectileRep::GetGameObj() const
{
	return m_pProjectileObj;
}

CProjectileRep::~CProjectileRep()
{
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
void CProjectileRep::RenderSub(const CFrustum &frustum, bool bVerify)
{
	m_pModelGeometry->Render();
}