/****************************************************************************\
*                                                                            *
* 07.05.2006                                                                 *
* CStartRep - Header File                                                 *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __CSTARTREP_H__
#define __CSTARTREP_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <Memory/mmgr.h>
#include <Memory/TSmartPointer.h>
#include <Display/IRepresentation.h>
#include <Renderer/IGlobalRenderSettings.h>

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/
class IModelGeometry;
class ISceneGraph;
class CStartObj;

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class CStartRep : public IRepresentation
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        CStartRep();
        
        bool Create(ISceneGraph *pSceneGraph, const std::string &sModel, CStartObj *pStartObj);

        virtual const CVector3f &GetPos() const;
        virtual const CVector3f &GetRot() const;
        virtual const CVector3f &GetScale() const;
        virtual const CMatrix44 &GetTransformation();
        virtual bool RayIntersection(const CVector3f &vOrigin, const CVector3f &vDir, CVector3f &vIntersectionPos, float &fIntersectionDistance) const;

        virtual IGameObj * const GetGameObj() const;

        virtual ~CStartRep();
        
    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        virtual void RenderSub(const CFrustum &frustum, bool bVerify);

        //Private copy constructor and assignment operator to prevent mistakes:
        CStartRep(const CStartRep &);
        const CStartRep &operator =(const CStartRep &);
        
    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
        CStartObj *m_pStartObj;
        TSmartPointer<IModelGeometry> m_pModelGeometry;
		TSmartPointer<IGlobalRenderSettings> m_pGlobalRenderSettings;
};

#endif
