/****************************************************************************\
*                                                                            *
* 06.05.2006                                                                 *
* CEnvironment - Source File                                                     *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CEnvironment.h"
#include "CGameCtrl.h"
#include "CSettings.h"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
bool CEnvironment::Create(const std::string & sLevelName)
{
	m_pRepresentation = new CEnvironmentRep();

	std::string sEnvType;
	std::string sEnvModelName;

	//get the EnvironmentType
	if(!CSettings::InstancePtr()->GetValue(sLevelName, "EnvironmentType", sEnvType))
	{
		LOG(TL_ERROR, "Error: Couldn't get \"EnvironmentType\"-Value from \"%s\" in CEnvironment::Create",sLevelName.c_str());
		return false;
	}

	//get the EnvironmentModelName
	if(!CSettings::InstancePtr()->GetValue(sLevelName, "EnvironmentModel", sEnvModelName))
	{
		LOG(TL_ERROR, "Error: Couldn't get \"EnvironmentModel\"-Value from \"%s\" in CEnvironment::Create",sLevelName.c_str());
		return false;
	}

	//EnvironmentSphere
	if(sEnvType.compare("Sphere") == 0)
	{

		//get the textureName
		std::string sEnvTexName;
		if(!CSettings::InstancePtr()->GetValue(sLevelName, "ReflectionMapTexture", sEnvTexName))
		{
			LOG(TL_ERROR, "Error: Couldn't get \"EnvironmentTexture\"-Value from \"%s\" in CEnvironment::Create",sLevelName.c_str());
			return false;
		}

		//create the sphere
		if(!((CEnvironmentRep*)(IRepresentation*)m_pRepresentation)->CreateSphere(sEnvModelName,sEnvTexName,m_pGameCtrl->GetSceneGraph(),this))
		{
			LOG(TL_ERROR, "Error: m_pEnvironmentRep->CreateSphere failed in CEnvironment::Create");
			return false;
		}

	}
	else
	{
		//EnvironmentCube
		if(sEnvType.compare("Cube") == 0)
		{

			//get the textureNames
			std::string sEnvTexNames[6];

			if(!CSettings::InstancePtr()->GetValue(sLevelName, "ReflectionMapTexture_posX", sEnvTexNames[0]))
			{
				LOG(TL_ERROR, "Error: Couldn't get \"ReflectionMapTexture_posX\"-Value from \"%s\" in CEnvironment::Create",sLevelName.c_str());
				return false;
			}
			if(!CSettings::InstancePtr()->GetValue(sLevelName, "ReflectionMapTexture_negX", sEnvTexNames[1]))
			{
				LOG(TL_ERROR, "Error: Couldn't get \"ReflectionMapTexture_negX\"-Value from \"%s\" in CEnvironment::Create",sLevelName.c_str());
				return false;
			}
			if(!CSettings::InstancePtr()->GetValue(sLevelName, "ReflectionMapTexture_posY", sEnvTexNames[2]))
			{
				LOG(TL_ERROR, "Error: Couldn't get \"ReflectionMapTexture_posY\"-Value from \"%s\" in CEnvironment::Create",sLevelName.c_str());
				return false;
			}
			if(!CSettings::InstancePtr()->GetValue(sLevelName, "ReflectionMapTexture_negY", sEnvTexNames[3]))
			{
				LOG(TL_ERROR, "Error: Couldn't get \"ReflectionMapTexture_negY\"-Value from \"%s\" in CEnvironment::Create",sLevelName.c_str());
				return false;
			}
			if(!CSettings::InstancePtr()->GetValue(sLevelName, "ReflectionMapTexture_posZ", sEnvTexNames[4]))
			{
				LOG(TL_ERROR, "Error: Couldn't get \"ReflectionMapTexture_posZ\"-Value from \"%s\" in CEnvironment::Create",sLevelName.c_str());
				return false;
			}
			if(!CSettings::InstancePtr()->GetValue(sLevelName, "ReflectionMapTexture_negZ", sEnvTexNames[5]))
			{
				LOG(TL_ERROR, "Error: Couldn't get \"ReflectionMapTexture_negZ\"-Value from \"%s\" in CEnvironment::Create",sLevelName.c_str());
				return false;
			}

			//create the cube
			if(!((CEnvironmentRep*)(IRepresentation*)m_pRepresentation)->CreateCube(sEnvModelName,sEnvTexNames[0],sEnvTexNames[1],sEnvTexNames[2],sEnvTexNames[3],
											                                        sEnvTexNames[4],sEnvTexNames[5],m_pGameCtrl->GetSceneGraph(),this))
			{
				LOG(TL_ERROR, "Error: m_pEnvironmentRep->CreateCube failed in CEnvironment::Create");
				return false;
			}
		}
		else
		{
			return false;
		}
	}

	//Read the sun direction
	std::string sValue;
	if(!CSettings::InstancePtr()->GetValue("GAME", "SunDir", sValue))
	{
		LOG(TL_ERROR, "Error: Couldn't get \"SunDir\"-Value from \"GAME\" in CEnvironment::Create");
		return false;
	}

	if(sscanf(sValue.c_str(), "%f;%f;%f", &m_vSunDir.m_X, &m_vSunDir.m_Y, &m_vSunDir.m_Z) != 3)
	{
		LOG(TL_ERROR, "Error: Couldn't parse \"SunDir\"-Value from \"GAME\"in CEnvironment::Create: %s", sValue.c_str());
		return false;
	}

	//Read the sun color values
	if(!CSettings::InstancePtr()->GetValue("GAME", "SunDiffuseColor", sValue))
	{
		LOG(TL_ERROR, "Error: Couldn't get \"SunDiffuseColor\"-Value from \"GAME\" in CEnvironment::Create");
		return false;
	}
	if(sscanf(sValue.c_str(), "%f;%f;%f", &m_vSunDiffuseColor.m_X, &m_vSunDiffuseColor.m_Y, &m_vSunDiffuseColor.m_Z) != 3)
	{
		LOG(TL_ERROR, "Error: Couldn't parse \"SunDiffuseColor\"-Value from \"GAME\"in CEnvironment::Create: %s", sValue.c_str());
		return false;
	}
	m_vSunDiffuseColor.m_W = 1.0f;


	if(!CSettings::InstancePtr()->GetValue("GAME", "SunAmbientColor", sValue))
	{
		LOG(TL_ERROR, "Error: Couldn't get \"SunAmbientColor\"-Value from \"GAME\" in CEnvironment::Create");
		return false;
	}
	if(sscanf(sValue.c_str(), "%f;%f;%f", &m_vSunAmbientColor.m_X, &m_vSunAmbientColor.m_Y, &m_vSunAmbientColor.m_Z) != 3)
	{
		LOG(TL_ERROR, "Error: Couldn't parse \"SunAmbientColor\"-Value from \"GAME\"in CEnvironment::Create: %s", sValue.c_str());
		return false;
	}
	m_vSunAmbientColor.m_W = 1.0f;


	return true;
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/