/****************************************************************************\
*                                                                            *
* 06.05.2006                                                                 *
* CEnvironmentRep - Source File                                              *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CEnvironmentRep.h"
#include "Cenvironment.h"
#include "CEngineFactory.h"
#include <Display/ISceneGraph.h>
#include <Renderer/IModelGeometry.h>
#include <Renderer/IModelFactory.h>

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
bool CEnvironmentRep::CreateSphere(const std::string & modelName,const std::string & texName,ISceneGraph *pSceneGraph,CEnvironment *obj)
{
	m_pEnvironment = obj;
	m_pSceneGraph  = pSceneGraph;
	
	m_pEnvironmentModel = CEngineFactory::InstancePtr()->GetModelFactory()->getModel(modelName);
	
	if(m_pEnvironmentModel){
		if(CEngineFactory::InstancePtr()->GetModelFactory()->CreateEnv(&texName,1)){
			IRepresentation::Create(m_pSceneGraph,m_pEnvironmentModel->GetBoundingSphere(),m_pEnvironmentModel->GetAABoundingBox());
			return true;
		}
	}
	return false;
}

bool CEnvironmentRep::CreateCube(const std::string & modelName,const std::string & posX_texName,const std::string & negX_texName,
								 const std::string & posY_texName,const std::string & negY_texName,const std::string & posZ_texName,
								 const std::string & negZ_texName,ISceneGraph *pSceneGraph,CEnvironment *obj)
{
	m_pEnvironment = obj;
	m_pSceneGraph  = pSceneGraph;
	
	m_pEnvironmentModel = CEngineFactory::InstancePtr()->GetModelFactory()->getModel(modelName);
	
	if(m_pEnvironmentModel){
		std::string texNames[6];
		texNames[0] = posX_texName;
		texNames[1] = negX_texName;
		texNames[2] = posY_texName;
		texNames[3] = negY_texName;
		texNames[4] = posZ_texName;
		texNames[5] = negZ_texName;

		if(CEngineFactory::InstancePtr()->GetModelFactory()->CreateEnv(texNames,6)){
			IRepresentation::Create(m_pSceneGraph,m_pEnvironmentModel->GetBoundingSphere(),m_pEnvironmentModel->GetAABoundingBox());
			return true;
		}
	}
	return false;
}

const CVector3f &CEnvironmentRep::GetPos() const
{
	return m_pEnvironment->GetPos();
}

const CVector3f &CEnvironmentRep::GetRot() const
{
	return m_pEnvironment->GetRot();
}

const CVector3f &CEnvironmentRep::GetScale() const
{
	return m_pEnvironment->GetScale();
}

const CMatrix44 &CEnvironmentRep::GetTransformation()
{
	return m_pEnvironment->GetTransformation();
}

IGameObj * const CEnvironmentRep::GetGameObj() const
{
	return m_pEnvironment;
}


void CEnvironmentRep::Destroy()
{
	Clear();
	IRepresentation::Destroy();
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
void CEnvironmentRep::Clear()
{
	m_pEnvironment = NULL;
	m_pEnvironmentModel = NULL;
}

void CEnvironmentRep::RenderSub(const CFrustum &frustum, bool bVerify)
{
	m_pEnvironmentModel->Render();
}