/****************************************************************************\
*                                                                            *
* 06.05.2006                                                                 *
* CEnvironmentRep - Header File                                                  *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __CENVIRONMENTREP_H__
#define __CENVIRONMENTREP_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <Memory/mmgr.h>
#include <Memory/TSmartPointer.h>
#include <Display/IRepresentation.h>
#include <Renderer/IModelGeometry.h>

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/
class CEnvironment;
class ISceneGraph;

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class CEnvironmentRep : public IRepresentation
{
	/**************************************************************************\
	* PUBLIC CLASS FUNCTIONS                                                   *
	\**************************************************************************/
public:
	CEnvironmentRep() {Clear();}

	bool CreateSphere(const std::string & modelName,
					  const std::string & texName,
					  ISceneGraph *pSceneGraph, 
					  CEnvironment *obj);
	
	bool CreateCube(const std::string & modelName,
				    const std::string & posX_texName,
					const std::string & negX_texName,
					const std::string & posY_texName,
					const std::string & negY_texName,
					const std::string & posZ_texName,
					const std::string & negZ_texName,
					ISceneGraph *pSceneGraph, 
					CEnvironment *obj);

	virtual const CVector3f &GetPos() const;
	virtual const CVector3f &GetRot() const;
	virtual const CVector3f &GetScale() const;
	virtual const CMatrix44 &GetTransformation();

	virtual IGameObj * const GetGameObj() const;
    virtual bool RayIntersection(const CVector3f &vOrigin, const CVector3f &vDir, CVector3f &vIntersectionPos, float &fIntersectionDistance) const {return false;}

	virtual void Destroy();

	virtual ~CEnvironmentRep() {Destroy();}

	/**************************************************************************\
	* PROTECTED CLASS FUNCTIONS                                                *
	\**************************************************************************/
protected:

	/**************************************************************************\
	* PRIVATE CLASS FUNCTIONS                                                  *
	\**************************************************************************/
private:
	void Clear();
	virtual void RenderSub(const CFrustum &frustum, bool bVerify);
	//Private copy constructor and assignment operator to prevent mistakes:
	CEnvironmentRep(const CEnvironmentRep &);
	const CEnvironmentRep &operator =(const CEnvironmentRep &);

	/**************************************************************************\
	* PROTECTED CLASS MEMBERS                                                  *
	\**************************************************************************/
protected:

	/**************************************************************************\
	* PRIVATE CLASS MEMBERS                                                    *
	\**************************************************************************/
private:
	CEnvironment *m_pEnvironment;
	TSmartPointer<IModelGeometry> m_pEnvironmentModel;

};

#endif
