/****************************************************************************\
*                                                                            *
* 13.05.2006                                                                 *
* ICharacterObj - Source File                                                *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "ICharacterObj.h"
#include <CEngineFactory.h>
/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
ICharacterObj::ICharacterObj(CGameCtrl *pGameCtrl, EObjectType type) : IGameObj(pGameCtrl, type), m_pPhysXChar(NULL),
    m_bInAir(true), m_fAirTime(0.0f), m_bUseGravity(false), m_fGravityFactor(1.0f), m_dwCollisionGroups(0xFFFFFFFF)
{

}

void ICharacterObj::Update(float dt)
{
    if(!m_bInAir)
    {
        m_fAirTime = 0.0f;
    }
    m_fAirTime += dt;
    if(m_pPhysXChar)
    {
        IGameObj::SetPos(m_pPhysXChar->GetPos());
    }
	m_bMoved = false;
    IGameObj::Update(dt);
	if(m_bUseGravity && !m_bMoved)
	{
		Move(CVector3f(0.0f, 0.0f, 0.0f), dt); //Force a movement to give gravity a chance to work
	}
}

void ICharacterObj::SetPos(const CVector3f &vPos)
{
    if(m_pPhysXChar)
    {
        m_pPhysXChar->SetPos(vPos);
    }

    IGameObj::SetPos(vPos);
}

void ICharacterObj::Move(const CVector3f &vDistance, float dt)
{
    if(m_pPhysXChar)
    {
        if(m_bUseGravity)
        {
            m_pPhysXChar->Move((vDistance + (CEngineFactory::InstancePtr()->GetPhysicsMgr()->GetGravity() * m_fGravityFactor * m_fAirTime)) * dt, m_dwCollisionGroups);
        }
        else
        {
            m_pPhysXChar->Move(vDistance * dt, m_dwCollisionGroups);
        }
		m_bInAir = !m_pPhysXChar->GroundContact();
    }
	m_bMoved = true;
}

void ICharacterObj::UseGravity(bool bUseGravity)
{
	m_bUseGravity = bUseGravity;
	m_fAirTime = 0.0f;
}

void ICharacterObj::SetGravityFactor(float fGravityFactor)
{
	m_fGravityFactor = fGravityFactor;
}

void ICharacterObj::Destroy()
{
    if(m_pPhysXChar)
    {
        CEngineFactory::InstancePtr()->GetPhysicsMgr()->DeleteCharacter(m_pPhysXChar);
        m_pPhysXChar = NULL;
    }
    IGameObj::Destroy();
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/
void ICharacterObj::SetCharacter(CPhysXCharacter *pCharacter)
{
    if(m_pPhysXChar)
    {
        CEngineFactory::InstancePtr()->GetPhysicsMgr()->DeleteCharacter(m_pPhysXChar);
    }
    m_pPhysXChar = pCharacter;
	if(m_pPhysXChar)
	{
		m_pPhysXChar->SetObject(this);
	}
}

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/


