/****************************************************************************\
*                                                                            *
* 10.06.2006                                                                 *
* CForest - Source File                                                      *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CForest.h"
#include "CGameCtrl.h"
#include "CSettings.h"
#include <Loader/CTGALoader.h>

#include <Display/CQTSceneGraph.h>
#include <Display/TQTNode.h>
#include <gl/gl.h>

#include <Display/GraphStates/CForestStateNode.h>

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/
struct SRemoveDeletedTree
{
	bool operator()(const TSmartPointer<CTreeObj> &pObj)
	{
		return pObj->IsDestroyed();
	}
};

const float cg_fTreeScaleDiff = 0.8f;

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
bool CForest::Create(const std::string & sLevel,CTerrain * pTerrain){

	m_pTerrain = pTerrain;

	m_pForestStateNode = new CForestStateNode();
	m_pSpecialRoot = ((CQTSceneGraph *)(ISceneGraph *)pTerrain->getGameControl()->GetSceneGraph())->CreateSpecialRootNode(((IStateNode*)(CForestStateNode*)m_pForestStateNode));

	std::string plantMap;
	if(!CSettings::InstancePtr()->GetValue(sLevel,"Treemap",plantMap)){
		LOG(TL_ERROR, "Error: Couldn't get \"Treemap\"-Value from \"%s\" in CForestr::Create", sLevel.c_str());
		return false;
	}

	std::string sValue;
	if(!CSettings::InstancePtr()->GetValue(sLevel,"NmbTrees",sValue)){
		LOG(TL_ERROR, "Error: Couldn't get \"NmbTrees\"-Value from \"%s\" in CForestr::Create", sLevel.c_str());
		return false;
	}
	int nmbOfModels = atoi(sValue.c_str());
	
	char buf[5];
	std::string treeModelNmb;
	for(int i = 0; i < nmbOfModels ; i++){
		treeModelNmb = "TreeModel";
		treeModelNmb.append(itoa(i+1,buf,10));
		if(!CSettings::InstancePtr()->GetValue(sLevel,treeModelNmb,sValue)){
			LOG(TL_ERROR, "Error: Couldn't get \"TreeModel%d \"-Value from \"%s\" in CForestr::Create",i,sLevel.c_str());
			return false;
		}
		m_vTreeModelNames.push_back(sValue);
	}

	TSmartPointer<CImageInfo> pM = new CImageInfo();
	CTGALoader tgaLoader;

	if(!tgaLoader.LoadImage(plantMap,pM)){
		LOG(TL_ERROR,"Could not Load PlantMap %s in CForest::Create()!");
		return false;
	}

	PlantTrees(pM->m_pData,pM->m_Width,pM->m_Height,(pM->m_bAlpha) ? 4 : 3);

	return true;
}

void CForest::PlantTrees(byte * const plantMap, dword mapSizeX, dword mapSizeZ, byte nmbChannels){

	//Todo - the algorithm now needs 3 different treeModels

	int maxNmbOfTreesPerType = 12;

	assert( m_pTerrain && "Error: m_pTerrain is NULL in CForest::PlantTrees");
	assert( (nmbChannels <= 3) && "Error: PlantMaps with Alpha Channel not allowed in CForest::PlantTrees");
			
	CTerrainInfo * pTerrainInfo = m_pTerrain->getTerrainInfo();

	float terrainSizeX = pTerrainInfo->GetSizeX();
	float terrainSizeZ = pTerrainInfo->GetSizeZ();
	float cellSizeX    = terrainSizeX/(float)mapSizeX;
	float cellSizeZ    = terrainSizeZ/(float)mapSizeZ;

	dword     nextChannelIndex;
	byte *    density = new byte[nmbChannels];	
	float     radius  = (float)(CVector2f(0,0) - CVector2f(cellSizeX/2.0f,cellSizeZ/2.0f)).Length();

	int       nmbOfTrees;
	CVector2f cellCenter;
	CVector2f mainJitterAxis;
	CVector2f perpJitterAxis;
	CVector2f finalPos;
	float     mainOffset;
	float     perpOffset;
	float     angleStep;
	float	  angleOffset;

	float     randNormal = 1/32767.0f;

	CVector3f vScale;
	CVector3f vRot(0.0f, 0.0f, 0.0f);

	for(dword i = 0 ; i < mapSizeZ ; ++i){
		for(dword j = 0 ; j < mapSizeX ; ++j){

			nextChannelIndex = nmbChannels*i*mapSizeX + (nmbChannels*j);

			density[0] = (byte)(plantMap[nextChannelIndex]/255.0f*maxNmbOfTreesPerType);
			density[1] = (byte)(plantMap[nextChannelIndex+1]/255.0f*maxNmbOfTreesPerType);
			density[2] = (byte)(plantMap[nextChannelIndex+2]/255.0f*maxNmbOfTreesPerType);
			
			cellCenter.Set( (float)(j*cellSizeX + cellSizeX/2.0) ,(float)(i*cellSizeZ  + cellSizeZ/2.0) );
			
			nmbOfTrees  = density[0] + density[1] + density[2];
						
			if(nmbOfTrees){

				int *treeTypeAtPos = new int[nmbOfTrees];
				ZeroMemory(treeTypeAtPos,nmbOfTrees*sizeof(int));

				int nmbNextType = density[0];
				int f   = 0;
				int l   = 0;
								
				while(true){
					if(l<3){
                        if(f < nmbNextType){

							int pos = (int)(rand()*randNormal*nmbOfTrees);
							
							if(treeTypeAtPos[pos]!=0){
								while(true){
									pos = (++pos)%nmbOfTrees;
									if(treeTypeAtPos[pos]==0){
										break;
									}
								}
							}
							treeTypeAtPos[pos] = l;
							f++;
						}else{
							f = 0;
							nmbNextType = density[++l];
						}
					}else{
						break;
					}
				}

				angleStep   =  2*MY_PI/nmbOfTrees;
				angleOffset =  2*MY_PI*(rand()*randNormal);
				                 
				CTreeObj *pTree;
				for(int k = 0 ; k < nmbOfTrees ; k++){

					mainJitterAxis.Set(cos(angleOffset+k*angleStep),sin(angleOffset+k*angleStep));
					perpJitterAxis.Set(-mainJitterAxis.m_Y,mainJitterAxis.m_X);

					mainOffset = (rand()*randNormal)*radius;
					perpOffset = (float)(((rand()*randNormal)-0.5)*2)*(tan(angleStep)*mainOffset);
					vScale.m_X = vScale.m_Z = vScale.m_Y = 1.0f + ((rand() * randNormal)-0.5f) * cg_fTreeScaleDiff;
					vRot.m_Y = (rand() * randNormal) * MY_PI * 2.0f;
					
					finalPos = cellCenter + (mainJitterAxis * mainOffset) + (perpJitterAxis * perpOffset);

					pTree = new CTreeObj(m_pGameCtrl, m_vTreeModelNames[treeTypeAtPos[k]]);
					pTree->SetPos(CVector3f(finalPos.m_X, pTerrainInfo->HeightAt(finalPos.m_X,finalPos.m_Y,false), finalPos.m_Y));
					pTree->SetScale(vScale);
					pTree->SetRot(vRot);
					//pTree->Plant(); //Plant weglassen, um den Tree nicht collidable zu machen
					//pTree->Update(0.0f); //To update the representation with the new position
					pTree->GrowIn(this);
	
					m_Trees.push_back(pTree);

				}
				//cleanup
				delete [] treeTypeAtPos;
			}
		}
	}

	//cleanup
	delete [] density;

}

void CForest::ClearForest(IGameObj *pObj)
{
	assert(pObj && "Error: pObj is NULL in CForest::ClearForest");
	CSpheref sphere = pObj->GetBoundingSphere();
	TreeList::iterator it;
	for(it = m_Trees.begin(); it != m_Trees.end(); ++it)
	{
		if((*it)->GetBoundingSphere().Intersects(sphere))
		{
			(*it)->Destroy();
		}
	}
	//Remove all objects that were destroyed:
	m_Trees.erase(std::remove_if(m_Trees.begin(), m_Trees.end(), SRemoveDeletedTree()), m_Trees.end());
}

void CForest::Destroy()
{
	m_pForestStateNode = NULL;
	m_Trees.clear();
	IGameObj::Destroy();
	
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/