/****************************************************************************\
*                                                                            *
* 10.06.2006                                                                 *
* CForest - Header File                                                      *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __CFOREST_H__
#define __CFOREST_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>
#include <Memory/mmgr.h>
#include <Objects/IGameObj.h>
#include <Memory/TSmartPointer.h>
#include "CTreeObj.h"

#include <Display/TQTNode.h>
#include <Display/IRepresentation.h>
#include <Display/GraphStates/CForestStateNode.h>

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/
class CGameCtrl;
class CTerrain;
class CTreeObj;

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class CForest : public IGameObj
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
		CForest(CGameCtrl *pGameCtrl) : IGameObj(pGameCtrl,EOT_UNKNOWN) {m_pTerrain = NULL;}

		virtual bool Create(const std::string & sLevel,CTerrain * pTerrain);
		virtual void PlantTrees(byte * const plantMap, dword mapSizeX, dword mapSizeZ, byte nmbChannels);
		void ClearForest(IGameObj *pObj);

		virtual TQTNode<IRepresentation> * const getContainer(){return m_pSpecialRoot;}
        
		virtual void Destroy();

		virtual ~CForest() {Destroy();}
        
    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        //Private copy constructor and assignment operator to prevent mistakes:
        CForest(const CForest &);
        CForest &operator =(const CForest &);

    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:
		virtual void SubUpdate(float dt){};
        
    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
		CTerrain * m_pTerrain;
		std::vector<std::string>  m_vTreeModelNames;
		
		typedef std::vector<TSmartPointer<CTreeObj> > TreeList;
		TreeList m_Trees;

		TQTNode<IRepresentation> * m_pSpecialRoot;
		TSmartPointer<CForestStateNode> m_pForestStateNode; 
};

#endif
