/****************************************************************************\
*                                                                            *
* 21.04.2006                                                                 *
* CTerrain - Source File                                                     *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CTerrain.h"
#include "CGameCtrl.h"
#include <CEngineFactory.h>

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
bool CTerrain::Create(const std::string &sLevel)
{
    CTerrainRep *pTerrainRep = new CTerrainRep;

    if(!pTerrainRep->Create(sLevel, &m_TerrainInfo, m_pGameCtrl->GetSceneGraph(), this))
    {
        LOG(TL_ERROR, "Error: m_pTerrainRep->Create failed in CTerrain::Create");
        return false;
    }

    m_pRepresentation = pTerrainRep;

	//Create the terrain actor
    CMatrix44 mat;
    mat.Identity();
    CEngineFactory::InstancePtr()->GetPhysicsMgr()->BeginActorCreation(GetTransformation(), ECG_TERRAIN, false);
    CEngineFactory::InstancePtr()->GetPhysicsMgr()->AddMesh(mat, m_TerrainInfo.GetMesh());
	//Create level borders to prevent the player from falling down
	//X-Borders
	mat.Translation(CVector3f((m_TerrainInfo.GetSizeX() + 10.0f) / 2.0f, 490.0f, -5.0f));
	CEngineFactory::InstancePtr()->GetPhysicsMgr()->AddCube(mat, CVector3f(m_TerrainInfo.GetSizeX() + 10.0f, 1000.0f, 10.0f));
	mat.Translation(CVector3f((m_TerrainInfo.GetSizeX() + 10.0f) / 2.0f, 490.0f, m_TerrainInfo.GetSizeZ() + 5.0f));
	CEngineFactory::InstancePtr()->GetPhysicsMgr()->AddCube(mat, CVector3f(m_TerrainInfo.GetSizeX() + 10.0f, 1000.0f, 10.0f));
	//Z-Borders
	mat.Translation(CVector3f(-5.0f, 490.0f, m_TerrainInfo.GetSizeZ() / 2.0f));
	CEngineFactory::InstancePtr()->GetPhysicsMgr()->AddCube(mat, CVector3f(10.0f, 1000.0f, m_TerrainInfo.GetSizeZ() + 10.0f));
	mat.Translation(CVector3f(m_TerrainInfo.GetSizeX() + 5.0f, 490.0f, m_TerrainInfo.GetSizeZ() / 2.0f));
	CEngineFactory::InstancePtr()->GetPhysicsMgr()->AddCube(mat, CVector3f(10.0f, 1000.0f, m_TerrainInfo.GetSizeZ() + 10.0f));

    SetActor(CEngineFactory::InstancePtr()->GetPhysicsMgr()->FinishActorCreation());

    return true;
}

bool CTerrain::CreateForest(const std::string &sLevel){

	if(!m_pForest){
		m_pForest= new CForest(m_pGameCtrl);
		if(!m_pForest->Create(sLevel,this)){
			LOG(TL_ERROR,"Could not create Forest in CTerrain::CreateForest()!");
			return false;
		}
	}

	return true;
}

void CTerrain::ClearForest(IGameObj *pObj)
{
	if(m_pForest)
	{
		m_pForest->ClearForest(pObj);
	}
}

void CTerrain::Destroy()
{
	m_pForest = NULL;
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/


