/****************************************************************************\
*                                                                            *
* 15.04.2006                                                                 *
* CTerrain - Header File                                                     *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __CTERRAIN_H__
#define __CTERRAIN_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>
#include <Memory/mmgr.h>
#include "Objects/ICollidableObj.h"
#include "CTerrainRep.h"
#include "Loader/CTerrainInfo.h"
#include "CForest.h"

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/
class CGameCtrl;
class CForest;

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class CTerrain : public ICollidableObj
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        CTerrain(CGameCtrl *pGameCtrl) : ICollidableObj(pGameCtrl, EOT_TERRAIN), m_pForest(NULL) {}
        
        bool Create(const std::string &sLevel); //sLevle = Name of the settings section that contains the necessary information
        
		bool CreateForest(const std::string & sLevel);
		
		inline float HeightAt(float fX, float fZ, bool bVerbose = false) const {return m_TerrainInfo.HeightAt(fX, fZ, bVerbose);}
		void ClearForest(IGameObj *pObj);

		inline CTerrainInfo * const getTerrainInfo(){return &m_TerrainInfo;}
		
		virtual void CollidedWith(IGameObj *pOther) {}

        virtual void Destroy();

        virtual ~CTerrain() {Destroy();}
        
    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        virtual void SubUpdate(float dt) {}
        //Private copy constructor and assignment operator to prevent mistakes:
        CTerrain(const CTerrain &);
        const CTerrain &operator =(const CTerrain &);
        
    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
        CTerrainInfo           m_TerrainInfo;
		TSmartPointer<CForest> m_pForest;

};

#endif
