/****************************************************************************\
*                                                                            *
* 18.04.2006                                                                 *
* CTerrainRep - Source File                                                  *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CTerrainRep.h"
#include "CTerrainPartRep.h"
#include "CEngineFactory.h"
#include "CTerrain.h"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
bool CTerrainRep::Create(const std::string &sLevel, CTerrainInfo *pTerrainInfo, ISceneGraph *pSceneGraph,
                         CTerrain *pTerrain)
{
    IRepresentation::Create(pSceneGraph, CSpheref(), CAABox3f());

    m_pTerrain = pTerrain;

    std::vector< TSmartPointer<IModelGeometry> > modelGeometries;
    if(!CEngineFactory::InstancePtr()->GetModelFactory()->CreateTerrain(sLevel, pTerrainInfo, modelGeometries))
    {
        LOG(TL_ERROR, "Error: CreateTerrain(%s) failed in CTerrainRep::Create", sLevel.c_str());
        return false;
    }

	m_dwNumParts = (dword) modelGeometries.size();
    m_pTerrainParts = new TSmartPointer<CTerrainPartRep> [modelGeometries.size()];

    std::vector< TSmartPointer<IModelGeometry> >::iterator it;
    dword i = 0;
    for(it = modelGeometries.begin(); it != modelGeometries.end(); ++it, ++i)
    {
        m_pTerrainParts[i] = new CTerrainPartRep;
        if(!m_pTerrainParts[i]->Create(this, (*it), pSceneGraph, pTerrain))
        {
            LOG(TL_ERROR, "Error: CTerrainPartRep::Create failed for part %lu!", i);
            return false;
        }
    }

    return true;
}

const CVector3f &CTerrainRep::GetPos() const
{
    return m_pTerrain->GetPos();
}

const CVector3f &CTerrainRep::GetRot() const
{
    return m_pTerrain->GetRot();
}

const CVector3f &CTerrainRep::GetScale() const
{
    return m_pTerrain->GetScale();
}

const CMatrix44 &CTerrainRep::GetTransformation()
{
    return m_pTerrain->GetTransformation();
}

IGameObj * const CTerrainRep::GetGameObj() const
{
    return m_pTerrain;
}

bool CTerrainRep::RayIntersection(const CVector3f &vOrigin, const CVector3f &vDir, CVector3f &vIntersectionPos, float &fIntersectionDistance) const
{
	float fMinDistance = cg_fMaxIntersectionDist;
	float fDistance;	
	CVector3f tmpIntPos;
	bool bHit = false;

	for(dword i = 0; i < m_dwNumParts; ++i)
	{
		if(m_pTerrainParts[i]->RayIntersection(vOrigin, vDir, tmpIntPos, fDistance))
		{
			bHit = true;
			if(fDistance < fMinDistance)
			{
				fMinDistance = fDistance;
				vIntersectionPos = tmpIntPos;
				fIntersectionDistance = fDistance;
			}
		}
	}
    return bHit;
}

void CTerrainRep::Destroy()
{
    SAFE_DELETE_ARRAY(m_pTerrainParts);
    Clear();
    IRepresentation::Destroy();
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
void CTerrainRep::Clear()
{
	m_dwNumParts = 0;
    m_pTerrain = NULL;
    m_pTerrainParts = NULL;
}

void CTerrainRep::RenderSub(const CFrustum &frustum, bool bVerify)
{
    LOG(TL_ERROR, "Error: CTerrainRep::Render should never be called!");
}