/****************************************************************************\
*                                                                            *
* 07.05.2006                                                                 *
* CTreeObj - Source File                                                     *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CTreeObj.h"
#include "CTreeRep.h"
#include <CEngineFactory.h>


/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
CTreeObj::CTreeObj(CGameCtrl *pGameCtrl, const std::string &sModel)
	: ICollidableObj(pGameCtrl, EOT_VEGETATION)
{
	CTreeRep *pRep = new CTreeRep();
	if(!pRep->Create(pGameCtrl->GetSceneGraph(), sModel, this))
	{
		LOG(TL_ERROR, "Error: pRep->Create failed in CTreeObj::CTreeObj");
	}

	m_pRepresentation = (IRepresentation *) pRep;
	m_pRepresentation->SetPickable(false);
}

void CTreeObj::Plant()
{
	//Create the actor
	CEngineFactory::InstancePtr()->GetPhysicsMgr()->BeginActorCreation(GetTransformation(), ECG_VEGETATION, false);
	CAABox3f box(m_pRepresentation->GetOriginalBoundingBox());
	CMatrix44 mat;
	mat.Translation(box.Center());
	CEngineFactory::InstancePtr()->GetPhysicsMgr()->AddCube(mat, box.Size());
	SetActor(CEngineFactory::InstancePtr()->GetPhysicsMgr()->FinishActorCreation());
}

void CTreeObj::CollidedWith(IGameObj *pOther)
{
	//ToDo: Could burn down the trees :-)) FEATUREMANIA ON
}

bool CTreeObj::GrowIn(CForest *pForest){
	if(m_pRepresentation){
		pForest->getContainer()->AddObject(m_pRepresentation);
		return true;
	}else{
		return false;
	}
}

CTreeObj::~CTreeObj()
{

}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
void CTreeObj::SubUpdate(float dt)
{

}