/****************************************************************************\
*                                                                            *
* 07.05.2006                                                                 *
* CTreeRep - Source File                                                     *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CTreeRep.h"
#include <Display/ISceneGraph.h>
#include <Renderer/IModelGeometry.h>
#include <Renderer/IRenderer.h>
#include <CEngineFactory.h>
#include "CTreeObj.h"
#include <Renderer/COGLGlobalRenderSettings.h>

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
CTreeRep::CTreeRep() : m_pTreeObj(NULL), m_pModelGeometry(NULL)
{
	
}

bool CTreeRep::Create(ISceneGraph *pSceneGraph, const std::string &sModel, CTreeObj *pTreeObj)
{
	m_pTreeObj = pTreeObj;

	m_pModelGeometry = CEngineFactory::InstancePtr()->GetModelFactory()->getModel(sModel);
	if(!m_pModelGeometry)
	{
		LOG(TL_ERROR, "Error: getModel failed in CTreeRep::Create");
		return false;
	}

	IRepresentation::Create(pSceneGraph, m_pModelGeometry->GetBoundingSphere(), m_pModelGeometry->GetAABoundingBox());

	return true;
}

const CVector3f &CTreeRep::GetPos() const
{
	return m_pTreeObj->GetPos();
}

const CVector3f &CTreeRep::GetRot() const
{
	return m_pTreeObj->GetRot();
}

const CVector3f &CTreeRep::GetScale() const
{
	return m_pTreeObj->GetScale();
}

const CMatrix44 &CTreeRep::GetTransformation()
{
	return m_pTreeObj->GetTransformation();
}

IGameObj * const CTreeRep::GetGameObj() const
{
	return m_pTreeObj;
}

bool CTreeRep::RayIntersection(const CVector3f &vOrigin, const CVector3f &vDir, CVector3f &vIntersectionPos, float &fIntersectionDistance) const
{
	return m_pModelGeometry->RayIntersection(GetPos(), GetRot(), vOrigin, vDir, vIntersectionPos, fIntersectionDistance);
}

CTreeRep::~CTreeRep()
{
	m_pTreeObj = NULL;
	m_pModelGeometry = NULL;
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
void CTreeRep::RenderSub(const CFrustum &frustum, bool bVerify)
{
	m_pModelGeometry->Render();
}