/****************************************************************************\
*                                                                            *
* 23.04.2006                                                                 *
* CBattleState - Source File                                                 *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CBattleState.h"
#include "CGameCtrl.h"
#include "CEngineFactory.h"

#include <Memory/TSmartPointer.h>
#include <Renderer/IModelGeometry.h>
#include "Objects/CHumanPlayerCtrl.h"
#include "Objects/Birds/CAiBirdCtrl.h"
#include "Objects/Birds/CAiKamikazeCtrl.h"
#include "CSettings.h"
#include "CPostGameState.h"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/
const dword cg_dwSpawnRadius = 10;
const float cg_fBirdScore = 10.0f;

struct SRemoveSpawnPoint
{
    bool operator()(const SSpawnPoint &sp)
    {
        return sp.bRemove;
    }
};

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
CBattleState::CBattleState()
{
    m_pPlayerCtrl       = NULL;
}

bool CBattleState::Create(CGameCtrl *pGameCtrl)
{
    if(!IGameState::Create(pGameCtrl))
        return false;

    m_pTimer = new CTimer(pGameCtrl->GetClock());
    //Get pointer to gui mgr
    m_pGuiMgr = CEngineFactory::InstancePtr()->GetGuiMgr();
    //Create own gui root object
    m_pGuiRoot = m_pGuiMgr->CreateContainer();
	TSmartPointer<IGuiObject> pWindow;
	//Create the crosshair
	pWindow = m_pGuiMgr->CreateWnd("Data\\windows.tga", CVector2f(0.0f, 0.5f), CVector2f(0.25f, 0.75f), CVector4f(1.0f, 1.0f, 1.0f, 1.0f), EBM_ALPHA);
	pWindow->SetPos(CVector3f(470.0f, 477.5f, 1.0f));
	pWindow->SetScale(CVector3f(60.0f, 45.0f, 1.0f));
	m_pGuiRoot->AddChild(pWindow);
    //Create the bottom gui window
    pWindow = m_pGuiMgr->CreateWnd("Data\\windows.tga", CVector2f(0.0f, 0.25f), CVector2f(1.0f, 0.5f), CVector4f(1.0f, 1.0f, 1.0f, 1.0f), EBM_ALPHA);
    pWindow->SetPos(CVector3f(0.0f, 850.0f, 1.0f));
    pWindow->SetScale(CVector3f(1000.0f, 150.0f, 1.0f));
    m_pGuiRoot->AddChild(pWindow);
	m_pLevelStatusText = m_pGuiMgr->CreateText();
	m_pLevelStatusText->SetColor(0xFF000000);
	m_pLevelStatusText->SetPos(CVector3f(700.0f, 20.0f, 1.0f));
	pWindow->AddChild((IGuiObject *)((CGuiText *)m_pLevelStatusText));
	m_pHqStatusText = m_pGuiMgr->CreateText();
	m_pHqStatusText->SetColor(0xFF000000);
	m_pHqStatusText->SetPos(CVector3f(700.0f, 40.0f, 1.0f));
	pWindow->AddChild((IGuiObject *)((CGuiText *)m_pHqStatusText));
	m_pScoreText = m_pGuiMgr->CreateText();
	m_pScoreText->SetColor(0xFF000000);
	m_pScoreText->SetPos(CVector3f(700.0f, 60.0f, 1.0f));
	pWindow->AddChild((IGuiObject *)((CGuiText *)m_pScoreText));

    TSmartPointer<CGuiText> pText;
    TSmartPointer<CGuiButton> pButton;
    //Create the "won" message box
    m_pWonWindow = m_pGuiMgr->CreateWnd("Data\\buttonTex.tga", CVector2f(0.0f, 0.0f), CVector2f(1.0f, 0.25f), CVector4f(1.0f, 1.0f, 1.0f, 1.0f));
    m_pWonWindow->Show(false);
    m_pWonWindow->SetPos(CVector3f(330.0f, 425.0f, 10.0f));
    m_pWonWindow->SetScale(CVector3f(340.0f, 150.0f, 1.0f));
    //Add info text
    pText = m_pGuiMgr->CreateText();
    pText->SetColor(0xFF000000);
    pText->SetPos(CVector3f(30.0f, 30.0f, 1.0f));
    pText->SetText("This attack is over. Well done!");
    m_pWonWindow->AddChild((IGuiObject *)((CGuiText *)pText));
    //Add button
    pButton = m_pGuiMgr->CreateButton();
    pButton->SetPos(CVector3f(95.0f, 80.0f, 1.0f));
    pButton->SetScale(CVector3f(150.0f, 50.0f, 1.0f));
    pButton->SetText("Next Level", 0xD0D0D000);
    pButton->SetOnClickHandler(new TParamFunctor<CBattleState>(this, OnWonButton));
    m_pWonWindow->AddChild((IGuiObject *)((CGuiButton *)pButton));
    m_pGuiRoot->AddChild(m_pWonWindow);

    //Create the "lost" message box
    m_pLostWindow = m_pGuiMgr->CreateWnd("Data\\buttonTex.tga", CVector2f(0.0f, 0.0f), CVector2f(1.0f, 0.25f), CVector4f(1.0f, 1.0f, 1.0f, 1.0f));
    m_pLostWindow->Show(false);
    m_pLostWindow->SetPos(CVector3f(330.0f, 425.0f, 10.0f));
    m_pLostWindow->SetScale(CVector3f(340.0f, 150.0f, 1.0f));
    //Add info text
    pText = m_pGuiMgr->CreateText();
    pText->SetColor(0xFF000000);
    pText->SetPos(CVector3f(30.0f, 30.0f, 1.0f));
    pText->SetText("Your town is devastated!");
    m_pLostWindow->AddChild((IGuiObject *)((CGuiText *)pText));
    //Add button
    pButton = m_pGuiMgr->CreateButton();
    pButton->SetPos(CVector3f(95.0f, 80.0f, 1.0f));
    pButton->SetScale(CVector3f(150.0f, 50.0f, 1.0f));
    pButton->SetText("Oh no!", 0xD0D0D000);
    pButton->SetOnClickHandler(new TParamFunctor<CBattleState>(this, OnLostButton));
    m_pLostWindow->AddChild((IGuiObject *)((CGuiButton *)pButton));
    m_pGuiRoot->AddChild(m_pLostWindow);

    //Create Player and PlayerCtrl
    m_pPlayer = new CPlayerObj(pGameCtrl);
    m_pPlayerCtrl = new CHumanPlayerCtrl(500);
    //Create Object camera
    m_pCamCtrl = new CObjCamCtrl;

	//Load background sound
	m_pBgSound = CEngineFactory::InstancePtr()->GetSoundMgr()->LoadSound(eSTREAM, "Data/sounds/bg_battle.mp3");
	m_pBgSound->SetLoopCount(20);
	m_pBgSound->SetVolume(0.2f);

    return true;
}

void CBattleState::Enter()
{
	m_pBgSound->Play();

    std::string sValue;
    dword dwSpawnPoints;
    //Get required parameters
    //Number of spawn points
    if(!CSettings::InstancePtr()->GetValue(m_pGameCtrl->GetLevelName(), "SpawnPoints", sValue))
    {
        LOG(TL_ERROR, "Error: Couldn't get \"SpawnPoints\"-Value from \"%s\" in CBattleState::Enter", m_pGameCtrl->GetLevelName().c_str());
        dwSpawnPoints = 0;
    }
    else
    {
        dwSpawnPoints = (word) strtoul(sValue.c_str(), NULL, 10);
    }
    //Get each spawnpoint
    char sText[32];
    m_dwNumBirds = 0;
	char cSpawnType;
    SSpawnPoint sp;
    sp.bRemove = false;
    for(dword i = 1; i <= dwSpawnPoints; ++i)
    {
        sprintf(sText, "SpawnPoint%lu", i);
        if(!CSettings::InstancePtr()->GetValue(m_pGameCtrl->GetLevelName(), sText, sValue))
        {
            LOG(TL_ERROR, "Error: Couldn't get \"%s\"-Value from \"%s\" in CBattleState::Enter", sText, m_pGameCtrl->GetLevelName().c_str());
        }
        else
        {
            if(sscanf(sValue.c_str(), "%f;%lu;%c;%f;%f;%f", &sp.fSpawnTime, &sp.dwNumBirds, &cSpawnType, &sp.vPos.m_X, &sp.vPos.m_Y, &sp.vPos.m_Z) != 6)
            {
                LOG(TL_ERROR, "Error: Couldn't parse \"%s\"-Value from \"%s\" in CBattleState::Enter: %s", sText, m_pGameCtrl->GetLevelName().c_str(), sValue.c_str());
            }
            else
            {
				if(cSpawnType == 'K')
				{
					sp.spawnType = EST_KAMIKAZE;
				}
				else
				{
					sp.spawnType = EST_BOMBING;
				}
                m_dwNumBirds += sp.dwNumBirds;
                m_SpawnPoints.push_back(sp);
            }
        }
    }

	m_dwMaxBirds = m_dwNumBirds;

    m_pTimer->Reset();
    m_pTimer->Enable(true);
    m_pGuiMgr->SetRootObject(m_pGuiRoot);
	CVector3f vStartPos(m_pGameCtrl->GetPlayerStartPos());
	vStartPos.m_Y = m_pGameCtrl->GetTerrain()->HeightAt(vStartPos.m_X, vStartPos.m_Z) + 3.0f;
    m_pPlayer->SetPos(vStartPos);
    m_pPlayer->SetPlayerCtrl(m_pPlayerCtrl);
	//Clean up trees around the player position
	m_pGameCtrl->GetTerrain()->ClearForest((IGameObj *)((CPlayerObj *)m_pPlayer));
    m_pCamCtrl->SetGameObj((IGameObj *)((CPlayerObj *)m_pPlayer));
    m_pCamCtrl->SetOffset(CVector3f(0.0f, 0.2f, 0.0f), false, false); //1st person cam
//    m_pCamCtrl->SetOffset(CVector3f(0.0f, 1.8f, 10.0f), true, true); //3rd person cam
    m_pGameCtrl->SetCamCtrl((ICamCtrl *)((CObjCamCtrl *)m_pCamCtrl));
	//Update status text
	UpdateLevelStatus(0);
	UpdateHqStatus(100);
	ScoreChanged();

    //Hide mouse cursor
    CEngineFactory::InstancePtr()->GetInputMgr()->ShowCursor(false);
	//Register
	CEngineFactory::InstancePtr()->GetInputMgr()->RegisterHandler(this);
}

bool CBattleState::Update()
{
    float dt = (float) m_pTimer->GetFrameDuration();
	m_pPlayer->Update(dt);

    //Check, if we should enable some spawn points
    //ToDo: Do this only every few seconds or so
    std::vector<SSpawnPoint>::iterator it;
    for(it = m_SpawnPoints.begin(); it != m_SpawnPoints.end(); ++it)
    {
        CBirdObj *pBirdObj;
        float rand1, rand2;
        if((*it).fSpawnTime <= m_pTimer->GetTime())
        {
            //Ok, this one is active. Create it's birds
            for(dword i = 0; i < (*it).dwNumBirds; ++i)
            {
                rand1 = (float) (rand() % (20 * cg_dwSpawnRadius));
                rand2 = (float) (rand() % (20 * cg_dwSpawnRadius));
                rand1 = (float)(rand1 - (10 * cg_dwSpawnRadius)) / 10.0f;
                rand2 = (float)(rand2 - (10 * cg_dwSpawnRadius)) / 10.0f;
                pBirdObj = new CBirdObj(m_pGameCtrl, GetBirdDestroyCb());
                pBirdObj->SetPos((*it).vPos + CVector3f(rand1, 0.0f, rand2));
				switch((*it).spawnType)
				{
					case EST_BOMBING:
						pBirdObj->SetBirdCtrl(new CAiBirdCtrl(m_pGameCtrl));
						break;
					case EST_KAMIKAZE:
						pBirdObj->SetBirdCtrl(new CAiKamikazeCtrl(m_pGameCtrl));
						break;
					default:
						LOG(TL_ERROR, "Error: Unknown spawn type in CBattleState::Update");
				}
                m_pGameCtrl->AddObject((IGameObj *) pBirdObj);
            }
            //Mark as "to be removed"
            (*it).bRemove = true;
        }
    }
    //Remove all finished spawn points
    m_SpawnPoints.erase(std::remove_if(m_SpawnPoints.begin(), m_SpawnPoints.end(), SRemoveSpawnPoint()), m_SpawnPoints.end());

    m_pGameCtrl->UpdateGameObjects(dt);

    //Update the physics manager
    CEngineFactory::InstancePtr()->GetPhysicsMgr()->Update(dt);

	return true;
}

void CBattleState::Leave()
{
    //Show mouse cursor
    CEngineFactory::InstancePtr()->GetInputMgr()->ShowCursor(true);
    m_pGameCtrl->SetCamCtrl(NULL);
    m_pPlayer->SetPlayerCtrl(NULL);
    m_pGuiMgr->SetRootObject(NULL);
    m_pTimer->Enable(false);

	m_pBgSound->Stop();
}

void CBattleState::ScoreChanged()
{
	char sText[64];
	sprintf(sText, "Score:                  %.0f", m_pGameCtrl->GetScore());
	m_pScoreText->SetText(sText);
}

const TSmartPointer<IParamFunctor> CBattleState::GetHqHitCb()
{
	return new TParamFunctor<CBattleState>(this, OnHqHit);
}

const TSmartPointer<IFunctor> CBattleState::GetHqDestroyCb()
{
	return new TFunctor<CBattleState>(this, OnHqDestroyed);
}

const TSmartPointer<IFunctor> CBattleState::GetBirdDestroyCb()
{
    return new TFunctor<CBattleState>(this, OnBirdDestroyed);
}

dword CBattleState::OnHqHit(dword dwStatus)
{
	UpdateHqStatus(dwStatus);
	return 0;
}

dword CBattleState::OnHqDestroyed()
{
    //Show mouse cursor
    CEngineFactory::InstancePtr()->GetInputMgr()->ShowCursor(true);
    //Show "You lost" message box and make it modal
	m_pPlayer->SetPlayerCtrl(NULL);
    m_pLostWindow->Show(true);
    m_pGuiMgr->GrabInput(m_pLostWindow);

    m_pTimer->Enable(false);
	return 0;
}

bool CBattleState::OnKeyDown(dword dwKey){
	return false;
}

dword CBattleState::OnBirdDestroyed()
{
    --m_dwNumBirds;
	UpdateLevelStatus((dword)(100.0f - (100.0f * m_dwNumBirds) / m_dwMaxBirds));
	m_pGameCtrl->AddScore(cg_fBirdScore);
    if(m_dwNumBirds == 0)
    {
        //Show mouse cursor
        CEngineFactory::InstancePtr()->GetInputMgr()->ShowCursor(true);
        //Show "You won" message box and make it modal
		m_pPlayer->SetPlayerCtrl(NULL);
        m_pWonWindow->Show(true);
        m_pGuiMgr->GrabInput(m_pWonWindow);

        m_pTimer->Enable(false);
    }
    return 0;
}

dword CBattleState::OnWonButton(dword dwButtonId)
{
    m_pGuiMgr->ReleaseInput(m_pWonWindow);
    m_pWonWindow->Show(false);
    m_pGameCtrl->SetCurrentState(m_pGameCtrl->GetPostLevelState());
    return 0;
}

dword CBattleState::OnLostButton(dword dwButtonId)
{
    m_pGuiMgr->ReleaseInput(m_pLostWindow);
    m_pLostWindow->Show(false);
	m_pGameCtrl->GetPostGameState()->GameWon(false);
	m_pGameCtrl->SetCurrentState(m_pGameCtrl->GetPostGameState());
    return 0;
}

CBattleState::~CBattleState()
{
    SAFE_DELETE(m_pPlayerCtrl);
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
void CBattleState::UpdateLevelStatus(dword dwValue)
{
	assert((bool) m_pHqStatusText && "Error: m_pLevelStatusText is NULL in CBattleState::UpdateLevelStatus");
	char sText[32];
	sprintf(sText, "Level done:      %3lu%%", dwValue);
	m_pLevelStatusText->SetText(sText);
}

void CBattleState::UpdateHqStatus(dword dwValue)
{
	assert((bool) m_pHqStatusText && "Error: m_pHqStatusText is NULL in CBattleState::UpdateHqStatus");
	char sText[32];
	sprintf(sText, "Statue:    % 3lu%%", dwValue);
	m_pHqStatusText->SetText(sText);
}
