/****************************************************************************\
*                                                                            *
* 23.04.2006                                                                 *
* CBattleState - Header File                                                 *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __CBATTLESTATE_H__
#define __CBATTLESTATE_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <Memory/mmgr.h>
#include <Memory/TSmartPointer.h>
#include "Objects/CObjCamCtrl.h"
#include "IGameState.h"
#include "Objects/CPlayerObj.h"
#include "Objects/Birds/CBirdObj.h"
#include <Misc/Functors.h>
#include <Input/IInputHandler.h>
#include <Display/Gui/CGuiText.h>

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/
class CGameCtrl;
class CTimer;
class IGuiObject;
class CGuiMgr;
class IBirdCtrl;

enum ESpawnType
{
	EST_BOMBING,
	EST_KAMIKAZE,
};

struct SSpawnPoint
{
    float fSpawnTime;
    dword dwNumBirds;
	ESpawnType spawnType;
    CVector3f vPos;
    bool bRemove;
};

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class CBattleState : public IGameState , private IInputHandler
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
		CBattleState();
        
        virtual bool Create(CGameCtrl *pGameCtrl);
        virtual void Enter();
        virtual bool Update();
        virtual void Leave();

		void ScoreChanged();

		const TSmartPointer<IParamFunctor> GetHqHitCb();
		const TSmartPointer<IFunctor> GetHqDestroyCb();
        const TSmartPointer<IFunctor> GetBirdDestroyCb();

		dword OnHqHit(dword dwStatus);
		dword OnHqDestroyed();

		virtual bool OnKeyDown(dword dwKey);
        dword OnBirdDestroyed();
        dword OnWonButton(dword dwButtonId);
        dword OnLostButton(dword dwButtonId);
        
        virtual ~CBattleState();
        
    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
		void UpdateLevelStatus(dword dwValue);
		void UpdateHqStatus(dword dwValue);
        //Private copy constructor and assignment operator to prevent mistakes:
        CBattleState(const CBattleState &);
        const CBattleState &operator =(const CBattleState &);
        
    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
        TSmartPointer<CTimer> m_pTimer;
        TSmartPointer<CObjCamCtrl> m_pCamCtrl;
        TSmartPointer<CPlayerObj> m_pPlayer;
        IPlayerCtrl *m_pPlayerCtrl;

        //Spawn points
        std::vector<SSpawnPoint> m_SpawnPoints;
		dword m_dwMaxBirds;
        dword m_dwNumBirds;

        TSmartPointer<IGuiObject> m_pGuiRoot;
        TSmartPointer<IGuiObject> m_pWonWindow;
        TSmartPointer<IGuiObject> m_pLostWindow;

		TSmartPointer<CGuiText> m_pLevelStatusText;
		TSmartPointer<CGuiText> m_pHqStatusText;
		TSmartPointer<CGuiText> m_pScoreText;

        //For quicker access
        TSmartPointer<CGuiMgr> m_pGuiMgr;

		TSmartPointer<ISound> m_pBgSound;
};

#endif
