/****************************************************************************\
*                                                                            *
* 12.04.2006                                                                 *
* CMainMenuState - Source File                                               *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CMainMenuState.h"
#include "CGameCtrl.h"
#include <CEngineFactory.h>

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
bool CMainMenuState::Create(CGameCtrl *pGameCtrl)
{
    if(!IGameState::Create(pGameCtrl))
        return false;

    //Get reference to GuiMgr
    m_pGuiMgr = CEngineFactory::InstancePtr()->GetGuiMgr();

    //Create window for main menu
    m_pGuiMenu = m_pGuiMgr->CreateWnd("Data/mainbg.tga");
    m_pGuiMenu->SetPos(CVector3f(0.0f, 0.0f, 1.0f));
	m_pGuiMenu->SetScale(CVector3f(1000.0f, 1000.0f, 1.0f));

    TSmartPointer<CGuiButton> pButton;
    pButton = m_pGuiMgr->CreateButton();
    pButton->SetPos(CVector3f(435.0f, 600.0f, 10.0f));
    pButton->SetScale(CVector3f(130.0f, 50.0f, 1.0f));
    pButton->SetText("New game", 0xD0D0D000);
    pButton->SetOnClickHandler(new TParamFunctor<CMainMenuState>(this, OnNewGame));
	m_pGuiMenu->AddChild((IGuiObject *)((CGuiButton *)pButton));

    pButton = m_pGuiMgr->CreateButton();
    pButton->SetPos(CVector3f(435.0f, 700.0f, 10.0f));
    pButton->SetScale(CVector3f(130.0f, 50.0f, 1.0f));
    pButton->SetText("Quit game", 0xD0D0D000);
    pButton->SetButtonId(1);
    pButton->SetOnClickHandler(new TParamFunctor<CMainMenuState>(this, OnQuit));

    m_pGuiMenu->AddChild((IGuiObject *)((CGuiButton *)pButton));

	//Sound
	m_pBgSound = CEngineFactory::InstancePtr()->GetSoundMgr()->LoadSound(eSTREAM, "Data/sounds/bg_battle.mp3");
	m_pBgSound->SetLoopCount(20);
	m_pBgSound->SetVolume(0.6f);

    m_bRun = true;

    return true;
}

void CMainMenuState::Enter()
{
//	m_pBgSound->Play();
    m_pGuiMgr->SetRootObject(m_pGuiMenu);
	
    m_bRun = true;
}

bool CMainMenuState::Update()
{
    return m_bRun;
}

void CMainMenuState::Leave()
{
//	m_pBgSound->Stop();
	m_pGuiMgr->SetRootObject(NULL);
}

void CMainMenuState::Destroy()
{
    Clear();
}

dword CMainMenuState::OnNewGame(dword dwButtonId)
{
    m_pGameCtrl->SetCurrentState(m_pGameCtrl->GetNewLevelState());
    return 0;
}

dword CMainMenuState::OnQuit(dword dwButtonId)
{
    m_bRun = false;
    return 0;
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
void CMainMenuState::Clear()
{
    m_pGuiMenu = NULL;
    m_pGuiMgr = NULL;
}
