/****************************************************************************\
*                                                                            *
* 21.04.2006                                                                 *
* CNewLevelState - Source File                                               *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CNewLevelState.h"
#include "CGameCtrl.h"
#include "Objects/Environment/CEnvironment.h"
#include <CEngineFactory.h>
#include <Display/CQTSceneGraph.h>
#include "Objects/Terrain/CTerrain.h"
#include <Display/IRepresentation.h>
#include <Objects/CStartObj.h>
#include <Objects/Buildings/CHqBuildingObj.h>
#include "CBattleState.h"
#include <Display/GraphStates/CAlphaBlendingStateNode.h>

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
bool CNewLevelState::Create(CGameCtrl *pGameCtrl)
{
	if(!IGameState::Create(pGameCtrl))
		return false;

	//Get reference to GuiMgr
	CGuiMgr *pGuiMgr = CEngineFactory::InstancePtr()->GetGuiMgr();

	TSmartPointer<CGuiButton> pButton = pGuiMgr->CreateButton();
	pButton->SetPos(CVector3f(435.0f, 800.0f, 1.0f));
	pButton->SetScale(CVector3f(130.0f, 50.0f, 1.0f));
	pButton->SetOnClickHandler(new TParamFunctor<CNewLevelState>(this, OnContinue));
	m_pContinue = pButton;

	return true;
}

void CNewLevelState::Enter()
{
	word wLevel = m_pGameCtrl->GetLevel();
	//Increase level counter and create level name
	++wLevel;
	m_pGameCtrl->SetLevel(wLevel);
	char sLevelName[16];
	sprintf(sLevelName, "Level%u", wLevel);
	m_pGameCtrl->SetLevelName(sLevelName);

	//Get reference to GuiMgr
	CGuiMgr *pGuiMgr = CEngineFactory::InstancePtr()->GetGuiMgr();

	//Create window for main menu
	std::string str("Data/loadingscreens/");
	str.append(m_pGameCtrl->GetLevelName());
	str.append(".tga");
	m_pLoadingScreen = pGuiMgr->CreateWnd(str);
	m_pLoadingScreen->SetPos(CVector3f(0.0f, 0.0f, 1.0f));
	m_pLoadingScreen->SetScale(CVector3f(1000.0f, 1000.0f, 1.0f));
	m_pLoadingScreen->AddChild((IGuiObject *)((CGuiButton *)m_pContinue));
	pGuiMgr->SetRootObject(m_pLoadingScreen);

	m_pContinue->SetText("Loading...", 0xFF000000);
	m_pContinue->Enable(false);
	m_bSkipUpdate = true;
}

bool CNewLevelState::Update()
{
	if(m_bSkipUpdate)
	{
		m_bSkipUpdate = false;
		return true;
	}
	if(!m_pGameCtrl->GetSceneGraph()) //Ok, this is a new game
	{
		//Create scene graph
		CQTSceneGraph *pSceneGraph = new CQTSceneGraph;
		if(!pSceneGraph->Create(1000, 1000, -10, 100, 5))
		{
			LOG(TL_ERROR, "Error: pSceneGraph->Create failed in CNewLevelState::Update");
			return false;
		}
		m_pGameCtrl->SetSceneGraph(pSceneGraph);

		//Terrain
		CTerrain *pTerrain = new CTerrain(m_pGameCtrl);
		if(!pTerrain->Create("GAME"))
		{
			LOG(TL_ERROR, "Error: pTerrain->Create failed in CNewLevelState::Update");
			return false;
		}
		//Set the terrain
		m_pGameCtrl->SetTerrain(pTerrain);

		//TerainProps
		TSmartPointer<CAlphaBlendingStateNode> pAlphaBlendStateNode;
		TQTNode<IRepresentation> *pSpecialAlphaBlendNode;

		pAlphaBlendStateNode = new CAlphaBlendingStateNode();
		pSpecialAlphaBlendNode = pSceneGraph->CreateSpecialRootNode((IStateNode*)pAlphaBlendStateNode);
		//	m_pDummy = (IRepresentation*) new CBuildingObj("Data/models/startpotest.fbx", m_pGameCtrl->GetPlayerStartPos());
		m_pGameCtrl->AddObject((IGameObj *)(new CStartObj(m_pGameCtrl, m_pGameCtrl->GetPlayerStartPos())));
		//m_pSpecialAlphaBlendNode->AddObject(m_pDummy);
		CBuildingObj *pBuilding = new CHqBuildingObj(m_pGameCtrl, ((CBattleState *)m_pGameCtrl->GetBattleState())->GetHqDestroyCb(),
			((CBattleState *)m_pGameCtrl->GetBattleState())->GetHqHitCb());
		CVector3f vPos(m_pGameCtrl->GetPlayerStartPos());
		vPos.m_X -= 20.0f;
		vPos.m_Y = pTerrain->HeightAt(vPos.m_X, vPos.m_Z);
		pBuilding->SetPos(vPos);
		pBuilding->UpdateState();
		pBuilding->SetPos(vPos);
		pBuilding->Build();
		m_pGameCtrl->AddObject(pBuilding);
		//Forest
		if(!pTerrain->CreateForest("GAME")){
			LOG(TL_ERROR, "Error: pTerrain->PlantTrees failed in CNewLevelState::Update");
			return false;
		}
	}
	//Environment
	CEnvironment *pEnvironment = new CEnvironment(m_pGameCtrl);
		
	if(!pEnvironment->Create(m_pGameCtrl->GetLevelName()))
	{
		LOG(TL_ERROR, "Error: pEnvironment->Create failed in CNewLevelState::Update");
		return false;
	}
	m_pGameCtrl->SetEnvironment(pEnvironment);

	//Quick and dirty: Preload models
	CEngineFactory::InstancePtr()->GetModelFactory()->getModel("Data/Models/bird.fbx");
	CEngineFactory::InstancePtr()->GetModelFactory()->getModel("Data/Models/pfeil.fbx");
	CEngineFactory::InstancePtr()->GetModelFactory()->getModel("Data/Models/shit.fbx");
	CEngineFactory::InstancePtr()->GetModelFactory()->getModel("Data/Models/startpodest.fbx");
	CEngineFactory::InstancePtr()->GetModelFactory()->getModel("Data/Models/tree1.fbx");
	CEngineFactory::InstancePtr()->GetModelFactory()->getModel("Data/Models/tree2.fbx");
	CEngineFactory::InstancePtr()->GetModelFactory()->getModel("Data/Models/tree3.fbx");
	CEngineFactory::InstancePtr()->GetModelFactory()->getModel("Data/Models/buildings/haus1.fbx");
	CEngineFactory::InstancePtr()->GetModelFactory()->getModel("Data/Models/buildings/haus2.fbx");
	CEngineFactory::InstancePtr()->GetModelFactory()->getModel("Data/Models/buildings/kirche.fbx");
	CEngineFactory::InstancePtr()->GetModelFactory()->getModel("Data/Models/buildings/klo.fbx");
	CEngineFactory::InstancePtr()->GetModelFactory()->getModel("Data/Models/buildings/marktstand.fbx");
	CEngineFactory::InstancePtr()->GetModelFactory()->getModel("Data/Models/buildings/statue.fbx");
	CEngineFactory::InstancePtr()->GetModelFactory()->getModel("Data/Models/buildings/tower.fbx");
	CEngineFactory::InstancePtr()->GetModelFactory()->getModel("Data/models/buildings/destroyed.fbx");
	CEngineFactory::InstancePtr()->GetModelFactory()->getModel("Data/environment/envSphere.fbx");
	
	m_pContinue->SetText("Continue", 0xFF000000);
	m_pContinue->Enable(true);

    return true;
}

void CNewLevelState::Leave()
{
	CEngineFactory::InstancePtr()->GetGuiMgr()->SetRootObject(NULL);
}

void CNewLevelState::Destroy()
{
    Clear();
    IGameState::Destroy();
}

dword CNewLevelState::OnContinue(dword dwButtonId)
{
	//We are done, switch to build mode
	m_pGameCtrl->SetCurrentState(m_pGameCtrl->GetBuildState());

	return 0;
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
void CNewLevelState::Clear()
{
	m_bSkipUpdate = true;
	m_pLoadingScreen = NULL;
	m_pContinue = NULL;
}
