/****************************************************************************\
*                                                                            *
* 21.04.2006                                                                 *
* CPostGameState - Source File                                               *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CPostGameState.h"
#include "CGameCtrl.h"
#include "Objects/Environment/CEnvironment.h"
#include <CEngineFactory.h>

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
bool CPostGameState::Create(CGameCtrl *pGameCtrl)
{
	if(!IGameState::Create(pGameCtrl))
		return false;

	//Get reference to GuiMgr
	CGuiMgr *pGuiMgr = CEngineFactory::InstancePtr()->GetGuiMgr();

	m_pRootObject = pGuiMgr->CreateContainer();

	//Create window
	m_pBgScreenLost = pGuiMgr->CreateWnd("Data/endbg.tga");
	m_pBgScreenLost->SetPos(CVector3f(0.0f, 0.0f, 1.0f));
	m_pBgScreenLost->SetScale(CVector3f(1000.0f, 1000.0f, 1.0f));
	m_pRootObject->AddChild(m_pBgScreenLost);
	m_pBgScreenWon = pGuiMgr->CreateWnd("Data/wonbg.tga");
	m_pBgScreenWon->SetPos(CVector3f(0.0f, 0.0f, 1.0f));
	m_pBgScreenWon->SetScale(CVector3f(1000.0f, 1000.0f, 1.0f));
	m_pRootObject->AddChild(m_pBgScreenWon);

	TSmartPointer<CGuiText> pText = pGuiMgr->CreateText();
	pText->SetPos(CVector3f(440.0f, 500.0f, 3.0f));
	pText->SetColor(0x00000000);
	m_pScoreText = pText;
	TSmartPointer<CGuiButton> pButton = pGuiMgr->CreateButton();
	pButton->SetPos(CVector3f(435.0f, 800.0f, 3.0f));
	pButton->SetScale(CVector3f(130.0f, 50.0f, 1.0f));
	pButton->SetText("Continue", 0x00FF0000);
	pButton->SetOnClickHandler(new TParamFunctor<CPostGameState>(this, OnContinue));
	m_pContinue = pButton;

	m_pRootObject->AddChild((IGuiObject *)((CGuiText *)m_pScoreText));
	m_pRootObject->AddChild(m_pContinue);

	return true;
}

void CPostGameState::Enter()
{
	char sText[30];
	sprintf(sText, "Your score: %.0f", m_pGameCtrl->GetScore());
	m_pScoreText->SetText(sText);
	CEngineFactory::InstancePtr()->GetGuiMgr()->SetRootObject(m_pRootObject);
}

bool CPostGameState::Update()
{
    return true;
}

void CPostGameState::Leave()
{
	CEngineFactory::InstancePtr()->GetGuiMgr()->SetRootObject(NULL);
}

void CPostGameState::Destroy()
{
    Clear();
    IGameState::Destroy();
}

void CPostGameState::GameWon(bool bWon)
{
	m_pBgScreenLost->Show(!bWon);
	m_pBgScreenWon->Show(bWon);
}

dword CPostGameState::OnContinue(dword dwButtonId)
{
	m_pGameCtrl->ResetGame();
	m_pGameCtrl->SetCurrentState(m_pGameCtrl->GetMainMenuState());

	return 0;
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
void CPostGameState::Clear()
{
	m_pRootObject = NULL;
	m_pBgScreenWon = NULL;
	m_pBgScreenLost = NULL;
	m_pScoreText = NULL;
	m_pContinue = NULL;
}
